@@ -43,10 +43,23 @@
  *          PIX       *pixGetInvBackgroundMap()   16 bpp
  *
  *      Apply inverse background map to image
- *          PIX       *pixApplyInvBackgroundGrayMap()  8 bpp
- *          PIX       *pixApplyInvBackgroundRGBMap()   32 bpp
+ *          PIX       *pixApplyInvBackgroundGrayMap()   8 bpp
+ *          PIX       *pixApplyInvBackgroundRGBMap()    32 bpp
  *
- *  Normalization is done by generating a reduced map (or set
+ *      Apply variable map
+ *          PIX       *pixApplyVariableGrayMap()        8 bpp
+ *
+ *      Non-adaptive (global) mapping
+ *          PIX       *pixGlobalNormRGB()               32 bpp or cmapped
+ *          PIX       *pixGlobalNormNoSatRGB()          32 bpp
+ *
+ *      Adaptive threshold spread normalization
+ *          l_int32    pixThresholdSpreadNorm()         8 bpp
+ *
+ *      Adaptive Otsu-based thresholding
+ *          l_int32    pixOtsuAdaptiveThreshold()       8 bpp
+ *
+ *  Background normalization is done by generating a reduced map (or set
  *  of maps) representing the estimated background value of the
  *  input image, and using this to shift the pixel values so that
  *  this background value is set to some constant value.
@@ -71,6 +84,19 @@
  *        into pixels covered by an optional image mask.  Invert the
  *        background map without preconditioning by convolutional smoothing.
  *
+ *  Note: Most of these functions make an implicit assumption about RGB
+ *        component ordering.
+ *
+ *  Other methods for adaptively normalizing the image are also given here.
+ *
+ *  (1) pixThresholdSpreadNorm() computes a local threshold over the image
+ *      and normalizes the input pixel values so that this computed threshold
+ *      is a constant across the entire image.
+ *
+ *  (2) pixOtsuAdaptiveThreshold() computes a global threshold over each
+ *      tile and performs the threshold operation, resulting in a
+ *      binary image for each tile.  These are stitched into the final result.
+ *      It does not generate an 8 bpp threshold-normalized image.
  */
 
 #include <stdio.h>
@@ -94,6 +120,11 @@ static const l_int32  DEFAULT_BG_VAL = 200;
 static const l_int32  DEFAULT_X_SMOOTH_SIZE = 2;
 static const l_int32  DEFAULT_Y_SMOOTH_SIZE = 1;
 
+#ifndef  NO_CONSOLE_IO
+#define  DEBUG_GLOBAL    0
+#endif  /* ~NO_CONSOLE_IO */
+
+
 
 /*------------------------------------------------------------------*
  *                Adaptive background normalization                 *
@@ -224,8 +255,10 @@ PIX     *pixmr, *pixmg, *pixmb, *pixmri, *pixmgi, *pixmbi;
     if (d == 8) {
         pixm = NULL;
         pixGetBackgroundGrayMap(pixs, pixim, sx, sy, thresh, mincount, &pixm);
-        if (!pixm)
-            return (PIX *)ERROR_PTR("pixm not made", procName, NULL);
+        if (!pixm) {
+            L_WARNING("map not made; returning a copy of the source", procName);
+            return pixCopy(NULL, pixs);
+        }
 
         pixmi = pixGetInvBackgroundMap(pixm, bgval, smoothx, smoothy);
         if (!pixmi)
@@ -244,7 +277,8 @@ PIX     *pixmr, *pixmg, *pixmb, *pixmri, *pixmgi, *pixmbi;
             pixDestroy(&pixmr);
             pixDestroy(&pixmg);
             pixDestroy(&pixmb);
-            return (PIX *)ERROR_PTR("not all pixm*", procName, NULL);
+            L_WARNING("map not made; returning a copy of the source", procName);
+            return pixCopy(NULL, pixs);
         }
 
         pixmri = pixGetInvBackgroundMap(pixmr, bgval, smoothx, smoothy);
@@ -511,7 +545,7 @@ PIX     *pixmr, *pixmg, *pixmb;
 
     if (!ppixr || !ppixg || !ppixb)
         return ERROR_INT("&pixr, &pixg, &pixb not all defined", procName, 1);
-    *ppixr = *ppixg = *ppixb;
+    *ppixr = *ppixg = *ppixb = NULL;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (pixGetDepth(pixs) != 32)
@@ -649,7 +683,7 @@ PIX     *pixmr, *pixmg, *pixmb;
 
     if (!ppixr || !ppixg || !ppixb)
         return ERROR_INT("&pixr, &pixg, &pixb not all defined", procName, 1);
-    *ppixr = *ppixg = *ppixb;
+    *ppixr = *ppixg = *ppixb = NULL;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     if (pixGetDepth(pixs) != 32)
@@ -841,7 +875,8 @@ PIX       *pixd, *piximi, *pixb, *pixf, *pixims;
         /* Fill all the holes in the map. */
     if (pixFillMapHoles(pixd, nx, ny, L_FILL_BLACK)) {
         pixDestroy(&pixd);
-        return ERROR_INT("fill error", procName, 1);
+        L_WARNING("can't make the map", procName);
+        return 1;
     }
 
         /* Finally, for each connected region corresponding to the
@@ -1029,7 +1064,8 @@ PIX       *pixmr, *pixmg, *pixmb;
         pixDestroy(&pixmr);
         pixDestroy(&pixmg);
         pixDestroy(&pixmb);
-        return ERROR_INT("fill error", procName, 1);
+        L_WARNING("can't make the maps", procName);
+        return 1;
     }
 
         /* Finally, for each connected region corresponding to the
@@ -1123,7 +1159,8 @@ PIX       *pixm, *pixt1, *pixt2, *pixt3, *pixims;
     ny = pixGetHeight(pixs) / reduction;
     if (pixFillMapHoles(pixm, nx, ny, L_FILL_BLACK)) {
         pixDestroy(&pixm);
-        return ERROR_INT("fill error", procName, 1);
+        L_WARNING("can't make the map", procName);
+        return 1;
     }
 
         /* Finally, for each connected region corresponding to the
@@ -1207,7 +1244,6 @@ PIX       *pixm, *pixmr, *pixmg, *pixmb, *pixt1, *pixt2, *pixt3, *pixims;
         pixmr = pixAnd(NULL, pixm, pixt3);
     else
         pixmr = pixClone(pixt3);
-    *ppixmr = pixmr;
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
@@ -1219,7 +1255,6 @@ PIX       *pixm, *pixmr, *pixmg, *pixmb, *pixt1, *pixt2, *pixt3, *pixims;
         pixmg = pixAnd(NULL, pixm, pixt3);
     else
         pixmg = pixClone(pixt3);
-    *ppixmg = pixmg;
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
@@ -1231,7 +1266,6 @@ PIX       *pixm, *pixmr, *pixmg, *pixmb, *pixt1, *pixt2, *pixt3, *pixims;
         pixmb = pixAnd(NULL, pixm, pixt3);
     else
         pixmb = pixClone(pixt3);
-    *ppixmb = pixmb;
     pixDestroy(&pixm);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
@@ -1240,9 +1274,15 @@ PIX       *pixm, *pixmr, *pixmg, *pixmb, *pixt1, *pixt2, *pixt3, *pixims;
         /* Fill all the holes in the three maps. */
     nx = pixGetWidth(pixs) / reduction;
     ny = pixGetHeight(pixs) / reduction;
-    pixFillMapHoles(pixmr, nx, ny, L_FILL_BLACK);
-    pixFillMapHoles(pixmg, nx, ny, L_FILL_BLACK);
-    pixFillMapHoles(pixmb, nx, ny, L_FILL_BLACK);
+    if (pixFillMapHoles(pixmr, nx, ny, L_FILL_BLACK) ||
+        pixFillMapHoles(pixmg, nx, ny, L_FILL_BLACK) ||
+        pixFillMapHoles(pixmb, nx, ny, L_FILL_BLACK)) {
+        pixDestroy(&pixmr);
+        pixDestroy(&pixmg);
+        pixDestroy(&pixmb);
+        L_WARNING("can't make the maps", procName);
+        return 1;
+    }
 
         /* Finally, for each connected region corresponding to the
          * fg mask in each component, reset all pixels to their
@@ -1254,6 +1294,9 @@ PIX       *pixm, *pixmr, *pixmg, *pixmb, *pixt1, *pixt2, *pixt3, *pixims;
         pixDestroy(&pixims);
     }
 
+    *ppixmr = pixmr;
+    *ppixmg = pixmg;
+    *ppixmb = pixmb;
     return 0;
 }
 
@@ -1347,7 +1390,8 @@ PIX      *pixt;
     
     if (nmiss == nx) {  /* no data in any column! */
         numaDestroy(&na);
-        return ERROR_INT("no background found", procName, 1);
+        L_WARNING("no bg found; no data in any column", procName);
+        return 1;
     }
 
     /* ---------- Fill in missing columns by replication ----------- */
@@ -1515,6 +1559,8 @@ PIXA      *pixa;
 /*------------------------------------------------------------------*
  *                 Measurement of local foreground                  *
  *------------------------------------------------------------------*/
+#if 0    /* Not working properly: do not use */
+
 /*!
  *  pixGetForegroundGrayMap()
  *
@@ -1639,13 +1685,14 @@ PIX     *pixd, *piximi, *pixim2, *pixims, *pixs2, *pixb, *pixt1, *pixt2, *pixt3;
     pixDestroy(&pixt3);
     return 0;
 }
+#endif   /* Not working properly: do not use */
 
 
 /*------------------------------------------------------------------*
  *                  Generate inverted background map                *
  *------------------------------------------------------------------*/
 /*!
- *  pixGetInvBackground()
+ *  pixGetInvBackgroundMap()
  *
  *      Input:  pixs (8 bpp)
  *              bgval (target bg val; typ. > 128)
@@ -1800,7 +1847,7 @@ pixApplyInvBackgroundRGBMap(PIX     *pixs,
 {
 l_int32    w, h, wm, hm, wpls, wpld, i, j, k, m, xoff, yoff;
 l_int32    rvald, gvald, bvald;
-l_uint32   vals, vald;
+l_uint32   vals;
 l_uint32   rval16, gval16, bval16;
 l_uint32  *datas, *datad, *lines, *lined, *flines, *flined;
 PIX       *pixd;
@@ -1848,8 +1895,7 @@ PIX       *pixd;
                     gvald = L_MIN(gvald, 255);
                     bvald = (((vals >> 8) & 0xff) * bval16) / 256;
                     bvald = L_MIN(bvald, 255);
-                    vald = (rvald << 24 | gvald << 16 | bvald << 8);
-                    *(flined + xoff + m) = vald;
+                    composeRGBPixel(rvald, gvald, bvald, flined + xoff + m);
                 }
             }
         }
@@ -1858,3 +1904,527 @@ PIX       *pixd;
     return pixd;
 }
 
+
+/*------------------------------------------------------------------*
+ *                         Apply variable map                       *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixApplyVariableGrayMap()
+ *
+ *      Input:  pixs (8 bpp)
+ *              pixg (8 bpp, variable map)
+ *              target (typ. 128 for threshold)
+ *      Return: pixd (8 bpp), or null on error
+ *
+ *  Notes:
+ *      (1) Suppose you have an image that you want to transform based
+ *          on some photometric measurement at each point, such as the
+ *          threshold value for binarization.  Representing the photometric
+ *          measurement as an image pixg, you can threshold in input image
+ *          using pixVarThresholdToBinary().  Alternatively, you can map
+ *          the input image pointwise so that the threshold over the
+ *          entire image becomes a constant, such as 128.  For example,
+ *          if a pixel in pixg is 150 and the target is 128, the
+ *          corresponding pixel in pixs is mapped linearly to a value
+ *          (128/150) of the input value.  If the resulting mapped image
+ *          pixd were then thresholded at 128, you would obtain the
+ *          same result as a direct binarization using pixg with
+ *          pixVarThresholdToBinary().
+ *      (2) The sizes of pixs and pixg must be equal.
+ */
+PIX *
+pixApplyVariableGrayMap(PIX     *pixs,
+                        PIX     *pixg,
+                        l_int32  target)
+{
+l_int32    i, j, w, h, d, wpls, wplg, wpld, vals, valg, vald;
+l_uint8   *lut;
+l_uint32  *datas, *datag, *datad, *lines, *lineg, *lined;
+l_float32  fval;
+PIX       *pixd;
+
+    PROCNAME("pixApplyVariableGrayMap");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (!pixg)
+        return (PIX *)ERROR_PTR("pixg not defined", procName, NULL);
+    if (!pixSizesEqual(pixs, pixg))
+        return (PIX *)ERROR_PTR("pix sizes not equal", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 8)
+        return (PIX *)ERROR_PTR("depth not 8 bpp", procName, NULL);
+
+        /* Generate a LUT for the mapping if the image is large enough
+         * to warrant the overhead.  The LUT is of size 2^16.  For the
+         * index to the table, get the MSB from pixs and the LSB from pixg.
+         * Note: this LUT is bigger than the typical 32K L1 cache, so
+         * we expect cache misses.  L2 latencies are about 5ns.  But 
+         * division is slooooow.  For large images, this function is about
+         * 4x faster when using the LUT.  C'est la vie.  */
+    lut = NULL;
+    if (w * h > 100000) {  /* more pixels than 2^16 */
+        if ((lut = (l_uint8 *)CALLOC(0x10000, sizeof(l_uint8))) == NULL)
+            return (PIX *)ERROR_PTR("lut not made", procName, NULL);
+        for (i = 0; i < 256; i++) {
+            for (j = 0; j < 256; j++) {
+                fval = (l_float32)(i * target) / (j + 0.5);
+                lut[(i << 8) + j] = L_MIN(255, (l_int32)(fval + 0.5));
+            }
+        }
+    }
+
+    pixd = pixCreateNoInit(w, h, 8);
+    datad = pixGetData(pixd);
+    wpld = pixGetWpl(pixd);
+    datas = pixGetData(pixs);
+    wpls = pixGetWpl(pixs);
+    datag = pixGetData(pixg);
+    wplg = pixGetWpl(pixg);
+    for (i = 0; i < h; i++) {
+        lines = datas + i * wpls;
+        lineg = datag + i * wplg;
+        lined = datad + i * wpld;
+        if (lut) {
+            for (j = 0; j < w; j++) {
+                vals = GET_DATA_BYTE(lines, j);
+                valg = GET_DATA_BYTE(lineg, j);
+                vald = lut[(vals << 8) + valg];
+                SET_DATA_BYTE(lined, j, vald);
+            }
+        }
+        else {
+            for (j = 0; j < w; j++) {
+                vals = GET_DATA_BYTE(lines, j);
+                valg = GET_DATA_BYTE(lineg, j);
+                fval = (l_float32)(vals * target) / (valg + 0.5);
+                vald = L_MIN(255, (l_int32)(fval + 0.5));
+                SET_DATA_BYTE(lined, j, vald);
+            }
+        }
+    }
+
+    if (lut) FREE(lut);
+    return pixd;
+}
+
+
+/*------------------------------------------------------------------*
+ *                  Non-adaptive (global) mapping                   *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixGlobalNormRGB()
+ *
+ *      Input:  pixd (<optional> null, existing or equal to pixs)
+ *              pixs (32 bpp rgb, or colormapped)
+ *              rval, gval, bval (pixel values in pixs that are 
+ *                                linearly mapped to mapval)
+ *              mapval (use 255 for mapping to white)
+ *      Return: pixd (32 bpp rgb or colormapped), or null on error
+ *
+ *  Notes:
+ *    (1) The value of pixd determines if the results are written to a
+ *        new pix (use NULL), in-place to pixs (use pixs), or to some
+ *        other existing pix.
+ *    (2) This does a global normalization of an image where the
+ *        r,g,b color components are not balanced.  Thus, white in pixs is
+ *        represented by a set of r,g,b values that are not all 255.
+ *    (3) The input values (rval, gval, bval) should be chosen to
+ *        represent the gray color (mapval, mapval, mapval) in src.
+ *        Thus, this function will map (rval, gval, bval) to that gray color.
+ *    (4) Typically, mapval = 255, so that (rval, gval, bval)
+ *        corresponds to the white point of src.  In that case, these
+ *        parameters should be chosen so that few pixels have higher values.
+ *    (5) In all cases, we do a linear TRC separately on each of the
+ *        components, saturating at 255.
+ *    (6) If the input pix is 8 bpp without a colormap, you can get
+ *        this functionality with mapval = 255 by calling:
+ *            pixGammaTRC(pixd, pixs, 1.0, 0, bgval);
+ *        where bgval is the value you want to be mapped to 255.
+ *        Or more generally, if you want bgval to be mapped to mapval:
+ *            pixGammaTRC(pixd, pixs, 1.0, 0, 255 * bgval / mapval);
+ */
+PIX *
+pixGlobalNormRGB(PIX     *pixd,
+                 PIX     *pixs,
+                 l_int32  rval,
+                 l_int32  gval,
+                 l_int32  bval,
+                 l_int32  mapval)
+{
+l_int32    w, h, d, i, j, ncolors, rv, gv, bv, wpl;
+l_int32   *rarray, *garray, *barray;
+l_uint32  *data, *line;
+NUMA      *nar, *nag, *nab;
+PIXCMAP   *cmap;
+
+    PROCNAME("pixGlobalNormRGB");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    cmap = pixGetColormap(pixs);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (!cmap && d != 32)
+        return (PIX *)ERROR_PTR("pixs not cmapped or 32 bpp", procName, NULL);
+    if (mapval <= 0) {
+        L_WARNING("mapval must be > 0; setting to 255", procName);
+        mapval = 255;
+    }
+
+        /* Prepare pixd to be a copy of pixs */
+    if ((pixd = pixCopy(pixd, pixs)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+
+        /* Generate the TRC maps for each component */
+    nar = numaGammaTRC(1.0, 0, 255 * rval / mapval);
+    nag = numaGammaTRC(1.0, 0, 255 * gval / mapval);
+    nab = numaGammaTRC(1.0, 0, 255 * bval / mapval);
+    if (!nar || !nag || !nab)
+        return (PIX *)ERROR_PTR("trc maps not all made", procName, pixd);
+
+        /* Extract copies of the internal arrays */
+    rarray = numaGetIArray(nar);
+    garray = numaGetIArray(nag);
+    barray = numaGetIArray(nab);
+    if (!rarray || !garray || !barray)
+        return (PIX *)ERROR_PTR("*arrays not all made", procName, pixd);
+
+    if (cmap) {
+        ncolors = pixcmapGetCount(cmap);
+        for (i = 0; i < ncolors; i++) {
+            pixcmapGetColor(cmap, i, &rv, &gv, &bv);
+            pixcmapResetColor(cmap, i, rarray[rv], garray[gv], barray[bv]);
+        }
+    }
+    else {
+        data = pixGetData(pixd);
+        wpl = pixGetWpl(pixd);
+        for (i = 0; i < h; i++) {
+            line = data + i * wpl;
+            for (j = 0; j < w; j++) {
+                extractRGBValues(line[j], &rv, &gv, &bv);
+                composeRGBPixel(rarray[rv], garray[gv], barray[bv], line + j);
+            }
+        }
+    }
+
+    numaDestroy(&nar);
+    numaDestroy(&nag);
+    numaDestroy(&nab);
+    FREE(rarray);
+    FREE(garray);
+    FREE(barray);
+    return pixd;
+}
+
+
+/*!
+ *  pixGlobalNormNoSatRGB()
+ *
+ *      Input:  pixd (<optional> null, existing or equal to pixs)
+ *              pixs (32 bpp rgb)
+ *              rval, gval, bval (pixel values in pixs that are 
+ *                                linearly mapped to mapval; but see below)
+ *              factor (subsampling factor; integer >= 1)
+ *              rank (between 0.0 and 1.0; typ. use a value near 1.0)
+ *      Return: pixd (32 bpp rgb), or null on error
+ *
+ *  Notes:
+ *    (1) This is a version of pixGlobalNormRGB(), where the output
+ *        intensity is scaled back so that a controlled fraction of
+ *        pixel components is allowed to saturate.  See comments in
+ *        pixGlobalNormRGB().
+ *    (2) The value of pixd determines if the results are written to a
+ *        new pix (use NULL), in-place to pixs (use pixs), or to some
+ *        other existing pix.
+ *    (3) This does a global normalization of an image where the
+ *        r,g,b color components are not balanced.  Thus, white in pixs is
+ *        represented by a set of r,g,b values that are not all 255.
+ *    (4) The input values (rval, gval, bval) can be chosen to be the
+ *        color that, after normalization, becomes white background.
+ *        For images that are mostly background, the closer these values
+ *        are to the median component values, the closer the resulting
+ *        background will be to gray, becoming white at the brightest places.
+ *    (5) The mapval used in pixGlobalNormRGB() is computed here to
+ *        avoid saturation of any component in the image (save for a
+ *        fraction of the pixels given by the input rank value).
+ */
+PIX *
+pixGlobalNormNoSatRGB(PIX       *pixd,
+                      PIX       *pixs,
+                      l_int32    rval,
+                      l_int32    gval,
+                      l_int32    bval,
+                      l_int32    factor,
+                      l_float32  rank)
+{
+l_int32    mapval;
+l_float32  rankrval, rankgval, rankbval;
+l_float32  rfract, gfract, bfract, maxfract;
+
+    PROCNAME("pixGlobalNormNoSatRGB");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixGetDepth(pixs) != 32)
+        return (PIX *)ERROR_PTR("pixs not 32 bpp", procName, NULL);
+    if (factor < 1)
+        return (PIX *)ERROR_PTR("sampling factor < 1", procName, NULL);
+    if (rank < 0.0 || rank > 1.0)
+        return (PIX *)ERROR_PTR("rank not in [0.0 ... 1.0]", procName, NULL);
+    if (rval <= 0 || gval <= 0 || bval <= 0)
+        return (PIX *)ERROR_PTR("invalid estim. color values", procName, NULL);
+
+        /* The max value for each component may be larger than the
+         * input estimated background value.  In that case, mapping
+         * for those pixels would saturate.  To prevent saturation,
+         * we compute the fraction for each component by which we
+         * would oversaturate.  Then take the max of these, and
+         * reduce, uniformly over all components, the output intensity
+         * by this value.  Then no component will saturate.
+         * In practice, if rank < 1.0, a fraction of pixels
+         * may have a component saturate.  By keeping rank close to 1.0,
+         * that fraction can be made arbitrarily small. */
+    pixGetRankValueMaskedRGB(pixs, NULL, 0, 0, factor, rank, &rankrval,
+                             &rankgval, &rankbval);
+    rfract = rankrval / (l_float32)rval;
+    gfract = rankgval / (l_float32)gval;
+    bfract = rankbval / (l_float32)bval;
+    maxfract = L_MAX(rfract, gfract);
+    maxfract = L_MAX(maxfract, bfract);
+#if  DEBUG_GLOBAL
+    fprintf(stderr, "rankrval = %7.2f, rankgval = %7.2f, rankbval = %7.2f\n",
+            rankrval, rankgval, rankbval);
+    fprintf(stderr, "rfract = %7.4f, gfract = %7.4f, bfract = %7.4f\n",
+            rfract, gfract, bfract);
+#endif  /* DEBUG_GLOBAL */
+
+    mapval = (l_int32)(255. / maxfract);
+    pixd = pixGlobalNormRGB(pixd, pixs, rval, gval, bval, mapval);
+    return pixd;
+}
+
+
+/*------------------------------------------------------------------*
+ *              Adaptive threshold spread normalization             *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixThresholdSpreadNorm()
+ *
+ *      Input:  pixs (8 bpp)
+ *              filtertype (L_SOBEL_EDGE or L_TWO_SIDED_EDGE);
+ *              edgethresh (threshold on magnitude of edge filter; typ 10-20)
+ *              smoothx, smoothy (half-width of convolution kernel applied to
+ *                                spread threshold: use 0 for no smoothing)
+ *              gamma (gamma correction; typ. about 0.7)
+ *              minval  (input value that gives 0 for output; typ. -25)
+ *              maxval  (input value that gives 255 for output; typ. 255)
+ *              targetthresh (target threshold for normalization)
+ *              &pixth (<optional return> computed local threshold value)
+ *              &pixb (<optional return> thresholded normalized image)
+ *              &pixd (<optional return> normalized image)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The basis of this approach is the use of seed spreading
+ *          on a (possibly) sparse set of estimates for the local threshold.
+ *          The resulting dense estimates are smoothed by convolution
+ *          and used to either threshold the input image or normalize it
+ *          with a local transformation that linearly maps the pixels so
+ *          that the local threshold estimate becomes constant over the
+ *          resulting image.  This approach is one of several that
+ *          have been suggested (and implemented) by Ray Smith.
+ *      (2) You can use either the Sobel or TwoSided edge filters.
+ *          The results appear to be similar, using typical values
+ *          of edgethresh in the rang 10-20.
+ *      (3) To skip the trc enhancement, use gamma = 1.0, minval = 0
+ *          and maxval = 255.
+ *      (4) For the normalized image pixd, each pixel is linearly mapped
+ *          in such a way that the local threshold is equal to targetthresh.
+ *      (5) The full width and height of the convolution kernel
+ *          are (2 * smoothx + 1) and (2 * smoothy + 1).
+ *      (6) This function can be used with the pixtiling utility if the
+ *          images are too large.  See pixOtsuAdaptiveThreshold() for
+ *          an example of this.
+ */
+l_int32
+pixThresholdSpreadNorm(PIX       *pixs,
+                       l_int32    filtertype,
+                       l_int32    edgethresh,
+                       l_int32    smoothx,
+                       l_int32    smoothy,
+                       l_float32  gamma,
+                       l_int32    minval,
+                       l_int32    maxval,
+                       l_int32    targetthresh,
+                       PIX      **ppixth,
+                       PIX      **ppixb,
+                       PIX      **ppixd)
+{
+l_int32  w, h, d;
+PIX     *pixe, *pixet, *pixsd, *pixg1, *pixg2, *pixth;
+
+    PROCNAME("pixThresholdSpreadNorm");
+
+    if (ppixth) *ppixth = NULL;
+    if (ppixb) *ppixb = NULL;
+    if (ppixd) *ppixd = NULL;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 8)
+        return ERROR_INT("pixs not 8 bpp", procName, 1);
+    if (!ppixth && !ppixb && !ppixd)
+        return ERROR_INT("no output requested", procName, 1);
+    if (filtertype != L_SOBEL_EDGE && filtertype != L_TWO_SIDED_EDGE)
+        return ERROR_INT("invalid filter type", procName, 1);
+
+        /* Get the thresholded edge pixels.  These are the ones
+         * that have values in pixs near the local optimal fg/bg threshold. */
+    if (filtertype == L_SOBEL_EDGE)
+        pixe = pixSobelEdgeFilter(pixs, L_VERTICAL_EDGES);
+    else  /* L_TWO_SIDED_EDGE */
+        pixe = pixTwoSidedEdgeFilter(pixs, L_VERTICAL_EDGES);
+    pixet = pixThresholdToBinary(pixe, edgethresh);
+    pixInvert(pixet, pixet);
+
+        /* Build a seed image whose only nonzero values are those
+         * values of pixs corresponding to pixels in the fg of pixet. */
+    pixsd = pixCreateTemplate(pixs);
+    pixCombineMasked(pixsd, pixs, pixet);
+
+        /* Spread the seed and optionally smooth to reduce noise */
+    pixg1 = pixSeedspread(pixsd, 4);
+    pixg2 = pixBlockconv(pixg1, smoothx, smoothy);
+
+        /* Optionally do a gamma enhancement */
+    pixth = pixGammaTRC(NULL, pixg2, gamma, minval, maxval);
+
+        /* Do the mapping and thresholding */
+    if (ppixd) {
+        *ppixd = pixApplyVariableGrayMap(pixs, pixth, targetthresh); 
+        if (ppixb)
+            *ppixb = pixThresholdToBinary(*ppixd, targetthresh);
+    }
+    else if (ppixb)
+        *ppixb = pixVarThresholdToBinary(pixs, pixth);
+
+    if (ppixth)
+        *ppixth = pixth;
+    else
+        pixDestroy(&pixth);
+
+    pixDestroy(&pixe);
+    pixDestroy(&pixet);
+    pixDestroy(&pixsd);
+    pixDestroy(&pixg1);
+    pixDestroy(&pixg2);
+    return 0;
+}
+
+
+/*------------------------------------------------------------------*
+ *                 Adaptive Otsu-based thresholding                 *
+ *------------------------------------------------------------------*/
+/*!
+ *  pixOtsuAdaptiveThreshold()
+ *
+ *      Input:  pixs (8 bpp)
+ *              sx, sy (desired tile dimensions; actual size may vary)
+ *              smoothx, smoothy (half-width of convolution kernel applied to
+ *                                threshold array: use 0 for no smoothing)
+ *              scorefract (fraction of the max Otsu score; typ. 0.1)
+ *              &pixth (<optional return> array of threshold values
+ *                      found for each tile)
+ *              &pixd (<optional return> thresholded input pixs, based on
+ *                     the threshold array)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The full width and height of the convolution kernel
+ *          are (2 * smoothx + 1) and (2 * smoothy + 1)
+ *      (2) The scorefract is the fraction of the maximum Otsu score, which
+ *          is used to determine the range over which the histogram minimum
+ *          is searched.  See numaSplitDistribution() for details on the
+ *          underlying method of choosing a threshold.
+ *      (3) This uses a modified version of the Otsu criterion for
+ *          splitting the distribution of pixels in each tile into a
+ *          fg and bg part.  The modification consists of searching for
+ *          a minimum in the histogram over a range of pixel values where
+ *          the Otsu score is close to the max score.
+ */
+l_int32
+pixOtsuAdaptiveThreshold(PIX       *pixs,
+                         l_int32    sx,
+                         l_int32    sy,
+                         l_int32    smoothx,
+                         l_int32    smoothy,
+                         l_float32  scorefract,
+                         PIX      **ppixth,
+                         PIX      **ppixd)
+{
+l_int32     w, h, d, nx, ny, i, j, thresh;
+l_uint32    val;
+PIX        *pixt, *pixb, *pixthresh, *pixth, *pixd;
+PIXTILING  *pt;
+
+    PROCNAME("pixOtsuAdaptiveThreshold");
+
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 8)
+        return ERROR_INT("pixs not 8 bpp", procName, 1);
+    if (sx < 16 || sy < 16)
+        return ERROR_INT("sx and sy must be >= 16", procName, 1);
+    if (!ppixth && !ppixd)
+        return ERROR_INT("neither &pixth nor &pixd defined", procName, 1);
+
+        /* Compute the threshold array for the tiles */
+    nx = w / sx;
+    ny = h / sy;
+    pt = pixTilingCreate(pixs, nx, ny, 0, 0, 0);
+    pixthresh = pixCreate(nx, ny, 8);
+    for (i = 0; i < ny; i++) {
+        for (j = 0; j < nx; j++) {
+            pixt = pixTilingGetTile(pt, i, j);
+            pixSplitDistributionFgBg(pixt, scorefract, 1, &thresh,
+                                     NULL, NULL, 0);
+            pixSetPixel(pixthresh, j, i, thresh);
+            pixDestroy(&pixt);
+        }
+    }
+
+        /* Optionally smooth the threshold array */
+    if (smoothx > 0 || smoothy > 0)
+        pixth = pixBlockconv(pixthresh, smoothx, smoothy);
+    else
+        pixth = pixClone(pixthresh);
+    pixDestroy(&pixthresh);
+
+        /* Optionally apply the threshold array to binarize pixs */
+    if (ppixd) {
+        pixd = pixCreate(w, h, 1);
+        for (i = 0; i < ny; i++) {
+            for (j = 0; j < nx; j++) {
+                pixt = pixTilingGetTile(pt, i, j);
+                pixGetPixel(pixth, j, i, &val);
+                pixb = pixThresholdToBinary(pixt, val);
+                pixTilingPaintTile(pixd, i, j, pixb, pt);
+                pixDestroy(&pixt);
+                pixDestroy(&pixb);
+            }
+        }
+        *ppixd = pixd;
+    }
+
+    if (ppixth)
+        *ppixth = pixth;
+    else
+        pixDestroy(&pixth);
+
+    pixTilingDestroy(&pt);
+    return 0;
+}
+
+
