@@ -13,21 +13,29 @@
  -  or altered from any source or modified source distribution.
  *====================================================================*/
 
-
 /*
  *  bmpio.c
  *
- *      routines for read/write bmp to pix
+ *      Read bmp from file
+ *           PIX          *pixReadStreamBmp()
  *
- *              PIX       *pixReadStreamBmp()
- *              l_int32    pixWriteStreamBmp()
+ *      Write bmp to file
+ *           l_int32       pixWriteStreamBmp()
  *
+ *      Read/write to memory   [not on windows]
+ *           PIX          *pixReadMemBmp()
+ *           l_int32       pixWriteMemBmp()
  */
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include "allheaders.h"
+#include "bmp.h"
+
+/* --------------------------------------------*/
+#if  USE_BMPIO   /* defined in environ.h */
+/* --------------------------------------------*/
 
 RGBA_QUAD   bwmap[2] = { {255,255,255,0}, {0,0,0,0} };
 
@@ -39,8 +47,8 @@ RGBA_QUAD   bwmap[2] = { {255,255,255,0}, {0,0,0,0} };
 /*!
  *  pixReadStreamBmp()
  *
- *     Input:  stream opened for read
- *     Return: pix, or null on error
+ *      Input:  stream opened for read
+ *      Return: pix, or null on error
  */
 PIX *
 pixReadStreamBmp(FILE  *fp)
@@ -51,11 +59,11 @@ l_int16    bfType, bfSize, bfFill1, bfReserved1, bfReserved2;
 l_int16    offset, bfFill2, biPlanes, depth, d;
 l_int32    biSize, width, height, xres, yres, compression;
 l_int32    imagebytes, biClrUsed, biClrImportant;
-l_uint8    *colormapBuf;
-l_int32           colormapEntries;
-l_int32           fileBpl, extrabytes, readerror;
-l_int32           pixWpl, pixBpl;
-l_int32           i, j, k;
+l_uint8   *colormapBuf;
+l_int32    colormapEntries;
+l_int32    fileBpl, extrabytes, readerror;
+l_int32    pixWpl, pixBpl;
+l_int32    i, j, k;
 l_uint8    pel[4];
 l_uint8   *data;
 l_uint32  *line, *pword;
@@ -67,7 +75,7 @@ PIXCMAP   *cmap;
     if (!fp)
         return (PIX *)ERROR_PTR("fp not defined", procName, NULL);
 
-        /* read bitmap file header */
+        /* Read bitmap file header */
     fread((char *)&sval, 1, 2, fp);
     bfType = convertOnBigEnd16(sval);
     if (bfType != BMP_ID)
@@ -86,7 +94,7 @@ PIXCMAP   *cmap;
     fread((char *)&sval, 1, 2, fp);
     bfFill2 = convertOnBigEnd16(sval);
 
-        /* read bitmap info header */
+        /* Read bitmap info header */
     fread((char *)&ival, 1, 4, fp);
     biSize = convertOnBigEnd32(ival);
     fread((char *)&ival, 1, 4, fp);
@@ -120,15 +128,15 @@ PIXCMAP   *cmap;
                                              sizeof(RGBA_QUAD))) == NULL)
             return (PIX *)ERROR_PTR("colormapBuf alloc fail", procName, NULL );
 
-            /* read colormap */
+            /* Read colormap */
         if (fread(colormapBuf, sizeof(RGBA_QUAD), colormapEntries, fp) 
                  != colormapEntries) {
-            FREE((void *)colormapBuf);
+            FREE(colormapBuf);
             return (PIX *)ERROR_PTR( "colormap read fail", procName, NULL);
         }
     }
 
-        /* make a 32 bpp pix if depth is 24 bpp */
+        /* Make a 32 bpp pix if depth is 24 bpp */
     d = depth;
     if (depth == 24)
         d = 32;
@@ -142,7 +150,7 @@ PIXCMAP   *cmap;
         L_WARNING("more than 256 colormap entries!", procName);
     if (colormapEntries > 0) {  /* import the colormap to the pix cmap */
         cmap = pixcmapCreate(L_MIN(d, 8));
-        FREE((void *)cmap->array);  /* remove generated cmap array */
+        FREE(cmap->array);  /* remove generated cmap array */
         cmap->array  = (void *)colormapBuf;  /* and replace */
         cmap->n = L_MIN(colormapEntries, 256);
     }
@@ -152,7 +160,7 @@ PIXCMAP   *cmap;
     pixWpl = pixGetWpl(pix);
     pixBpl = 4 * pixWpl;
 
-        /* seek to the start of the bitmap in the file */
+        /* Seek to the start of the bitmap in the file */
     fseek(fp, offset, 0);
 
     if (depth != 24) {  /* typ. 1 or 8 bpp */
@@ -229,7 +237,7 @@ PIXCMAP   *cmap;
     pixEndianByteSwap(pix);
 
         /* ----------------------------------------------
-         * the bmp colormap determines the values of black
+         * The bmp colormap determines the values of black
          * and white pixels for binary in the following way:
          * if black = 1 (255), white = 0
          *      255, 255, 255, 0, 0, 0, 0, 0
@@ -257,10 +265,10 @@ PIXCMAP   *cmap;
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
- *      - We position fp at the beginning of the stream, so it
- *        truncates any existing data
- *      - 2 bpp Bmp files are apparently not valid (!)  We can
- *        write and read them, but nobody else can read ours.
+ *      (1) We position fp at the beginning of the stream, so it
+ *          truncates any existing data
+ *      (2) 2 bpp Bmp files are apparently not valid!.  We can
+ *          write and read them, but nobody else can read ours.
  */
 l_int32
 pixWriteStreamBmp(FILE  *fp,
@@ -273,17 +281,17 @@ l_uint16    bfOffBits, bfFill2, biPlanes, biBitCount;
 l_uint16    sval;
 l_uint32    biSize, biWidth, biHeight, biCompression, biSizeImage;
 l_uint32    biXPelsPerMeter, biYPelsPerMeter, biClrUsed, biClrImportant;
-l_int32            pixWpl, pixBpl, extrabytes, writeerror;
-l_int32            fileBpl, fileWpl;
-l_int32            i, j, k;
+l_int32     pixWpl, pixBpl, extrabytes, writeerror;
+l_int32     fileBpl, fileWpl;
+l_int32     i, j, k;
 l_int32     heapcm;  /* extra copy of cta on the heap ? 1 : 0 */
-l_uint8           *data;
+l_uint8    *data;
 l_uint8     pel[4];
 l_uint32   *line, *pword;
 PIXCMAP    *cmap;
-l_uint8           *cta;          /* address of the bmp color table array */
-l_int32            cmaplen;      /* number of bytes in the bmp colormap */
-l_int32            ncolors, val, stepsize;
+l_uint8    *cta;          /* address of the bmp color table array */
+l_int32     cmaplen;      /* number of bytes in the bmp colormap */
+l_int32     ncolors, val, stepsize;
 RGBA_QUAD  *pquad;
 
     PROCNAME("pixWriteStreamBmp");
@@ -355,7 +363,7 @@ RGBA_QUAD  *pquad;
 
     fseek(fp, 0L, 0);
 
-        /* convert to little-endian and write the file header data */
+        /* Convert to little-endian and write the file header data */
     bfType = convertOnBigEnd16(BMP_ID);
     offbytes = BMP_FHBYTES + BMP_IHBYTES + cmaplen;
     filebytes = offbytes + fileimagebytes;
@@ -377,7 +385,7 @@ RGBA_QUAD  *pquad;
     fwrite(&bfOffBits, 1, 2, fp);
     fwrite(&bfFill2, 1, 2, fp);
 
-        /* convert to little-endian and write the info header data */
+        /* Convert to little-endian and write the info header data */
     biSize = convertOnBigEnd32(BMP_IHBYTES);
     biWidth = convertOnBigEnd32(width);
     biHeight = convertOnBigEnd32(height);
@@ -401,18 +409,18 @@ RGBA_QUAD  *pquad;
     fwrite(&biClrUsed, 1, 4, fp);
     fwrite(&biClrImportant, 1, 4, fp);
 
-        /* write the colormap data */
+        /* Write the colormap data */
     if (ncolors > 0) {
         if (fwrite(cta, 1, cmaplen, fp) != cmaplen) {
             if (heapcm)
-                FREE((void *)cta);
+                FREE(cta);
             return ERROR_INT("colormap write fail", procName, 1);
         }
         if (heapcm)
-            FREE((void *)cta);
+            FREE(cta);
     }
 
-        /* when you write a binary image with a colormap
+        /* When you write a binary image with a colormap
          * that sets BLACK to 0, you must invert the data */
     if (depth == 1 && cmap && ((l_uint8 *)(cmap->array))[0] == 0x0) {
         pixInvert(pix, pix);
@@ -457,7 +465,7 @@ RGBA_QUAD  *pquad;
         }
     }
 
-        /* restore to original state */
+        /* Restore to original state */
     pixEndianByteSwap(pix);
     if (depth == 1 && cmap && ((l_uint8 *)(cmap->array))[0] == 0x0)
         pixInvert(pix, pix);
@@ -469,3 +477,112 @@ RGBA_QUAD  *pquad;
 }
 
 
+/*---------------------------------------------------------------------*
+ *                         Read/write to memory                        *
+ *---------------------------------------------------------------------*/
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif  /* HAVE_CONFIG_H */
+
+#if HAVE_FMEMOPEN || \
+ (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+
+extern FILE *open_memstream(char **data, size_t *size);
+extern FILE *fmemopen(void *data, size_t size, const char *mode);
+
+/*!
+ *  pixReadMemBmp()
+ *
+ *      Input:  cdata (const; bmp-encoded)
+ *              size (of data)
+ *      Return: pix, or null on error
+ *
+ *  Notes:
+ *      (1) The @size byte of @data must be a null character.
+ */
+PIX *
+pixReadMemBmp(const l_uint8  *cdata,
+              size_t          size)
+{
+l_uint8  *data;
+FILE     *fp;
+PIX      *pix;
+
+    PROCNAME("pixReadMemBmp");
+
+    if (!cdata)
+        return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
+
+    data = (l_uint8 *)cdata;  /* we're really not going to change this */
+    if ((fp = fmemopen(data, size, "r")) == NULL)
+        return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
+    pix = pixReadStreamBmp(fp);
+    fclose(fp);
+    return pix;
+}
+
+
+/*!
+ *  pixWriteMemBmp()
+ *
+ *      Input:  &data (<return> data of tiff compressed image)
+ *              &size (<return> size of returned data)
+ *              pix
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) See pixWriteStreamBmp() for usage.  This version writes to
+ *          memory instead of to a file stream.
+ */
+l_int32
+pixWriteMemBmp(l_uint8  **pdata,
+               size_t    *psize,
+               PIX       *pix)
+{
+l_int32  ret;
+FILE    *fp;
+
+    PROCNAME("pixWriteMemBmp");
+
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1 );
+    if (!psize)
+        return ERROR_INT("&size not defined", procName, 1 );
+    if (!pix)
+        return ERROR_INT("&pix not defined", procName, 1 );
+
+    if ((fp = open_memstream((char **)pdata, psize)) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    ret = pixWriteStreamBmp(fp, pix);
+    fclose(fp);
+    return ret;
+}
+
+#else
+
+PIX *
+pixReadMemBmp(const l_uint8  *cdata,
+              size_t          size)
+{
+    return (PIX *)ERROR_PTR(
+        "bmp read from memory not implemented on this platform",
+        "pixReadMemBmp", NULL);
+}
+
+
+l_int32
+pixWriteMemBmp(l_uint8  **pdata,
+               size_t    *psize,
+               PIX       *pix)
+{
+    return ERROR_INT(
+        "bmp write to memory not implemented on this platform",
+        "pixWriteMemBmp", 1);
+}
+
+#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+
+/* --------------------------------------------*/
+#endif  /* USE_BMPIO */
+/* --------------------------------------------*/
+
