@@ -25,6 +25,7 @@
  *           l_int32     pixcmapAddColor()
  *           l_int32     pixcmapAddNewColor()
  *           l_int32     pixcmapAddBlackOrWhite()
+ *           l_int32     pixcmapSetBlackAndWhite()
  *           l_int32     pixcmapGetCount()
  *           l_int32     pixcmapGetFreeCount()
  *           l_int32     pixcmapClear()
@@ -35,6 +36,8 @@
  *           l_int32     pixcmapGetIndex()
  *           l_int32     pixcmapHasColor()
  *           l_int32     pixcmapGetRankIntensity()
+ *           l_int32     pixcmapGetNearestIndex()
+ *           l_int32     pixcmapGetExtremeValue()
  *
  *      Colormap I/O
  *           l_int32     pixcmapReadStream()
@@ -54,6 +57,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "allheaders.h"
 
 
@@ -290,11 +294,11 @@ pixcmapAddNewColor(PIXCMAP  *cmap,
 {
     PROCNAME("pixcmapAddNewColor");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!pindex)
         return ERROR_INT("&index not defined", procName, 1);
-    *pindex = 0;  /* init for safety if caller doesn't check return val */
+    *pindex = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
 
     if (!pixcmapGetIndex(cmap, rval, gval, bval, pindex))  /* found */
         return 0;
@@ -330,10 +334,11 @@ pixcmapAddBlackOrWhite(PIXCMAP  *cmap,
 {
     PROCNAME("pixcmapAddBlackOrWhite");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!pindex)
         return ERROR_INT("&index not defined", procName, 1);
+    *pindex = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
 
     if (color == 0) {  /* black */
         if (pixcmapGetFreeCount(cmap) > 0)
@@ -352,6 +357,39 @@ pixcmapAddBlackOrWhite(PIXCMAP  *cmap,
 }
 
 
+/*!
+ *  pixcmapSetBlackAndWhite()
+ *
+ *      Input:  cmap
+ *              setblack (0 for no operation; 1 to set darkest color to black)
+ *              setwhite (0 for no operation; 1 to set lightest color to white)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixcmapSetBlackAndWhite(PIXCMAP  *cmap,
+                        l_int32   setblack,
+                        l_int32   setwhite)
+{
+l_int32  index;
+
+    PROCNAME("pixcmapSetBlackAndWhite");
+
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    if (setblack) {
+        pixcmapGetRankIntensity(cmap, 0.0, &index);
+        pixcmapResetColor(cmap, index, 0, 0, 0);
+    }
+    if (setwhite) {
+        pixcmapGetRankIntensity(cmap, 1.0, &index);
+        pixcmapResetColor(cmap, index, 255, 255, 255);
+    }
+
+    return 0;
+}
+
+
 /*!
  *  pixcmapGetCount()
  *
@@ -430,11 +468,12 @@ RGBA_QUAD  *cta;
 
     PROCNAME("pixcmapGetColor");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!prval || !pgval || !pbval)
         return ERROR_INT("&rval, &gval, &bval not all defined", procName, 1);
-    if (index >= cmap->n)
+    *prval = *pgval = *pbval = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+    if (index < 0 || index >= cmap->n)
         return ERROR_INT("index out of bounds", procName, 1);
 
     cta = (RGBA_QUAD *)cmap->array;
@@ -471,7 +510,7 @@ RGBA_QUAD  *cta;
 
     if (!cmap)
         return ERROR_INT("cmap not defined", procName, 1);
-    if (index >= cmap->n)
+    if (index < 0 || index >= cmap->n)
         return ERROR_INT("index out of bounds", procName, 1);
 
     cta = (RGBA_QUAD *)cmap->array;
@@ -504,11 +543,11 @@ RGBA_QUAD  *cta;
 
     PROCNAME("pixcmapGetIndex");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!pindex)
         return ERROR_INT("&index not defined", procName, 1);
-    *pindex = 0;  /* init; not very meaningful */
+    *pindex = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
     n = pixcmapGetCount(cmap);
 
     cta = (RGBA_QUAD *)cmap->array;
@@ -581,10 +620,11 @@ NUMA    *na, *nasort;
 
     PROCNAME("pixcmapGetRankIntensity");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!pindex)
         return ERROR_INT("&index not defined", procName, 1);
+    *pindex = 0;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
     if (rankval < 0.0 || rankval > 1.0)
         return ERROR_INT("rankval not in [0.0 ... 1.0]", procName, 1);
 
@@ -604,6 +644,126 @@ NUMA    *na, *nasort;
 }
 
 
+/*!
+ *  pixcmapGetNearestIndex()
+ *
+ *      Input:  cmap
+ *              rval, gval, bval (colormap colors to search for; each number
+ *                                is in range [0, ... 255])
+ *              &index (<return> the index of the nearest color)
+ *      Return: 0 if OK, 1 on error (caller must check)
+ *
+ *  Notes:
+ *      (1) Returns the index of the exact color if possible, otherwise the
+ *          index of the color closest to the target color.
+ *      (2) Nearest color is that which is the least sum-of-squares distance
+ *          from the target color.
+ */
+l_int32
+pixcmapGetNearestIndex(PIXCMAP  *cmap,
+                       l_int32   rval,
+                       l_int32   gval,
+                       l_int32   bval,
+                       l_int32  *pindex)
+{
+l_int32     i, n, delta, dist, mindist;
+RGBA_QUAD  *cta;
+
+    PROCNAME("pixcmapGetNearestIndex");
+
+    if (!pindex)
+        return ERROR_INT("&index not defined", procName, 1);
+    *pindex = UNDEF;
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+
+    if ((cta = (RGBA_QUAD *)cmap->array) == NULL)
+        return ERROR_INT("cta not defined(!)", procName, 1);
+    n = pixcmapGetCount(cmap);
+
+    mindist = 3 * 255 * 255 + 1;
+    for (i = 0; i < n; i++) {
+        delta = cta[i].red - rval;
+        dist = delta * delta;
+        delta = cta[i].green - gval;
+        dist += delta * delta;
+        delta = cta[i].blue - bval;
+        dist += delta * delta;
+        if (dist < mindist) {
+            *pindex = i; 
+            if (dist == 0)
+                break;
+            mindist = dist;
+        }
+    }
+
+    return 0;
+}
+
+
+/*!
+ *  pixcmapGetExtremeValue()
+ *
+ *      Input:  cmap
+ *              type (L_CHOOSE_MIN or L_CHOOSE_MAX)
+ *              &rval (<optional return> red component)
+ *              &gval (<optional return> green component)
+ *              &bval (<optional return> blue component)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Returns for selected components the extreme value
+ *          (either min or max) of the color component that is
+ *          found in the colormap.
+ */
+l_int32
+pixcmapGetExtremeValue(PIXCMAP  *cmap,
+                       l_int32   type,
+                       l_int32  *prval,
+                       l_int32  *pgval,
+                       l_int32  *pbval)
+{
+l_int32  i, n, rval, gval, bval, extrval, extgval, extbval;
+
+    PROCNAME("pixcmapGetExtremeValue");
+
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
+    if (type != L_CHOOSE_MIN && type != L_CHOOSE_MAX)
+        return ERROR_INT("invalid type", procName, 1);
+    if (!prval && !pgval && !pbval)
+        return ERROR_INT("no result requested for return", procName, 1);
+
+    if (type == L_CHOOSE_MIN) {
+        extrval = 100000;
+        extgval = 100000;
+        extbval = 100000;
+    }
+    else {
+        extrval = 0;
+        extgval = 0;
+        extbval = 0;
+    }
+
+    n = pixcmapGetCount(cmap);
+    for (i = 0; i < n; i++) {
+        pixcmapGetColor(cmap, i, &rval, &gval, &bval);
+        if ((type == L_CHOOSE_MIN && rval < extrval) ||
+            (type == L_CHOOSE_MAX && rval > extrval))
+            extrval = rval;
+        if ((type == L_CHOOSE_MIN && gval < extgval) ||
+            (type == L_CHOOSE_MAX && gval > extgval))
+            extgval = gval;
+        if ((type == L_CHOOSE_MIN && bval < extbval) ||
+            (type == L_CHOOSE_MAX && bval > extbval))
+            extbval = bval;
+    }
+    if (prval) *prval = extrval;
+    if (pgval) *pgval = extgval;
+    if (pbval) *pbval = extbval;
+    return 0;
+}
+
 
 /*-------------------------------------------------------------*
  *                         Colormap I/O                        *
@@ -668,7 +828,7 @@ l_int32   i;
         return ERROR_INT("cmap not defined", procName, 1);
 
     if (pixcmapToArrays(cmap, &rmap, &gmap, &bmap))
-        return ERROR_INT("cmap not defined", procName, 1);
+        return ERROR_INT("colormap arrays not made", procName, 1);
 
     fprintf(fp, "\nPixcmap: depth = %d bpp; %d colors\n", cmap->depth, cmap->n);
     fprintf(fp, "Color    R-val    G-val    B-val\n");
@@ -707,14 +867,13 @@ RGBA_QUAD  *cta;
 
     PROCNAME("pixcmapToArrays");
 
-    if (!cmap)
-        return ERROR_INT("cmap not defined", procName, 1);
     if (!prmap || !pgmap || !pbmap)
         return ERROR_INT("&rmap, &gmap, &bmap not all defined", procName, 1);
-
     *prmap = *pgmap = *pbmap = NULL;
-    ncolors = pixcmapGetCount(cmap);
+    if (!cmap)
+        return ERROR_INT("cmap not defined", procName, 1);
 
+    ncolors = pixcmapGetCount(cmap);
     if (((rmap = (l_int32 *)CALLOC(ncolors, sizeof(l_int32))) == NULL) ||
         ((gmap = (l_int32 *)CALLOC(ncolors, sizeof(l_int32))) == NULL) ||
         ((bmap = (l_int32 *)CALLOC(ncolors, sizeof(l_int32))) == NULL))
