@@ -124,17 +124,23 @@ PIXCMAP   *cmap1, *cmap2;
         pixUsesCmapColor(pix1, &color);
         if (color && d2 <= 8)  /* can't be equal */
             return 0;
+        if (d2 < 8)
+            pixs2 = pixConvertTo8(pix2, FALSE);
+        else
+            pixs2 = pixClone(pix2);
         if (d2 <= 8)
             pixs1 = pixRemoveColormap(pix1, REMOVE_CMAP_TO_GRAYSCALE);
         else
             pixs1 = pixRemoveColormap(pix1, REMOVE_CMAP_TO_FULL_COLOR);
-        pixs2 = pixClone(pix2);
     }
     else if (!cmap1 && cmap2) {
         pixUsesCmapColor(pix2, &color);
         if (color && d1 <= 8)  /* can't be equal */
             return 0;
-        pixs1 = pixClone(pix1);
+        if (d1 < 8)
+            pixs1 = pixConvertTo8(pix1, FALSE);
+        else
+            pixs1 = pixClone(pix1);
         if (d1 <= 8)
             pixs2 = pixRemoveColormap(pix2, REMOVE_CMAP_TO_GRAYSCALE);
         else
@@ -395,7 +401,7 @@ PIXCMAP  *cmap;
         return 0;
 
         /* The cmap has color entries.  Are they used? */
-    na = pixGetHistogram(pixs);
+    na = pixGetGrayHistogram(pixs, 1);
     n = pixcmapGetCount(cmap);
     for (i = 0; i < n; i++) {
         pixcmapGetColor(cmap, i, &rval, &gval, &bval);
@@ -496,6 +502,7 @@ PIX      *pixt;
 
     PROCNAME("pixCompareBinary");
 
+    if (ppixdiff) *ppixdiff = NULL;
     if (!pfract)
         return ERROR_INT("&pfract not defined", procName, 1);
     *pfract = 0.0;
@@ -572,6 +579,7 @@ PIX     *pixt1, *pixt2;
 
     PROCNAME("pixCompareGrayOrRGB");
 
+    if (ppixdiff) *ppixdiff = NULL;
     if (!pix1)
         return ERROR_INT("pix1 not defined", procName, 1);
     if (!pix2)
@@ -635,11 +643,12 @@ pixCompareGray(PIX        *pix1,
 {
 l_int32  first, last;
 GPLOT   *gplot;
-NUMA    *na, *nac;;
+NUMA    *na, *nac;
 PIX     *pixt;
 
     PROCNAME("pixCompareGray");
 
+    if (ppixdiff) *ppixdiff = NULL;
     if (!pix1 || pixGetDepth(pix1) != 8)
         return ERROR_INT("pix1 not defined or not 8 bpp", procName, 1);
     if (!pix2 || pixGetDepth(pix2) != 8)
@@ -663,7 +672,7 @@ PIX     *pixt;
         pixGetAverageMasked(pixt, NULL, 0, 0, 1, L_MEAN_ABSVAL, pdiff);
 
     if (plottype) {
-        na = pixGetHistogram(pixt);
+        na = pixGetGrayHistogram(pixt, 1);
         numaGetNonzeroRange(na, TINY, &first, &last);
         nac = numaClipToInterval(na, 0, last);
         gplot = gplotCreate("/usr/tmp/junkgrayroot", plottype, 
@@ -726,6 +735,7 @@ PIX       *pixr1, *pixr2, *pixg1, *pixg2, *pixb1, *pixb2, *pixr, *pixg, *pixb;
 
     PROCNAME("pixCompareRGB");
 
+    if (ppixdiff) *ppixdiff = NULL;
     if (!pix1 || pixGetDepth(pix1) != 32)
         return ERROR_INT("pix1 not defined or not 32 bpp", procName, 1);
     if (!pix2 || pixGetDepth(pix2) != 32)
@@ -770,9 +780,9 @@ PIX       *pixr1, *pixr2, *pixg1, *pixg2, *pixb1, *pixb2, *pixr, *pixg, *pixb;
     }
 
     if (plottype) {
-        nar = pixGetHistogram(pixr);
-        nag = pixGetHistogram(pixg);
-        nab = pixGetHistogram(pixb);
+        nar = pixGetGrayHistogram(pixr, 1);
+        nag = pixGetGrayHistogram(pixg, 1);
+        nab = pixGetGrayHistogram(pixb, 1);
         numaGetNonzeroRange(nar, TINY, &first, &rlast);
         numaGetNonzeroRange(nag, TINY, &first, &glast);
         numaGetNonzeroRange(nab, TINY, &first, &blast);
@@ -860,11 +870,14 @@ pixCompareTiled(PIX     *pix1,
 {
 l_int32    d1, d2, w, h;
 PIX       *pixt, *pixr, *pixg, *pixb;
-PIX       *pixdiff, *pixrdiff, *pixgdiff, *pixbdiff;
+PIX       *pixrdiff, *pixgdiff, *pixbdiff;
 PIXACC    *pixacc;
 
     PROCNAME("pixCompareTiled");
 
+    if (!ppixdiff)
+        return ERROR_INT("&pixdiff not defined", procName, 1);
+    *ppixdiff = NULL;
     if (!pix1)
         return ERROR_INT("pix1 not defined", procName, 1);
     if (!pix2)
@@ -970,7 +983,7 @@ PIX        *pixt1, *pixt2;
     }
 
     nah = numaCreate(256);
-    nah->n = 256;  /* all initialized to 0.0 */
+    numaSetCount(nah, 256);  /* all initialized to 0.0 */
     array1 = numaGetFArray(nah, L_NOCOPY);
     w = L_MIN(w1, w2);
     h = L_MIN(h1, h2);
@@ -1017,7 +1030,7 @@ PIX        *pixt1, *pixt2;
     array1 = numaGetFArray(nan, L_NOCOPY);
 
     nad = numaCreate(256);
-    nad->n = 256;  /* all initialized to 0.0 */
+    numaSetCount(nad, 256);  /* all initialized to 0.0 */
     array2 = numaGetFArray(nad, L_NOCOPY);
 
         /* Finally, do rank accumulation on normalized histo of diffs */
