@@ -14,25 +14,37 @@
  *====================================================================*/
 
 /*
- *  conncomp.c:  stack-based filling and connected component extraction
+ *  conncomp.c
  *
- *      4- and 8-connected components: bounding boxes and images
+ *    Connected component counting and extraction, using Heckbert's
+ *    stack-based filling algorithm.
  *
+ *      4- and 8-connected components: counts, bounding boxes and images
+ *
+ *      Top-level calls:
  *           BOXA     *pixConnComp()
  *           BOXA     *pixConnCompPixa()
  *           BOXA     *pixConnCompBB()
+ *           l_int32   pixCountConnComp()
  *
+ *      Identify the next c.c. to be erased:
  *           l_int32   nextOnPixelInRaster()
  *           l_int32   nextOnPixelInRasterLow()
  *
+ *      Erase the c.c., saving the b.b.:
  *           BOX      *pixSeedfillBB()
  *           BOX      *pixSeedfill4BB()
  *           BOX      *pixSeedfill8BB()
  *
- *      Stack helper functions for seedfill
+ *      Just erase the c.c.:
+ *           l_int32   pixSeedfill()
+ *           l_int32   pixSeedfill4()
+ *           l_int32   pixSeedfill8()
  *
- *           void      pushFillsegBB()
- *           void      popFillsegBB()
+ *      Static stack helper functions for single raster line seedfill:
+ *           static void    pushFillsegBB()
+ *           static void    pushFillseg()
+ *           static void    popFillseg()
  *
  *  The basic method in pixConnCompBB() is very simple.  We scan the
  *  image in raster order, looking for the next ON pixel.  When it
@@ -57,12 +69,44 @@
  *  Rasterop does all the work.  At the end, we have an array
  *  of the 4- or 8-connected components, as well as an array of the
  *  bounding boxes that describe where they came from in the original image.
+ *
+ *  If you just want the number of connected components, pixCountConnComp()
+ *  is a bit faster than pixConnCompBB(), because it doesn't have to
+ *  keep track of the bounding rectangles for each c.c.
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include "allheaders.h"
 
+
+/*
+ *  The struct FillSeg is used by the Heckbert seedfill algorithm to
+ *  hold information about image segments that are waiting to be
+ *  investigated.  We use two Stacks, one to hold the FillSegs in use,
+ *  and an auxiliary Stack as a reservoir to hold FillSegs for re-use.
+ */
+struct FillSeg
+{
+    l_int32    xleft;    /* left edge of run */
+    l_int32    xright;   /* right edge of run */
+    l_int32    y;        /* run y  */
+    l_int32    dy;       /* parent segment direction: 1 above, -1 below) */
+};
+typedef struct FillSeg    FILLSEG;
+
+
+    /* Static accessors for FillSegs on a stack */
+static void pushFillsegBB(PSTACK *pstack, l_int32 xleft, l_int32 xright,
+                          l_int32 y, l_int32 dy, l_int32 ymax,
+                          l_int32 *pminx, l_int32 *pmaxx,
+                          l_int32 *pminy, l_int32 *pmaxy);
+static void pushFillseg(PSTACK *pstack, l_int32 xleft, l_int32 xright,
+                        l_int32 y, l_int32 dy, l_int32 ymax);
+static void popFillseg(PSTACK *pstack, l_int32 *pxleft, l_int32 *pxright,
+                       l_int32 *py, l_int32 *pdy);
+
+
 #ifndef  NO_CONSOLE_IO
 #define   DEBUG    0
 #endif  /* ~NO_CONSOLE_IO */
@@ -75,13 +119,14 @@
  *  pixConnComp()
  *
  *      Input:  pixs (1 bpp)
- *              &pixa   (<optional> pixa of each c.c.)
+ *              &pixa   (<optional return> pixa of each c.c.)
  *              connectivity (4 or 8)
  *      Return: boxa, or null on error
  *
  *  Notes:
- *      (1) This is the top-level call, and it can be used instead
- *          of either pixConnCompBB() or pixConnCompPixa().
+ *      (1) This is the top-level call for getting bounding boxes or
+ *          a pixa of the components, and it can be used instead
+ *          of either pixConnCompBB() or pixConnCompPixa(), rsp.
  */
 BOXA *
 pixConnComp(PIX     *pixs,
@@ -91,6 +136,7 @@ pixConnComp(PIX     *pixs,
 
     PROCNAME("pixConnComp");
 
+    if (ppixa) *ppixa = NULL;
     if (!pixs)
         return (BOXA *)ERROR_PTR("pixs not defined", procName, NULL);
     if (pixGetDepth(pixs) != 1)
@@ -109,7 +155,7 @@ pixConnComp(PIX     *pixs,
  *  pixConnCompPixa()
  *
  *      Input:  pixs (1 bpp)
- *              &pixa (return pixa of each c.c.)
+ *              &pixa (<return> pixa of each c.c.)
  *              connectivity (4 or 8)
  *      Return: boxa, or null on error
  *
@@ -123,6 +169,8 @@ pixConnComp(PIX     *pixs,
  *          an XOR, and finally erases the c.c. from the second pix.
  *      (3) A clone of the returned boxa (where all boxes in the array
  *          are clones) is inserted into the pixa.
+ *      (4) If the input is valid, this always returns a boxa and a pixa.
+ *          If pixs is empty, the boxa and pixa will be empty.
  */
 BOXA *
 pixConnCompPixa(PIX     *pixs,
@@ -135,20 +183,20 @@ PIX     *pixt1, *pixt2, *pixt3, *pixt4;
 PIXA    *pixa;
 BOX     *box;
 BOXA    *boxa;
-STACK   *stack, *auxstack;
+PSTACK  *pstack, *auxstack;
 
     PROCNAME("pixConnCompPixa");
 
-    if (!pixs)
-        return (BOXA *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) != 1)
-        return (BOXA *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
     if (!ppixa)
         return (BOXA *)ERROR_PTR("&pixa not defined", procName, NULL);
+    *ppixa = NULL;
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOXA *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
     if (connectivity != 4 && connectivity != 8)
         return (BOXA *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
 
-    *ppixa = NULL;  /* init */
+    pixa = pixaCreate(0);
+    *ppixa = pixa;
     pixZero(pixs, &iszero);
     if (iszero)
         return boxaCreate(1);  /* return empty boxa */
@@ -157,16 +205,13 @@ STACK   *stack, *auxstack;
         return (BOXA *)ERROR_PTR("pixt1 not made", procName, NULL);
     if ((pixt2 = pixCopy(NULL, pixs)) == NULL)
         return (BOXA *)ERROR_PTR("pixt2 not made", procName, NULL);
-    if ((pixa = pixaCreate(0)) == NULL)
-        return (BOXA *)ERROR_PTR("pixa not made", procName, NULL);
-    *ppixa = pixa;
 
     h = pixGetHeight(pixs);
-    if ((stack = stackCreate(h)) == NULL)
-        return (BOXA *)ERROR_PTR("stack not made", procName, NULL);
-    if ((auxstack = stackCreate(0)) == NULL)
+    if ((pstack = pstackCreate(h)) == NULL)
+        return (BOXA *)ERROR_PTR("pstack not made", procName, NULL);
+    if ((auxstack = pstackCreate(0)) == NULL)
         return (BOXA *)ERROR_PTR("auxstack not made", procName, NULL);
-    stack->auxstack = auxstack;
+    pstack->auxstack = auxstack;
     if ((boxa = boxaCreate(0)) == NULL)
         return (BOXA *)ERROR_PTR("boxa not made", procName, NULL);
 
@@ -177,7 +222,7 @@ STACK   *stack, *auxstack;
         if (!nextOnPixelInRaster(pixt1, xstart, ystart, &x, &y))
             break;
 
-        if ((box = pixSeedfillBB(pixt1, stack, x, y, connectivity)) == NULL)
+        if ((box = pixSeedfillBB(pixt1, pstack, x, y, connectivity)) == NULL)
             return (BOXA *)ERROR_PTR("box not made", procName, NULL);
         boxaAddBox(boxa, box, L_INSERT);
 
@@ -197,7 +242,7 @@ STACK   *stack, *auxstack;
 #if  DEBUG
     pixCountPixels(pixt1, &iszero, NULL);
     fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
-    pixWrite("junkout7", pixt1, IFF_PNG);
+    pixWrite("junkremain", pixt1, IFF_PNG);
 #endif  /* DEBUG */
 
         /* Remove old boxa of pixa and replace with a clone copy */
@@ -205,7 +250,7 @@ STACK   *stack, *auxstack;
     pixa->boxa = boxaCopy(boxa, L_CLONE);
 
         /* Cleanup, freeing the fillsegs on each stack */
-    stackDestroy(&stack, TRUE);
+    pstackDestroy(&pstack, TRUE);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
 
@@ -236,14 +281,12 @@ l_int32  x, y, xstart, ystart;
 PIX     *pixt;
 BOX     *box;
 BOXA    *boxa;
-STACK   *stack, *auxstack;
+PSTACK  *pstack, *auxstack;
 
     PROCNAME("pixConnCompBB");
 
-    if (!pixs)
-        return (BOXA *)ERROR_PTR("pixs not defined", procName, NULL);
-    if (pixGetDepth(pixs) != 1)
-        return (BOXA *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOXA *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
     if (connectivity != 4 && connectivity != 8)
         return (BOXA *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
 
@@ -255,11 +298,11 @@ STACK   *stack, *auxstack;
         return (BOXA *)ERROR_PTR("pixt not made", procName, NULL);
 
     h = pixGetHeight(pixs);
-    if ((stack = stackCreate(h)) == NULL)
-        return (BOXA *)ERROR_PTR("stack not made", procName, NULL);
-    if ((auxstack = stackCreate(0)) == NULL)
+    if ((pstack = pstackCreate(h)) == NULL)
+        return (BOXA *)ERROR_PTR("pstack not made", procName, NULL);
+    if ((auxstack = pstackCreate(0)) == NULL)
         return (BOXA *)ERROR_PTR("auxstack not made", procName, NULL);
-    stack->auxstack = auxstack;
+    pstack->auxstack = auxstack;
     if ((boxa = boxaCreate(0)) == NULL)
         return (BOXA *)ERROR_PTR("boxa not made", procName, NULL);
 
@@ -270,7 +313,7 @@ STACK   *stack, *auxstack;
         if (!nextOnPixelInRaster(pixt, xstart, ystart, &x, &y))
             break;
 
-        if ((box = pixSeedfillBB(pixt, stack, x, y, connectivity)) == NULL)
+        if ((box = pixSeedfillBB(pixt, pstack, x, y, connectivity)) == NULL)
             return (BOXA *)ERROR_PTR("box not made", procName, NULL);
         boxaAddBox(boxa, box, L_INSERT);
 
@@ -281,17 +324,86 @@ STACK   *stack, *auxstack;
 #if  DEBUG
     pixCountPixels(pixt, &iszero, NULL);
     fprintf(stderr, "Number of remaining pixels = %d\n", iszero);
-    pixWrite("junkout7", pixt, IFF_PNG);
+    pixWrite("junkremain", pixt1, IFF_PNG);
 #endif  /* DEBUG */
 
         /* Cleanup, freeing the fillsegs on each stack */
-    stackDestroy(&stack, TRUE);
+    pstackDestroy(&pstack, TRUE);
     pixDestroy(&pixt);
 
     return boxa;
 }
 
 
+/*!
+ *  pixCountConnComp()
+ *
+ *      Input:  pixs (1 bpp)
+ *              connectivity (4 or 8)
+ *              &count (<return>
+ *      Return: 0 if OK, 1 on error
+ *
+ * Notes:
+ *     (1) This is the top-level call for getting the number of
+ *         4- or 8-connected components in a 1 bpp image.
+ *     (2) It works on a copy of the input pix.  The c.c. are located
+ *         in raster order and erased one at a time.
+ */
+l_int32
+pixCountConnComp(PIX      *pixs,
+                 l_int32   connectivity,
+                 l_int32  *pcount)
+{
+l_int32  h, iszero;
+l_int32  x, y, xstart, ystart;
+PIX     *pixt;
+PSTACK  *pstack, *auxstack;
+
+    PROCNAME("pixCountConnComp");
+
+    if (!pcount)
+        return ERROR_INT("&count not defined", procName, 1);
+    *pcount = 0;  /* initialize the count to 0 */
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
+    if (connectivity != 4 && connectivity != 8)
+        return ERROR_INT("connectivity not 4 or 8", procName, 1);
+
+    pixZero(pixs, &iszero);
+    if (iszero)
+        return 0;
+
+    if ((pixt = pixCopy(NULL, pixs)) == NULL)
+        return ERROR_INT("pixt not made", procName, 1);
+
+    h = pixGetDepth(pixs);
+    if ((pstack = pstackCreate(h)) == NULL)
+        return ERROR_INT("pstack not made", procName, 1);
+    if ((auxstack = pstackCreate(0)) == NULL)
+        return ERROR_INT("auxstack not made", procName, 1);
+    pstack->auxstack = auxstack;
+
+    xstart = 0;
+    ystart = 0;
+    while (1)
+    {
+        if (!nextOnPixelInRaster(pixt, xstart, ystart, &x, &y))
+            break;
+
+        pixSeedfill(pixt, pstack, x, y, connectivity);
+        (*pcount)++;
+        xstart = x;
+        ystart = y;
+    }
+
+        /* Cleanup, freeing the fillsegs on each stack */
+    pstackDestroy(&pstack, TRUE);
+    pixDestroy(&pixt);
+
+    return 0;
+}
+
+
 /*!
  *  nextOnPixelInRaster()
  *
@@ -301,27 +413,25 @@ STACK   *stack, *auxstack;
  *      Return: 1 if a pixel is found; 0 otherwise or on error
  */
 l_int32
-nextOnPixelInRaster(PIX      *pix,
+nextOnPixelInRaster(PIX      *pixs,
                     l_int32   xstart,
                     l_int32   ystart,
                     l_int32  *px,
                     l_int32  *py)
 {
-l_int32    w, h, wpl;
+l_int32    w, h, d, wpl;
 l_uint32  *data;
 
     PROCNAME("nextOnPixelInRaster");
 
-    if (!pix)
+    if (!pixs)
         return ERROR_INT("pixs not defined", procName, 0);
-    if (pixGetDepth(pix) != 1)
-        return ERROR_INT("pix not 1 bpp", procName, 0);
-
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
-    wpl = pixGetWpl(pix);
-    data = pixGetData(pix);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 1)
+        return ERROR_INT("pixs not 1 bpp", procName, 0);
 
+    wpl = pixGetWpl(pixs);
+    data = pixGetData(pixs);
     return nextOnPixelInRasterLow(data, w, h, wpl, xstart, ystart, px, py);
 }
 
@@ -392,7 +502,7 @@ l_uint32  *line, *pword;
  *  pixSeedfillBB()
  *
  *      Input:  pixs (1 bpp)
- *              stack (for holding fillsegs)
+ *              pstack (for holding fillsegs)
  *              x,y   (location of seed pixel)
  *              connectivity  (4 or 8)
  *      Return: box or null on error
@@ -402,8 +512,8 @@ l_uint32  *line, *pword;
  *          stack-based seedfill algorithm.
  */
 BOX *
-pixSeedfillBB(PIX     *pix,
-              STACK   *stack,
+pixSeedfillBB(PIX     *pixs,
+              PSTACK  *pstack,
               l_int32  x,
               l_int32  y,
               l_int32  connectivity)
@@ -412,21 +522,19 @@ BOX  *box;
 
     PROCNAME("pixSeedfillBB");
 
-    if (!pix)
-        return (BOX *)ERROR_PTR("pix not defined", procName, NULL);
-    if (pixGetDepth(pix) != 1)
-        return (BOX *)ERROR_PTR("pix not 1 bpp", procName, NULL);
-    if (!stack)
-        return (BOX *)ERROR_PTR("stack not defined", procName, NULL);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOX *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
+    if (!pstack)
+        return (BOX *)ERROR_PTR("pstack not defined", procName, NULL);
     if (connectivity != 4 && connectivity != 8)
         return (BOX *)ERROR_PTR("connectivity not 4 or 8", procName, NULL);
 
     if (connectivity == 4) {
-        if ((box = pixSeedfill4BB(pix, stack, x, y)) == NULL)
+        if ((box = pixSeedfill4BB(pixs, pstack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
     }
     else if (connectivity == 8) {
-        if ((box = pixSeedfill8BB(pix, stack, x, y)) == NULL)
+        if ((box = pixSeedfill8BB(pixs, pstack, x, y)) == NULL)
             return (BOX *)ERROR_PTR("box not made", procName, NULL);
     }
     else
@@ -440,7 +548,7 @@ BOX  *box;
  *  pixSeedfill4BB()
  *
  *      Input:  pixs (1 bpp)
- *              stack (for holding fillsegs)
+ *              pstack (for holding fillsegs)
  *              x,y   (location of seed pixel)
  *      Return: box or null on error.
  *
@@ -466,12 +574,12 @@ BOX  *box;
  *          overhead in the function calls (vs. macros) is negligible.
  */
 BOX *
-pixSeedfill4BB(PIX     *pix,
-               STACK   *stack,
+pixSeedfill4BB(PIX     *pixs,
+               PSTACK  *pstack,
                l_int32  x,
                l_int32  y)
 {
-l_int32    xstart, wpl, x1, x2, dy;
+l_int32    w, h, xstart, wpl, x1, x2, dy;
 l_int32    xmax, ymax;
 l_int32    minx, maxx, miny, maxy;  /* for bounding box of this c.c. */
 l_uint32  *data, *line;
@@ -479,18 +587,16 @@ BOX       *box;
 
     PROCNAME("pixSeedfill4BB");
 
-    if (!pix)
-        return (BOX *)ERROR_PTR("pix not defined", procName, NULL);
-    if (pixGetDepth(pix) != 1)
-        return (BOX *)ERROR_PTR("pix not 1 bpp", procName, NULL);
-    if (!stack)
-        return (BOX *)ERROR_PTR("stack not defined", procName, NULL);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOX *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
+    if (!pstack)
+        return (BOX *)ERROR_PTR("pstack not defined", procName, NULL);
 
-    xmax = pixGetWidth(pix) - 1;
-    ymax = pixGetHeight(pix) - 1;
-
-    data = pixGetData(pix);
-    wpl = pixGetWpl(pix);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    xmax = w - 1;
+    ymax = h - 1;
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
     line = data + y * wpl;
 
         /* Check pix value of seed; must be within the image and ON */
@@ -502,15 +608,15 @@ BOX       *box;
          * then init b.b. boundaries correctly to seed.  */
     minx = miny = 100000;
     maxx = maxy = 0;
-    pushFillsegBB(stack, x, x, y, 1, ymax, &minx, &maxx, &miny, &maxy);
-    pushFillsegBB(stack, x, x, y + 1, -1, ymax, &minx, &maxx, &miny, &maxy);
+    pushFillsegBB(pstack, x, x, y, 1, ymax, &minx, &maxx, &miny, &maxy);
+    pushFillsegBB(pstack, x, x, y + 1, -1, ymax, &minx, &maxx, &miny, &maxy);
     minx = maxx = x;
     miny = maxy = y;
 
-    while (stack->n > 0)
+    while (pstack->n > 0)
     {
             /* Pop segment off stack and fill a neighboring scan line */
-        popFillsegBB(stack, &x1, &x2, &y, &dy);
+        popFillseg(pstack, &x1, &x2, &y, &dy);
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
@@ -530,17 +636,17 @@ BOX       *box;
             goto skip;
         xstart = x + 1;
         if (xstart < x1 - 1)   /* leak on left? */
-            pushFillsegBB(stack, xstart, x1 - 1, y, -dy,
+            pushFillsegBB(pstack, xstart, x1 - 1, y, -dy,
                           ymax, &minx, &maxx, &miny, &maxy);
 
         x = x1 + 1;
         do {
             for (; x <= xmax && (GET_DATA_BIT(line, x) == 1); x++)
                 CLEAR_DATA_BIT(line, x);
-            pushFillsegBB(stack, xstart, x - 1, y, dy,
+            pushFillsegBB(pstack, xstart, x - 1, y, dy,
                           ymax, &minx, &maxx, &miny, &maxy);
             if (x > x2 + 1)   /* leak on right? */
-                pushFillsegBB(stack, x2 + 1, x - 1, y, -dy,
+                pushFillsegBB(pstack, x2 + 1, x - 1, y, -dy,
                               ymax, &minx, &maxx, &miny, &maxy);
     skip:   for (x++; x <= x2 &&
                       x <= xmax &&
@@ -561,7 +667,7 @@ BOX       *box;
  *  pixSeedfill8BB()
  *
  *      Input:  pixs (1 bpp)
- *              stack (for holding fillsegs)
+ *              pstack (for holding fillsegs)
  *              x,y   (location of seed pixel)
  *      Return: box or null on error.
  *
@@ -580,12 +686,12 @@ BOX       *box;
  *          See comments on pixSeedfill4BB() for more details.
  */
 BOX *
-pixSeedfill8BB(PIX     *pix,
-               STACK   *stack,
+pixSeedfill8BB(PIX     *pixs,
+               PSTACK  *pstack,
                l_int32  x,
                l_int32  y)
 {
-l_int32    xstart, wpl, x1, x2, dy;
+l_int32    w, h, xstart, wpl, x1, x2, dy;
 l_int32    xmax, ymax;
 l_int32    minx, maxx, miny, maxy;  /* for bounding box of this c.c. */
 l_uint32  *data, *line;
@@ -593,18 +699,16 @@ BOX       *box;
 
     PROCNAME("pixSeedfill8BB");
 
-    if (!pix)
-        return (BOX *)ERROR_PTR("pix not defined", procName, NULL);
-    if (pixGetDepth(pix) != 1)
-        return (BOX *)ERROR_PTR("pix not 1 bpp", procName, NULL);
-    if (!stack)
-        return (BOX *)ERROR_PTR("stack not defined", procName, NULL);
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return (BOX *)ERROR_PTR("pixs undefined or not 1 bpp", procName, NULL);
+    if (!pstack)
+        return (BOX *)ERROR_PTR("pstack not defined", procName, NULL);
 
-    xmax = pixGetWidth(pix) - 1;
-    ymax = pixGetHeight(pix) - 1;
-
-    data = pixGetData(pix);
-    wpl = pixGetWpl(pix);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    xmax = w - 1;
+    ymax = h - 1;
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
     line = data + y * wpl;
 
         /* Check pix value of seed; must be ON */
@@ -616,15 +720,15 @@ BOX       *box;
          * then init b.b. boundaries correctly to seed.  */
     minx = miny = 100000;
     maxx = maxy = 0;
-    pushFillsegBB(stack, x, x, y, 1, ymax, &minx, &maxx, &miny, &maxy);
-    pushFillsegBB(stack, x, x, y + 1, -1, ymax, &minx, &maxx, &miny, &maxy);
+    pushFillsegBB(pstack, x, x, y, 1, ymax, &minx, &maxx, &miny, &maxy);
+    pushFillsegBB(pstack, x, x, y + 1, -1, ymax, &minx, &maxx, &miny, &maxy);
     minx = maxx = x;
     miny = maxy = y;
 
-    while (stack->n > 0)
+    while (pstack->n > 0)
     {
             /* Pop segment off stack and fill a neighboring scan line */
-        popFillsegBB(stack, &x1, &x2, &y, &dy);
+        popFillseg(pstack, &x1, &x2, &y, &dy);
         line = data + y * wpl;
 
             /* A segment of scanline y - dy for x1 <= x <= x2 was
@@ -644,17 +748,17 @@ BOX       *box;
             goto skip;
         xstart = x + 1;
         if (xstart < x1)   /* leak on left? */
-            pushFillsegBB(stack, xstart, x1 - 1, y, -dy,
+            pushFillsegBB(pstack, xstart, x1 - 1, y, -dy,
                           ymax, &minx, &maxx, &miny, &maxy);
 
         x = x1;
         do {
             for (; x <= xmax && (GET_DATA_BIT(line, x) == 1); x++)
                 CLEAR_DATA_BIT(line, x);
-            pushFillsegBB(stack, xstart, x - 1, y, dy,
+            pushFillsegBB(pstack, xstart, x - 1, y, dy,
                           ymax, &minx, &maxx, &miny, &maxy);
             if (x > x2)   /* leak on right? */
-                pushFillsegBB(stack, x2 + 1, x - 1, y, -dy,
+                pushFillsegBB(pstack, x2 + 1, x - 1, y, -dy,
                               ymax, &minx, &maxx, &miny, &maxy);
     skip:   for (x++; x <= x2 + 1 && 
                       x <= xmax &&
@@ -671,14 +775,236 @@ BOX       *box;
 }
             
 
+/*!
+ *  pixSeedfill()
+ *
+ *      Input:  pixs (1 bpp)
+ *              pstack (for holding fillsegs)
+ *              x,y   (location of seed pixel)
+ *              connectivity  (4 or 8)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This removes the component from pixs with a fg pixel at (x,y).
+ *      (2) See pixSeedfill4() and pixSeedfill8() for details.
+ */
+l_int32
+pixSeedfill(PIX     *pixs,
+            PSTACK  *pstack,
+            l_int32  x,
+            l_int32  y,
+            l_int32  connectivity)
+{
+l_int32  retval;
+
+    PROCNAME("pixSeedfill");
+
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
+    if (!pstack)
+        return ERROR_INT("pstack not defined", procName, 1);
+    if (connectivity != 4 && connectivity != 8)
+        return ERROR_INT("connectivity not 4 or 8", procName, 1);
+
+    if (connectivity == 4)
+        retval = pixSeedfill4(pixs, pstack, x, y);
+    else  /* connectivity == 8  */
+        retval = pixSeedfill8(pixs, pstack, x, y);
+
+    return retval;
+}
+
+
+/*!
+ *  pixSeedfill4()
+ *
+ *      Input:  pixs (1 bpp)
+ *              pstack (for holding fillsegs)
+ *              x,y   (location of seed pixel)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This is Paul Heckbert's stack-based 4-cc seedfill algorithm.
+ *      (2) This operates on the input 1 bpp pix to remove the fg seed
+ *          pixel, at (x,y), and all pixels that are 4-connected to it.
+ *          The seed pixel at (x,y) must initially be ON.
+ *      (3) Reference: see pixSeedFill4BB()
+ */
+l_int32
+pixSeedfill4(PIX     *pixs,
+             PSTACK  *pstack,
+             l_int32  x,
+             l_int32  y)
+{
+l_int32    w, h, xstart, wpl, x1, x2, dy;
+l_int32    xmax, ymax;
+l_uint32  *data, *line;
+
+    PROCNAME("pixSeedfill4");
+
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
+    if (!pstack)
+        return ERROR_INT("pstack not defined", procName, 1);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    xmax = w - 1;
+    ymax = h - 1;
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+    line = data + y * wpl;
+
+        /* Check pix value of seed; must be within the image and ON */
+    if (x < 0 || x > xmax || y < 0 || y > ymax || (GET_DATA_BIT(line, x) == 0))
+        return 0;
+
+        /* Init stack to seed */
+    pushFillseg(pstack, x, x, y, 1, ymax);
+    pushFillseg(pstack, x, x, y + 1, -1, ymax);
+
+    while (pstack->n > 0)
+    {
+            /* Pop segment off stack and fill a neighboring scan line */
+        popFillseg(pstack, &x1, &x2, &y, &dy);
+        line = data + y * wpl;
+
+            /* A segment of scanline y - dy for x1 <= x <= x2 was
+             * previously filled.  We now explore adjacent pixels 
+             * in scan line y.  There are three regions: to the
+             * left of x1 - 1, between x1 and x2, and to the right of x2.
+             * These regions are handled differently.  Leaks are
+             * possible expansions beyond the previous segment and
+             * going back in the -dy direction.  These can happen 
+             * for x < x1 - 1 and for x > x2 + 1.  Any "leak" segments
+             * are plugged with a push in the -dy (opposite) direction.
+             * And any segments found anywhere are always extended
+             * in the +dy direction.  */
+        for (x = x1; x >= 0 && (GET_DATA_BIT(line, x) == 1); x--)
+            CLEAR_DATA_BIT(line,x);
+        if (x >= x1)  /* pix at x1 was off and was not cleared */
+            goto skip;
+        xstart = x + 1;
+        if (xstart < x1 - 1)   /* leak on left? */
+            pushFillseg(pstack, xstart, x1 - 1, y, -dy, ymax);
+
+        x = x1 + 1;
+        do {
+            for (; x <= xmax && (GET_DATA_BIT(line, x) == 1); x++)
+                CLEAR_DATA_BIT(line, x);
+            pushFillseg(pstack, xstart, x - 1, y, dy, ymax);
+            if (x > x2 + 1)   /* leak on right? */
+                pushFillseg(pstack, x2 + 1, x - 1, y, -dy, ymax);
+    skip:   for (x++; x <= x2 &&
+                      x <= xmax &&
+                      (GET_DATA_BIT(line, x) == 0); x++)
+                ;
+            xstart = x;
+        } while (x <= x2 && x <= xmax);
+    }
+
+    return 0;
+}
+            
+
+/*!
+ *  pixSeedfill8()
+ *
+ *      Input:  pixs (1 bpp)
+ *              pstack (for holding fillsegs)
+ *              x,y   (location of seed pixel)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This is Paul Heckbert's stack-based 8-cc seedfill algorithm.
+ *      (2) This operates on the input 1 bpp pix to remove the fg seed
+ *          pixel, at (x,y), and all pixels that are 8-connected to it.
+ *          The seed pixel at (x,y) must initially be ON.
+ *      (3) Reference: see pixSeedFill8BB()
+ */
+l_int32
+pixSeedfill8(PIX     *pixs,
+             PSTACK  *pstack,
+             l_int32  x,
+             l_int32  y)
+{
+l_int32    w, h, xstart, wpl, x1, x2, dy;
+l_int32    xmax, ymax;
+l_uint32  *data, *line;
+
+    PROCNAME("pixSeedfill8");
+
+    if (!pixs || pixGetDepth(pixs) != 1)
+        return ERROR_INT("pixs not defined or not 1 bpp", procName, 1);
+    if (!pstack)
+        return ERROR_INT("pstack not defined", procName, 1);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    xmax = w - 1;
+    ymax = h - 1;
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+    line = data + y * wpl;
+
+        /* Check pix value of seed; must be ON */
+    if (x < 0 || x > xmax || y < 0 || y > ymax || (GET_DATA_BIT(line, x) == 0))
+        return 0;
+
+        /* Init stack to seed */
+    pushFillseg(pstack, x, x, y, 1, ymax);
+    pushFillseg(pstack, x, x, y + 1, -1, ymax);
+
+    while (pstack->n > 0)
+    {
+            /* Pop segment off stack and fill a neighboring scan line */
+        popFillseg(pstack, &x1, &x2, &y, &dy);
+        line = data + y * wpl;
+
+            /* A segment of scanline y - dy for x1 <= x <= x2 was
+             * previously filled.  We now explore adjacent pixels 
+             * in scan line y.  There are three regions: to the
+             * left of x1, between x1 and x2, and to the right of x2.
+             * These regions are handled differently.  Leaks are
+             * possible expansions beyond the previous segment and
+             * going back in the -dy direction.  These can happen 
+             * for x < x1 and for x > x2.  Any "leak" segments
+             * are plugged with a push in the -dy (opposite) direction.
+             * And any segments found anywhere are always extended
+             * in the +dy direction.  */
+        for (x = x1 - 1; x >= 0 && (GET_DATA_BIT(line, x) == 1); x--)
+            CLEAR_DATA_BIT(line,x);
+        if (x >= x1 - 1)  /* pix at x1 - 1 was off and was not cleared */
+            goto skip;
+        xstart = x + 1;
+        if (xstart < x1)   /* leak on left? */
+            pushFillseg(pstack, xstart, x1 - 1, y, -dy, ymax);
+
+        x = x1;
+        do {
+            for (; x <= xmax && (GET_DATA_BIT(line, x) == 1); x++)
+                CLEAR_DATA_BIT(line, x);
+            pushFillseg(pstack, xstart, x - 1, y, dy, ymax);
+            if (x > x2)   /* leak on right? */
+                pushFillseg(pstack, x2 + 1, x - 1, y, -dy, ymax);
+    skip:   for (x++; x <= x2 + 1 && 
+                      x <= xmax &&
+                      (GET_DATA_BIT(line, x) == 0); x++)
+                ;
+            xstart = x;
+        } while (x <= x2 + 1 && x <= xmax);
+    }
+
+    return 0;
+}
+            
+
 
 /*-----------------------------------------------------------------------*
- *               Stack helper functions: push and pop fillsegs           *
+ *          Static stack helper functions: push and pop fillsegs         *
  *-----------------------------------------------------------------------*/
 /*!
  *  pushFillsegBB()
  *
- *      Input:  stack
+ *      Input:  pstack
  *              xleft, xright
  *              y
  *              dy
@@ -695,8 +1021,8 @@ BOX       *box;
  *          fillsegs that are no longer in use.  We only calloc new
  *          fillsegs if the auxiliary stack is empty.
  */
-void
-pushFillsegBB(STACK    *stack,
+static void
+pushFillsegBB(PSTACK   *pstack,
               l_int32   xleft,
               l_int32   xright,
               l_int32   y,
@@ -708,12 +1034,12 @@ pushFillsegBB(STACK    *stack,
               l_int32  *pmaxy)
 {
 FILLSEG  *fseg;
-STACK    *auxstack;
+PSTACK   *auxstack;
 
     PROCNAME("pushFillsegBB");
 
-    if (!stack)
-        return ERROR_VOID(procName, "stack not defined");
+    if (!pstack)
+        return ERROR_VOID(procName, "pstack not defined");
 
     *pminx = L_MIN(*pminx, xleft);
     *pmaxx = L_MAX(*pmaxx, xright);
@@ -721,12 +1047,12 @@ STACK    *auxstack;
     *pmaxy = L_MAX(*pmaxy, y);
 
     if (y + dy >= 0 && y + dy <= ymax) {
-        if ((auxstack = stack->auxstack) == NULL)
+        if ((auxstack = pstack->auxstack) == NULL)
             return ERROR_VOID("auxstack not defined", procName);
 
             /* Get a fillseg to use */
         if (auxstack->n > 0)
-            fseg = (FILLSEG *)stackRemove(auxstack);
+            fseg = (FILLSEG *)pstackRemove(auxstack);
         else {
             if ((fseg = (FILLSEG *)calloc(1, sizeof(FILLSEG))) == NULL)
                 return ERROR_VOID("fillseg not made", procName);
@@ -736,16 +1062,70 @@ STACK    *auxstack;
         fseg->xright = xright;
         fseg->y = y;
         fseg->dy = dy;
-        stackAdd(stack, fseg);
+        pstackAdd(pstack, fseg);
     }
     return;
 }
 
 
 /*!
- *  popFillsegBB()
+ *  pushFillseg()
+ *
+ *      Input:  pstack
+ *              xleft, xright
+ *              y
+ *              dy
+ *              ymax
+ *      Return: void
+ *
+ *  Notes:
+ *      (1) This adds a line segment to the stack.
+ *      (2) The auxiliary stack is used as a storage area to recycle
+ *          fillsegs that are no longer in use.  We only calloc new
+ *          fillsegs if the auxiliary stack is empty.
+ */
+static void
+pushFillseg(PSTACK  *pstack,
+            l_int32  xleft,
+            l_int32  xright,
+            l_int32  y,
+            l_int32  dy,
+            l_int32  ymax)
+{
+FILLSEG  *fseg;
+PSTACK   *auxstack;
+
+    PROCNAME("pushFillseg");
+
+    if (!pstack)
+        return ERROR_VOID(procName, "pstack not defined");
+
+    if (y + dy >= 0 && y + dy <= ymax) {
+        if ((auxstack = pstack->auxstack) == NULL)
+            return ERROR_VOID("auxstack not defined", procName);
+
+            /* Get a fillseg to use */
+        if (auxstack->n > 0)
+            fseg = (FILLSEG *)pstackRemove(auxstack);
+        else {
+            if ((fseg = (FILLSEG *)calloc(1, sizeof(FILLSEG))) == NULL)
+                return ERROR_VOID("fillseg not made", procName);
+        }
+
+        fseg->xleft = xleft;
+        fseg->xright = xright;
+        fseg->y = y;
+        fseg->dy = dy;
+        pstackAdd(pstack, fseg);
+    }
+    return;
+}
+
+
+/*!
+ *  popFillseg()
  * 
- *      Input:  stack
+ *      Input:  pstack
  *              &xleft (<return>)
  *              &xright (<return>)
  *              &y (<return>)
@@ -753,29 +1133,28 @@ STACK    *auxstack;
  *      Return: void
  *
  *  Notes:
- *      (1) This removes a line segment from the stack, and returns
- *          its size.
+ *      (1) This removes a line segment from the stack, and returns its size.
  *      (2) The surplussed fillseg is placed on the auxiliary stack
  *          for future use.
  */
-void
-popFillsegBB(STACK    *stack,
-             l_int32  *pxleft,
-             l_int32  *pxright,
-             l_int32  *py,
-             l_int32  *pdy)
+static void
+popFillseg(PSTACK   *pstack,
+           l_int32  *pxleft,
+           l_int32  *pxright,
+           l_int32  *py,
+           l_int32  *pdy)
 {
 FILLSEG  *fseg;
-STACK    *auxstack;
+PSTACK   *auxstack;
 
-    PROCNAME("popFillsegBB");
+    PROCNAME("popFillseg");
 
-    if (!stack)
-        return ERROR_VOID("stack not defined", procName);
-    if ((auxstack = stack->auxstack) == NULL)
+    if (!pstack)
+        return ERROR_VOID("pstack not defined", procName);
+    if ((auxstack = pstack->auxstack) == NULL)
         return ERROR_VOID("auxstack not defined", procName);
 
-    if ((fseg = (FILLSEG *)stackRemove(stack)) == NULL)
+    if ((fseg = (FILLSEG *)pstackRemove(pstack)) == NULL)
         return;
 
     *pxleft = fseg->xleft;
@@ -784,7 +1163,9 @@ STACK    *auxstack;
     *pdy = fseg->dy;
 
         /* Save it for re-use */
-    stackAdd(auxstack, fseg);
+    pstackAdd(auxstack, fseg);
     return;
 }
 
+
+
