@@ -26,12 +26,19 @@
  *          PIX      *pixBlockconvGray()
  *          PIX      *pixBlockconvAccum()
  *
+ *      Un-normalized grayscale block convolution
+ *          PIX      *pixBlockconvGrayUnnormalized()
+ *
  *      Binary block sum and rank filter
  *          PIX      *pixBlockrank()
  *          PIX      *pixBlocksum()
  *
  *      Woodfill transform
  *          PIX      *pixWoodfillTransform()
+ *
+ *      Generic convolution
+ *          PIX      *pixConvolve()
+ *
  */
 
 #include <stdio.h>
@@ -177,7 +184,7 @@ PIX     *pixd;
  *      Input:  pix (8 bpp)
  *              accum pix (32 bpp; can be null)
  *              wc, hc   (half width/height of convolution kernel)
- *      Return: pix (8 bpp)
+ *      Return: pix (8 bpp), or null on error
  *
  *  Notes:
  *      (1) If accum pix is null, make one and destroy it before
@@ -284,26 +291,118 @@ PIX       *pixd;
 }
 
 
+/*----------------------------------------------------------------------*
+ *               Un-normalized grayscale block convolution              *
+ *----------------------------------------------------------------------*/
+/*!
+ *  pixBlockconvGrayUnnormalized()
+ *
+ *      Input:  pixs (8 bpp)
+ *              wc, hc   (half width/height of convolution kernel)
+ *      Return: pix (32 bpp; containing the convolution without normalizing
+ *                   for the window size), or null on error
+ *
+ *  Notes:
+ *      (1) The full width and height of the convolution kernel
+ *          are (2 * wc + 1) and (2 * hc + 1).
+ *      (2) Returns an error if both wc and hc are 0.
+ *      (3) Adds mirrored border to avoid treating the boundary pixels
+ *          specially.  Note that we add wc + 1 pixels to the left
+ *          and wc to the right.  The added width is 2 * wc + 1 pixels,
+ *          and the particular choice simplifies the indexing in the loop.
+ *          Likewise, add hc + 1 pixels to the top and hc to the bottom.
+ *      (4) To get the normalized result, divide by the area of the
+ *          convolution kernel: (2 * wc + 1) * (2 * hc + 1)
+ *          Specifically, do this:
+ *               pixc = pixBlockconvGrayUnnormalized(pixs, wc, hc);
+ *               fract = 1. / ((2 * wc + 1) * (2 * hc + 1));
+ *               pixMultConstantGray(pixc, fract);
+ *               pixd = pixGetRGBComponent(pixc, L_ALPHA_CHANNEL);
+ *      (5) Unlike pixBlockconvGray(), this always computes the accumulation
+ *          pix because its size is tied to wc and hc.
+ *      (6) Compare this implementation with pixBlockconvGray(), where
+ *          most of the code in blockconvLow() is special casing for
+ *          efficiently handling the boundary.  Here, the use of
+ *          mirrored borders and destination indexing makes the
+ *          implementation very simple.
+ */
+PIX *
+pixBlockconvGrayUnnormalized(PIX     *pixs,
+                             l_int32  wc,
+                             l_int32  hc)
+{
+l_int32    i, j, w, h, d, wpla, wpld, jmax;
+l_uint32  *linemina, *linemaxa, *lined, *dataa, *datad;
+PIX       *pixsb, *pixacc, *pixd;
+
+    PROCNAME("pixBlockconvGrayUnnormalized");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, &d);
+    if (d != 8)
+        return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
+    if (wc < 0) wc = 0;
+    if (hc < 0) hc = 0;
+    if (wc == 0 && hc == 0)   /* no-op */
+        return (PIX *)ERROR_PTR("both wc and hc are 0", procName, NULL);
+    if (w <= wc || h <= hc)
+        L_WARNING("conv kernel half-size >= image dimension!", procName);
+
+    if ((pixsb = pixAddMirroredBorder(pixs, wc + 1, wc, hc + 1, hc)) == NULL)
+        return (PIX *)ERROR_PTR("pixsb not made", procName, NULL);
+    if ((pixacc = pixBlockconvAccum(pixsb)) == NULL)
+        return (PIX *)ERROR_PTR("pixacc not made", procName, NULL);
+    pixDestroy(&pixsb);
+    if ((pixd = pixCreate(w, h, 32)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    
+    wpla = pixGetWpl(pixacc);
+    wpld = pixGetWpl(pixd);
+    datad = pixGetData(pixd);
+    dataa = pixGetData(pixacc);
+    for (i = 0; i < h; i++) {
+        lined = datad + i * wpld;
+        linemina = dataa + i * wpla;
+        linemaxa = dataa + (i + 2 * hc + 1) * wpla;
+        for (j = 0; j < w; j++) {
+            jmax = j + 2 * wc + 1;
+            lined[j] = linemaxa[jmax] - linemaxa[j] -
+                       linemina[jmax] + linemina[j];
+        }
+    }
+
+    pixDestroy(&pixacc);
+    return pixd;
+}
+
+
 /*----------------------------------------------------------------------*
  *                        Binary block sum/rank                         *
  *----------------------------------------------------------------------*/
 /*!
  *  pixBlockrank()
  *
- *      Input:  pix (1 bpp)
+ *      Input:  pixs (1 bpp)
  *              accum pix (<optional> 32 bpp)
  *              wc, hc   (half width/height of block sum/rank kernel)
  *              rank   (between 0.0 and 1.0; 0.5 is median filter)
- *      Return: pix (1 bpp)
+ *      Return: pixd (1 bpp)
  *
  *  Notes:
- *      (1) If accum pix is null, make one, use it, and destroy it
- *          before returning; otherwise, just use the input accum pix
- *      (2) The full width and height of the convolution kernel
+ *      (1) The full width and height of the convolution kernel
  *          are (2 * wc + 1) and (2 * hc + 1)
- *      (3) If both wc and hc are 0, returns a copy unless rank == 0.0,
- *          in which case this returns an all-set image.
- *      (4) Require that wc < w and hc < h, where (w,h) are the dimensions
+ *      (2) This returns a pixd where each pixel is a 1 if the
+ *          neighborhood (2 * wc + 1) x (2 * hc + 1)) pixels
+ *          contains the rank fraction of 1 pixels.  Otherwise,
+ *          the returned pixel is 0.  Note that the special case
+ *          of rank = 0.0 is always satisfied, so the returned
+ *          pixd has all pixels with value 1.
+ *      (3) If accum pix is null, make one, use it, and destroy it
+ *          before returning; otherwise, just use the input accum pix
+ *      (4) If both wc and hc are 0, returns a copy unless rank == 0.0,
+ *          in which case this returns an all-ones image.
+ *      (5) Require that wc < w and hc < h, where (w,h) are the dimensions
  *          of pixs.
  */
 PIX *
@@ -313,10 +412,8 @@ pixBlockrank(PIX       *pixs,
              l_int32    hc,
              l_float32  rank)
 {
-l_int32    i, j, w, h, d, wplbs, wpld;
-l_uint8    thresh, val;
-l_uint32  *databs, *datad, *linebs, *lined;
-PIX       *pixbs, *pixd;
+l_int32  w, h, d, thresh;
+PIX     *pixt, *pixd;
 
     PROCNAME("pixBlockrank");
 
@@ -327,46 +424,32 @@ PIX       *pixbs, *pixd;
         return (PIX *)ERROR_PTR("pixs not 1 bpp", procName, NULL);
     if (rank < 0.0 || rank > 1.0)
         return (PIX *)ERROR_PTR("rank must be in [0.0, 1.0]", procName, NULL);
-    if (wc < 0) wc = 0;
-    if (hc < 0) hc = 0;
+    if (wc < 0 || hc < 0)
+        return (PIX *)ERROR_PTR("wc and hc not both >= 0", procName, NULL);
+    if (rank == 0.0) {
+        pixd = pixCreateTemplate(pixs);
+        pixSetAll(pixd);
+    }
+
     if (wc == 0 && hc == 0) {
         L_WARNING("block of unit size", procName);
-        if (rank == 0.0) {
-            pixd = pixCreateTemplate(pixs);
-            pixSetAll(pixd);
-        }
-        else
-            pixd = pixCopy(NULL, pixs);
-        return pixd;
+        return pixCopy(NULL, pixs);
     }
     if (w <= wc || h <= hc) {
         L_WARNING("conv kernel half-size >= image dimension!", procName);
         return pixCopy(NULL, pixs);
     }
 
-    if ((pixbs = pixBlocksum(pixs, pixacc, wc, hc)) == NULL)
-        return (PIX *)ERROR_PTR("block sum pix not made", procName, NULL);
+    if ((pixt = pixBlocksum(pixs, pixacc, wc, hc)) == NULL)
+        return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
 
-        /* 1 bpp block rank filter output */
-    if ((pixd = pixCreateTemplate(pixs)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    databs = pixGetData(pixbs);
-    wplbs = pixGetWpl(pixbs);
-    datad = pixGetData(pixd);
-    wpld = pixGetWpl(pixd);
-    
-    thresh = (l_int32)(rank * (l_float32)255);
-    for (i = 0; i < h; i++) {
-        linebs = databs + i * wplbs;
-        lined = datad + i * wpld;
-        for (j = 0; j < w; j++) {
-            val = GET_DATA_BYTE(linebs, j);
-            if (val >= thresh)
-                SET_DATA_BIT(lined, j);
-        }
-    }
-
-    pixDestroy(&pixbs);
+        /* 1 bpp block rank filter output.
+         * Must invert because threshold gives 1 for values < thresh,
+         * but we need a 1 if the value is >= thresh. */
+    thresh = (l_int32)(255. * rank);
+    pixd = pixThresholdToBinary(pixt, thresh);
+    pixInvert(pixd, pixd);
+    pixDestroy(&pixt);
     return pixd;
 }
 
@@ -377,7 +460,7 @@ PIX       *pixbs, *pixd;
  *      Input:  pixs (1 bpp)
  *              accum pix (<optional> 32 bpp)
  *              wc, hc   (half width/height of block sum/rank kernel)
- *      Return: pix (8 bpp)
+ *      Return: pixd (8 bpp)
  *
  *  Notes:
  *      (1) If accum pix is null, make one and destroy it before
@@ -406,9 +489,9 @@ pixBlocksum(PIX     *pixs,
             l_int32  wc,
             l_int32  hc)
 {
-l_int32    w, h, d, wplb, wpla;
-l_uint32  *dataa, *datab;
-PIX       *pixt, *pixb;
+l_int32    w, h, d, wplt, wpld;
+l_uint32  *datat, *datad;
+PIX       *pixt, *pixd;
 
     PROCNAME("pixBlocksum");
 
@@ -432,18 +515,18 @@ PIX       *pixt, *pixb;
     }
         
         /* 8 bpp block sum output */
-    if ((pixb = pixCreate(w, h, 8)) == NULL)
-        return (PIX *)ERROR_PTR("pixb not made", procName, NULL);
-    pixCopyResolution(pixb, pixs);
+    if ((pixd = pixCreate(w, h, 8)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    pixCopyResolution(pixd, pixs);
 
-    wplb = pixGetWpl(pixb);
-    wpla = pixGetWpl(pixt);
-    datab = pixGetData(pixb);
-    dataa = pixGetData(pixt);
-    blocksumLow(datab, w, h, wplb, dataa, wpla, wc, hc);
+    wpld = pixGetWpl(pixd);
+    wplt = pixGetWpl(pixt);
+    datad = pixGetData(pixd);
+    datat = pixGetData(pixt);
+    blocksumLow(datad, w, h, wpld, datat, wplt, wc, hc);
 
     pixDestroy(&pixt);
-    return pixb;
+    return pixd;
 }
 
 
@@ -524,3 +607,107 @@ PIX       *pixav, *pixd;
 }
         
 
+/*----------------------------------------------------------------------*
+ *                         Generic convolution                          *
+ *----------------------------------------------------------------------*/
+/*!
+ *  pixConvolve()
+ *
+ *      Input:  pixs (8 bpp)
+ *              kernel
+ *              outdepth (of pixd: 8, 16 or 32)
+ *              normflag (1 to normalize kernel to unit sum; 0 otherwise)
+ *      Return: pixd (8, 16 or 32 bpp)
+ *
+ *  Notes:
+ *      (1) This gives a convolution with an arbitrary kernel.
+ *      (2) The parameter @outdepth determines the depth of the result.
+ *      (3) If normflag == 1, the result is normalized by scaling
+ *          all kernel values for a unit sum.  Do not normalize if
+ *          the kernel has null sum, such as a DoG.
+ *      (4) If the kernel is normalized to unit sum, the output values
+ *          can never exceed 255, so an output depth of 8 bpp is sufficient.
+ *          If the kernel is not normalized, it may be necessary to use
+ *          16 or 32 bpp output to avoid overflow.
+ *      (5) The kernel values can be positive or negative, but the
+ *          result for the convolution can only be stored as a positive
+ *          number.  Consequently, if it goes negative, the choices are
+ *          to clip to 0 or take the absolute value.  We're choosing
+ *          the former for now.  Another possibility would be to output
+ *          a second unsigned image for the negative values.
+ *      (6) This uses a mirrored border to avoid special casing on
+ *          the boundaries.
+ *      (7) The function is slow, running at about 12 machine cycles for
+ *          each pixel-op in the convolution.  For example, with a 3 GHz
+ *          cpu, a 1 Mpixel grayscale image, and a kernel with
+ *          (sx * sy) = 25 elements, the convolution takes about 100 msec.
+ */
+PIX *
+pixConvolve(PIX       *pixs,
+            L_KERNEL  *kel,
+	    l_int32    outdepth,
+	    l_int32    normflag)
+{
+l_int32    i, j, k, m, w, h, sx, sy, cx, cy, wplt, wpld;
+l_int32    val, normsum;
+l_uint32  *datat, *datad, *linet, *lined;
+l_float32  sum, norm;
+PIX       *pixt, *pixd;
+
+    PROCNAME("pixConvolve");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixGetDimensions(pixs, &w, &h, NULL);
+    if (pixGetDepth(pixs) != 8)
+        return (PIX *)ERROR_PTR("pixs not 8 bpp", procName, NULL);
+    if (!kel)
+        return (PIX *)ERROR_PTR("kel not defined", procName, NULL);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    kernelGetParameters(kel, &sy, &sx, &cy, &cx);
+    norm = 1.0;
+    if (normflag) {
+        kernelGetSum(kel, &sum);
+        if (L_ABS(sum) < 0.01)
+            L_ERROR("null sum: skipping normalization", procName);
+	else
+            norm = 1.0 / sum;
+    }
+    if ((pixt = pixAddMirroredBorder(pixs, cx, sx - cx, cy, sy - cy)) == NULL)
+        return (PIX *)ERROR_PTR("pixt not made", procName, NULL);
+
+    pixd = pixCreate(w, h, outdepth);
+    wplt = pixGetWpl(pixt);
+    wpld = pixGetWpl(pixd);
+    datat = pixGetData(pixt);
+    datad = pixGetData(pixd);
+    for (i = 0; i < h; i++) {
+        lined = datad + i * wpld;
+        for (j = 0; j < w; j++) {
+            sum = 0.0;
+            for (k = 0; k < sy; k++) {
+                linet = datat + (i + k) * wplt;
+                for (m = 0; m < sx; m++) {
+                    val = GET_DATA_BYTE(linet, j + m);
+                    sum += val * kel->data[k][m];
+                }
+            }
+#if 1
+	    if (sum < 0.0) sum = -sum;  /* make it non-negative */
+#endif
+            normsum = (l_int32)(norm * sum);
+            if (outdepth == 8)
+                SET_DATA_BYTE(lined, j, normsum);
+            else if (outdepth == 16)
+                SET_DATA_TWO_BYTES(lined, j, normsum);
+            else  /* outdepth == 32 */
+                *(lined + j) = normsum;
+        }
+    }
+
+    pixDestroy(&pixt);
+    return pixd;
+}
+
+
