@@ -185,14 +185,9 @@ static const l_float32  DEFAULT_MIN_UP_DOWN_RATIO = 2.5;
 static const l_int32  DEFAULT_MIN_MIRROR_FLIP_COUNT = 100;
 static const l_float32  DEFAULT_MIN_MIRROR_FLIP_CONF = 5.0;
 
-    /* Debug function */
-static void pixDebugFlipDetect(const char *filename, PIX *pixs, PIX *pixhm);
-
-#ifndef NO_CONSOLE_IO
-#define  DEBUG_UD_DISPLAY  0
-#define  DEBUG_M_DISPLAY   0
-#define  DEBUG_MIRROR      0
-#endif   /* ~NO_CONSOLE_IO */
+    /* Static debug function */
+static void pixDebugFlipDetect(const char *filename, PIX *pixs,
+                               PIX *pixhm, l_int32 enable);
 
 
 /*----------------------------------------------------------------*
@@ -397,10 +392,11 @@ SEL       *sel1, *sel2, *sel3, *sel4;
 
     PROCNAME("pixUpDownDetect");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
     if (!pconf)
         return ERROR_INT("&conf not defined", procName, 1);
+    *pconf = 0.0;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (mincount == 0)
         mincount = DEFAULT_MIN_UP_DOWN_COUNT;
 
@@ -430,9 +426,7 @@ SEL       *sel1, *sel2, *sel3, *sel4;
     pixOr(pixt1, pixt1, pixt2);
     pixt3 = pixReduceRankBinaryCascade(pixt1, 1, 1, 0, 0);
     pixCountPixels(pixt3, &countup, NULL);
-#if DEBUG_UD_DISPLAY
-    pixDebugFlipDetect("junkpixup", pixs, pixt1);
-#endif   /* DEBUG_UD_DISPLAY */
+    pixDebugFlipDetect("junkpixup", pixs, pixt1, debug);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
@@ -442,9 +436,7 @@ SEL       *sel1, *sel2, *sel3, *sel4;
     pixOr(pixt1, pixt1, pixt2);
     pixt3 = pixReduceRankBinaryCascade(pixt1, 1, 1, 0, 0);
     pixCountPixels(pixt3, &countdown, NULL);
-#if DEBUG_UD_DISPLAY
-    pixDebugFlipDetect("junkpixdown", pixs, pixt1);
-#endif   /* DEBUG_UD_DISPLAY */
+    pixDebugFlipDetect("junkpixdown", pixs, pixt1, debug);
     pixDestroy(&pixt1);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
@@ -458,7 +450,6 @@ SEL       *sel1, *sel2, *sel3, *sel4;
     nup = (l_float32)(countup);
     ndown = (l_float32)(countdown);
     nmax = L_MAX(countup, countdown);
-    *pconf = 0.0;
     if (nmax > mincount)
         *pconf = 2. * ((nup - ndown) / sqrt(nup + ndown));
 
@@ -558,10 +549,11 @@ PIX       *pixt, *pixt0, *pixt1, *pixt2, *pixt3;
 
     PROCNAME("pixUpDownDetectDwa");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
     if (!pconf)
         return ERROR_INT("&conf not defined", procName, 1);
+    *pconf = 0.0;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (mincount == 0)
         mincount = DEFAULT_MIN_UP_DOWN_COUNT;
 
@@ -596,7 +588,6 @@ PIX       *pixt, *pixt0, *pixt1, *pixt2, *pixt3;
     ndown = (l_float32)(countdown);
     nmax = L_MAX(countup, countdown);
 
-    *pconf = 0.0;
     if (nmax > mincount)
         *pconf = 2. * ((nup - ndown) / sqrt(nup + ndown));
 
@@ -667,10 +658,11 @@ SEL       *sel1, *sel2;
 
     PROCNAME("pixMirrorDetect");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
     if (!pconf)
         return ERROR_INT("&conf not defined", procName, 1);
+    *pconf = 0.0;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (mincount == 0)
         mincount = DEFAULT_MIN_MIRROR_FLIP_COUNT;
 
@@ -691,9 +683,7 @@ SEL       *sel1, *sel2;
     pixt1 = pixHMT(NULL, pixt0, sel1);
     pixt3 = pixReduceRankBinaryCascade(pixt1, 1, 1, 0, 0);
     pixCountPixels(pixt3, &count1, NULL);
-#if DEBUG_M_DISPLAY
-    pixDebugFlipDetect("junkpixright", pixs, pixt1);
-#endif   /* DEBUG_M_DISPLAY */
+    pixDebugFlipDetect("junkpixright", pixs, pixt1, debug);
     pixDestroy(&pixt1);
     pixDestroy(&pixt3);
 
@@ -701,9 +691,7 @@ SEL       *sel1, *sel2;
     pixt2 = pixHMT(NULL, pixt0, sel2);
     pixt3 = pixReduceRankBinaryCascade(pixt2, 1, 1, 0, 0);
     pixCountPixels(pixt3, &count2, NULL);
-#if DEBUG_M_DISPLAY
-    pixDebugFlipDetect("junkpixleft", pixs, pixt2);
-#endif   /* DEBUG_M_DISPLAY */
+    pixDebugFlipDetect("junkpixleft", pixs, pixt2, debug);
     pixDestroy(&pixt2);
     pixDestroy(&pixt3);
 
@@ -714,7 +702,6 @@ SEL       *sel1, *sel2;
     selDestroy(&sel1);
     selDestroy(&sel2);
 
-    *pconf = 0.0;
     if (nmax > mincount)
         *pconf = 2. * ((nright - nleft) / sqrt(nright + nleft));
 
@@ -760,10 +747,11 @@ PIX       *pixt0, *pixt1, *pixt2, *pixt3;
 
     PROCNAME("pixMirrorDetectDwa");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
     if (!pconf)
         return ERROR_INT("&conf not defined", procName, 1);
+    *pconf = 0.0;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (mincount == 0)
         mincount = DEFAULT_MIN_MIRROR_FLIP_COUNT;
 
@@ -798,7 +786,6 @@ PIX       *pixt0, *pixt1, *pixt2, *pixt3;
     nleft = (l_float32)count2;
     nmax = L_MAX(count1, count2);
 
-    *pconf = 0.0;
     if (nmax > mincount)
         *pconf = 2. * ((nright - nleft) / sqrt(nright + nleft));
 
@@ -823,16 +810,20 @@ PIX       *pixt0, *pixt1, *pixt2, *pixt3;
  *      Input:  filename (for output debug file)
  *              pixs (input to pix*Detect)
  *              pixhm (hit-miss result from ascenders or descenders)
+ *              enable (1 to enable this function; 0 to disable)
  *      Return: void
  */
 static void
 pixDebugFlipDetect(const char *filename,
                    PIX        *pixs,
-                   PIX        *pixhm)
+                   PIX        *pixhm,
+                   l_int32     enable)
 {
 PIX  *pixt, *pixthm;
 
-        /* display with red dot at counted locations */
+   if (!enable) return;
+
+        /* Display with red dot at counted locations */
     pixt = pixConvert1To4Cmap(pixs);
     pixthm = pixMorphSequence(pixhm, "d5.5", 0);
     pixSetMaskedCmap(pixt, pixthm, 0, 0, 255, 0, 0);
