@@ -83,10 +83,6 @@
 #include <string.h>
 #include "allheaders.h"
 
-#if defined(__MINGW32__) || defined(_WIN32)
-#define snprintf _snprintf
-#endif
-
     /* MS VC++ can't handle array initialization with static consts ! */
 #define L_BUF_SIZE      512
 #define MAX_NUM_GPLOTS  40
@@ -169,7 +165,7 @@ GPLOT  *gplot;
     else if (outformat == GPLOT_LATEX)
         snprintf(buf, L_BUF_SIZE, "%s.tex", rootname);
     else  /* outformat == GPLOT_X11 */
-        sprintf(buf, "");
+        buf[0] = '\0';
     gplot->outname = stringNew(buf);
     if (title) gplot->title = stringNew(title);
     if (xlabel) gplot->xlabel = stringNew(xlabel);
@@ -250,7 +246,7 @@ gplotAddPlot(GPLOT       *gplot,
 char       buf[L_BUF_SIZE];
 char      *datastr, *title;
 l_int32    n, i;
-l_float32  valx, valy;
+l_float32  valx, valy, startx, delx;
 SARRAY    *sa;
 
     PROCNAME("gplotAddPlot");
@@ -265,6 +261,7 @@ SARRAY    *sa;
         return ERROR_INT("invalid plotstyle", procName, 1);
 
     n = numaGetCount(nay);
+    numaGetXParameters(nay, &startx, &delx);
     if (nax) {
         if (n != numaGetCount(nax))
             return ERROR_INT("nax and nay sizes differ", procName, 1);
@@ -290,7 +287,7 @@ SARRAY    *sa;
         if (nax)
             numaGetFValue(nax, i, &valx);
         else
-            valx = (l_float32)i;
+            valx = startx + i * delx;
         numaGetFValue(nay, i, &valy);
         snprintf(buf, L_BUF_SIZE, "%f %f\n", valx, valy);
         sarrayAddString(sa, buf, L_COPY);
@@ -391,15 +388,15 @@ FILE    *fp;
         /* Generate command data instructions */
     if (gplot->title) {   /* set title */
         snprintf(buf, L_BUF_SIZE, "set title '%s'", gplot->title);
-	sarrayAddString(gplot->cmddata, buf, L_COPY);
+        sarrayAddString(gplot->cmddata, buf, L_COPY);
     }
     if (gplot->xlabel) {   /* set xlabel */
         snprintf(buf, L_BUF_SIZE, "set xlabel '%s'", gplot->xlabel);
-	sarrayAddString(gplot->cmddata, buf, L_COPY);
+        sarrayAddString(gplot->cmddata, buf, L_COPY);
     }
     if (gplot->ylabel) {   /* set ylabel */
         snprintf(buf, L_BUF_SIZE, "set ylabel '%s'", gplot->ylabel);
-	sarrayAddString(gplot->cmddata, buf, L_COPY);
+        sarrayAddString(gplot->cmddata, buf, L_COPY);
     }
 
     if (gplot->outformat == GPLOT_PNG)    /* set terminal type and output */
@@ -434,7 +431,7 @@ FILE    *fp;
     for (i = 0; i < nplots; i++) {
         plottitle = sarrayGetString(gplot->plottitles, i, L_NOCOPY);
         dataname = sarrayGetString(gplot->datanames, i, L_NOCOPY);
-	numaGetIValue(gplot->plotstyles, i, &plotstyle);
+        numaGetIValue(gplot->plotstyles, i, &plotstyle);
         if (nplots == 1)
             snprintf(buf, L_BUF_SIZE, "plot '%s' title '%s' %s",
                      dataname, plottitle, gplotstylenames[plotstyle]);
