@@ -18,12 +18,15 @@
  *                     
  *      Pta generation for arbitrary shapes built with lines
  *
- *          PTA        *ptaGenerateLine()
- *          PTA        *ptaGenerateWideLine()
- *          PTA        *ptaGenerateBox()
- *          PTA        *ptaGenerateBoxa()
- *          PTA        *ptaGeneratePolyline()
- *          PTA        *ptaGenerateFilledCircle()
+ *          PTA        *generatePtaLine()
+ *          PTA        *generatePtaWideLine()
+ *          PTA        *generatePtaBox()
+ *          PTA        *generatePtaHashBox()
+ *          PTA        *generatePtaBoxa()
+ *          PTAA       *generatePtaaBoxa()
+ *          PTAA       *generatePtaaHashBoxa()
+ *          PTA        *generatePtaPolyline()
+ *          PTA        *generatePtaFilledCircle()
  *
  *      Pta rendering
  *
@@ -41,6 +44,10 @@
  *          l_int32     pixRenderBoxArb()
  *          l_int32     pixRenderBoxBlend()
  *
+ *          l_int32     pixRenderHashBox()
+ *          l_int32     pixRenderHashBoxArb()
+ *          l_int32     pixRenderHashBoxBlend()
+ *
  *          l_int32     pixRenderBoxa()
  *          l_int32     pixRenderBoxaArb()
  *          l_int32     pixRenderBoxaBlend()
@@ -73,14 +80,14 @@
  *        Pta generation for arbitrary shapes built with lines      *
  *------------------------------------------------------------------*/
 /*!
- *  ptaGenerateLine()
+ *  generatePtaLine()
  *
  *      Input:  x1, y1  (end point 1)
  *              x2, y2  (end point 2)
  *      Return: pta, or null on error
  */
 PTA  *
-ptaGenerateLine(l_int32  x1,
+generatePtaLine(l_int32  x1,
                 l_int32  y1,
                 l_int32  x2,
                 l_int32  y2)
@@ -89,9 +96,9 @@ l_int32    npts, diff, getyofx, sign, i, x, y;
 l_float32  slope;
 PTA       *pta;
 
-    PROCNAME("ptaGenerateLine");
+    PROCNAME("generatePtaLine");
 
-        /* generate line parameters */
+        /* Generate line parameters */
     if (L_ABS(x2 - x1) >= L_ABS(y2 - y1)) {
         getyofx = TRUE;
         npts = L_ABS(x2 - x1) + 1;
@@ -115,7 +122,7 @@ PTA       *pta;
         return pta;
     }
 
-        /* generate the set of points */
+        /* Generate the set of points */
     if (getyofx) {  /* y = y(x) */
         for (i = 0; i < npts; i++) {
             x = x1 + sign * i;
@@ -136,14 +143,14 @@ PTA       *pta;
 
 
 /*!
- *  ptaGenerateWideLine()
+ *  generatePtaWideLine()
  *
  *      Input:  x1, y1  (end point 1)
  *              x2, y2  (end point 2)
  *      Return: ptaj, or null on error
  */
 PTA  *
-ptaGenerateWideLine(l_int32  x1,
+generatePtaWideLine(l_int32  x1,
                     l_int32  y1,
                     l_int32  x2,
                     l_int32  y2,
@@ -152,14 +159,14 @@ ptaGenerateWideLine(l_int32  x1,
 l_int32  i, x1a, x2a, y1a, y2a;
 PTA     *pta, *ptaj;
 
-    PROCNAME("ptaGenerateWideLine");
+    PROCNAME("generatePtaWideLine");
 
     if (width < 1) {
         L_WARNING("width < 1; setting to 1", procName);
         width = 1;
     }
 
-    if ((ptaj = ptaGenerateLine(x1, y1, x2, y2)) == NULL)
+    if ((ptaj = generatePtaLine(x1, y1, x2, y2)) == NULL)
         return (PTA *)ERROR_PTR("ptaj not made", procName, NULL);
     if (width == 1)
         return ptaj;
@@ -175,7 +182,7 @@ PTA     *pta, *ptaj;
                 y1a = y1 + (i + 1) / 2;
                 y2a = y2 + (i + 1) / 2;
             }
-            if ((pta = ptaGenerateLine(x1, y1a, x2, y2a)) == NULL)
+            if ((pta = generatePtaLine(x1, y1a, x2, y2a)) == NULL)
                 return (PTA *)ERROR_PTR("pta not made", procName, NULL);
             ptaJoin(ptaj, pta, 0, 0);
             ptaDestroy(&pta);
@@ -191,7 +198,7 @@ PTA     *pta, *ptaj;
                 x1a = x1 + (i + 1) / 2;
                 x2a = x2 + (i + 1) / 2;
             }
-            if ((pta = ptaGenerateLine(x1a, y1, x2a, y2)) == NULL)
+            if ((pta = generatePtaLine(x1a, y1, x2a, y2)) == NULL)
                 return (PTA *)ERROR_PTR("pta not made", procName, NULL);
             ptaJoin(ptaj, pta, 0, 0);
             ptaDestroy(&pta);
@@ -203,7 +210,7 @@ PTA     *pta, *ptaj;
 
 
 /*!
- *  ptaGenerateBox()
+ *  generatePtaBox()
  *
  *      Input:  box
  *              width
@@ -214,13 +221,13 @@ PTA     *pta, *ptaj;
  *          overlapping lines, there is no need to remove duplicates.
  */
 PTA  *
-ptaGenerateBox(BOX     *box,
+generatePtaBox(BOX     *box,
                l_int32  width)
 {
 l_int32  x, y, w, h;
 PTA     *ptad, *pta;
 
-    PROCNAME("ptaGenerateBox");
+    PROCNAME("generatePtaBox");
 
     if (!box)
         return (PTA *)ERROR_PTR("box not defined", procName, NULL);
@@ -229,40 +236,40 @@ PTA     *ptad, *pta;
     boxGetGeometry(box, &x, &y, &w, &h);
     ptad = ptaCreate(0);
     if ((width & 1) == 1) {   /* odd width */
-        pta = ptaGenerateWideLine(x - width / 2, y,
+        pta = generatePtaWideLine(x - width / 2, y,
                                   x + w - 1 + width / 2, y, width);
         ptaJoin(ptad, pta, 0, 0);
         ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x - width / 2, y + h - 1,
-                                  x + w - 1 + width / 2, y + h - 1, width);
-        ptaJoin(ptad, pta, 0, 0);
-        ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x, y + 1 + width / 2,
-                                  x, y + h - 2 - width / 2, width);
-        ptaJoin(ptad, pta, 0, 0);
-        ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x + w - 1, y + 1 + width / 2,
+        pta = generatePtaWideLine(x + w - 1, y + 1 + width / 2,
                                   x + w - 1, y + h - 2 - width / 2, width);
         ptaJoin(ptad, pta, 0, 0);
         ptaDestroy(&pta);
+        pta = generatePtaWideLine(x + w - 1 + width / 2, y + h - 1,
+                                  x - width / 2, y + h - 1, width);
+        ptaJoin(ptad, pta, 0, 0);
+        ptaDestroy(&pta);
+        pta = generatePtaWideLine(x, y + h - 2 - width / 2,
+                                  x, y + 1 + width / 2, width);
+        ptaJoin(ptad, pta, 0, 0);
+        ptaDestroy(&pta);
     }
     else {   /* even width */
-        pta = ptaGenerateWideLine(x - width / 2, y,
+        pta = generatePtaWideLine(x - width / 2, y,
                                   x + w - 2 + width / 2, y, width);
         ptaJoin(ptad, pta, 0, 0);
         ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x - width / 2, y + h - 1,
-                                  x + w - 2 + width / 2, y + h - 1, width);
-        ptaJoin(ptad, pta, 0, 0);
-        ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x, y + 0 + width / 2,
-                                  x, y + h - 2 - width / 2, width);
-        ptaJoin(ptad, pta, 0, 0);
-        ptaDestroy(&pta);
-        pta = ptaGenerateWideLine(x + w - 1, y + 0 + width / 2,
+        pta = generatePtaWideLine(x + w - 1, y + 0 + width / 2,
                                   x + w - 1, y + h - 2 - width / 2, width);
         ptaJoin(ptad, pta, 0, 0);
         ptaDestroy(&pta);
+        pta = generatePtaWideLine(x + w - 2 + width / 2, y + h - 1,
+                                  x - width / 2, y + h - 1, width);
+        ptaJoin(ptad, pta, 0, 0);
+        ptaDestroy(&pta);
+        pta = generatePtaWideLine(x, y + h - 2 - width / 2,
+                                  x, y + 0 + width / 2, width);
+        ptaJoin(ptad, pta, 0, 0);
+        ptaDestroy(&pta);
     }
 
     return ptad;
@@ -270,7 +277,97 @@ PTA     *ptad, *pta;
 
 
 /*!
- *  ptaGenerateBoxa()
+ *  generatePtaHashBox()
+ *
+ *      Input:  box
+ *              spacing (spacing between lines; must be > 1)
+ *              width  (line width)
+ *              orient  (orientation of lines: L_HORIZONTAL_LINE, ...)
+ *              outline  (0 to skip drawing box outline)
+ *      Return: ptad, or null on error
+ *
+ *  Notes:
+ *      (1) The orientation takes on one of 4 orientations (horiz, vertical,
+ *          slope +1, slope -1).
+ *      (2) The full outline is also drawn if @outline = 1.
+ */
+PTA  *
+generatePtaHashBox(BOX     *box,
+                   l_int32  spacing,
+                   l_int32  width,
+                   l_int32  orient,
+                   l_int32  outline)
+{
+l_int32  bx, by, bh, bw, x, y, x1, y1, x2, y2, i, n, npts;
+PTA     *ptad, *pta;
+
+    PROCNAME("generatePtaHashBox");
+
+    if (!box)
+        return (PTA *)ERROR_PTR("box not defined", procName, NULL);
+    if (spacing <= 1)
+        return (PTA *)ERROR_PTR("spacing not > 1", procName, NULL);
+    if (orient != L_HORIZONTAL_LINE && orient != L_POS_SLOPE_LINE &&
+        orient != L_VERTICAL_LINE && orient != L_NEG_SLOPE_LINE)
+        return (PTA *)ERROR_PTR("invalid line orientation", procName, NULL);
+
+        /* Generate line points and add them to the pta. */
+    boxGetGeometry(box, &bx, &by, &bw, &bh);
+    ptad = ptaCreate(0);
+    if (outline) {
+        pta = generatePtaBox(box, width);
+        ptaJoin(ptad, pta, 0, 0);
+        ptaDestroy(&pta);
+    }
+    if (orient == L_HORIZONTAL_LINE) {
+        n = 1 + bh / spacing;
+        for (i = 0; i < n; i++) {
+            y = by + (i * (bh - 1)) / (n - 1);
+            pta = generatePtaWideLine(bx, y, bx + bw - 1, y, width);
+            ptaJoin(ptad, pta, 0, 0);
+            ptaDestroy(&pta);
+	}
+    }
+    else if (orient == L_VERTICAL_LINE) {
+        n = 1 + bw / spacing;
+        for (i = 0; i < n; i++) {
+            x = bx + (i * (bw - 1)) / (n - 1);
+            pta = generatePtaWideLine(x, by, x, by + bh - 1, width);
+            ptaJoin(ptad, pta, 0, 0);
+            ptaDestroy(&pta);
+	}
+    }
+    else if (orient == L_POS_SLOPE_LINE) {
+        n = 2 + (l_int32)((bw + bh) / (1.4 * spacing));
+        for (i = 0; i < n; i++) {
+            x = (l_int32)(bx + (i + 0.5) * 1.4 * spacing);
+            boxIntersectByLine(box, x, by - 1, 1.0, &x1, &y1, &x2, &y2, &npts);
+            if (npts == 2) {
+                pta = generatePtaWideLine(x1, y1, x2, y2, width);
+                ptaJoin(ptad, pta, 0, 0);
+                ptaDestroy(&pta);
+            }
+        }
+    }
+    else {  /* orient == L_NEG_SLOPE_LINE */
+        n = 2 + (l_int32)((bw + bh) / (1.4 * spacing));
+        for (i = 0; i < n; i++) {
+            x = (l_int32)(bx - bh + (i + 0.5) * 1.4 * spacing);
+            boxIntersectByLine(box, x, by - 1, -1.0, &x1, &y1, &x2, &y2, &npts);
+            if (npts == 2) {
+                pta = generatePtaWideLine(x1, y1, x2, y2, width);
+                ptaJoin(ptad, pta, 0, 0);
+                ptaDestroy(&pta);
+            }
+        }
+    }
+
+    return ptad;
+}
+
+
+/*!
+ *  generatePtaBoxa()
  *
  *      Input:  boxa
  *              width
@@ -284,7 +381,7 @@ PTA     *ptad, *pta;
  *          removedups = 1.
  */
 PTA  *
-ptaGenerateBoxa(BOXA    *boxa,
+generatePtaBoxa(BOXA    *boxa,
                 l_int32  width,
                 l_int32  removedups)
 {
@@ -292,7 +389,7 @@ l_int32  i, n;
 BOX     *box;
 PTA     *ptad, *ptat, *pta;
 
-    PROCNAME("ptaGenerateBoxa");
+    PROCNAME("generatePtaBoxa");
 
     if (!boxa)
         return (PTA *)ERROR_PTR("boxa not defined", procName, NULL);
@@ -301,7 +398,7 @@ PTA     *ptad, *ptat, *pta;
     ptat = ptaCreate(0);
     for (i = 0; i < n; i++) {
         box = boxaGetBox(boxa, i, L_CLONE);
-        pta = ptaGenerateBox(box, width);
+        pta = generatePtaBox(box, width);
         ptaJoin(ptat, pta, 0, 0);
         ptaDestroy(&pta);
         boxDestroy(&box);
@@ -318,10 +415,107 @@ PTA     *ptad, *ptat, *pta;
 
 
 /*!
- *  ptaGeneratePolyline()
+ *  generatePtaaBoxa()
+ *
+ *      Input:  boxa
+ *      Return: ptaa, or null on error
+ *
+ *  Notes:
+ *      (1) This generates a pta of the four corners for each box in
+ *          the boxa.
+ *      (2) Each of these pta can be rendered onto a pix with random colors,
+ *          by using pixRenderRandomCmapPtaa() with closeflag = 1.
+ */
+PTAA  *
+generatePtaaBoxa(BOXA  *boxa)
+{
+l_int32  i, n, x, y, w, h;
+BOX     *box;
+PTA     *pta;
+PTAA    *ptaa;
+
+    PROCNAME("generatePtaaBoxa");
+
+    if (!boxa)
+        return (PTAA *)ERROR_PTR("boxa not defined", procName, NULL);
+
+    n = boxaGetCount(boxa);
+    ptaa = ptaaCreate(n);
+    for (i = 0; i < n; i++) {
+        box = boxaGetBox(boxa, i, L_CLONE);
+        boxGetGeometry(box, &x, &y, &w, &h);
+        pta = ptaCreate(4);
+        ptaAddPt(pta, x, y);
+        ptaAddPt(pta, x + w - 1, y);
+        ptaAddPt(pta, x + w - 1, y + h - 1);
+        ptaAddPt(pta, x, y + h - 1);
+        ptaaAddPta(ptaa, pta, L_INSERT);
+        boxDestroy(&box);
+    }
+
+    return ptaa;
+}
+
+
+/*!
+ *  generatePtaaHashBoxa()
+ *
+ *      Input:  boxa
+ *              spacing (spacing between hash lines; must be > 1)
+ *              width  (hash line width)
+ *              orient  (orientation of lines: L_HORIZONTAL_LINE, ...)
+ *              outline  (0 to skip drawing box outline)
+ *      Return: ptaa, or null on error
+ *
+ *  Notes:
+ *      (1) The orientation takes on one of 4 orientations (horiz, vertical,
+ *          slope +1, slope -1).
+ *      (2) The full outline is also drawn if @outline = 1.
+ *      (3) Each of these pta can be rendered onto a pix with random colors,
+ *          by using pixRenderRandomCmapPtaa() with closeflag = 1.
+ *
+ */
+PTAA  *
+generatePtaaHashBoxa(BOXA    *boxa,
+                     l_int32  spacing,
+                     l_int32  width,
+                     l_int32  orient,
+                     l_int32  outline)
+{
+l_int32  i, n;
+BOX     *box;
+PTA     *pta;
+PTAA    *ptaa;
+
+    PROCNAME("generatePtaaHashBoxa");
+
+    if (!boxa)
+        return (PTAA *)ERROR_PTR("boxa not defined", procName, NULL);
+    if (spacing <= 1)
+        return (PTAA *)ERROR_PTR("spacing not > 1", procName, NULL);
+    if (orient != L_HORIZONTAL_LINE && orient != L_POS_SLOPE_LINE &&
+        orient != L_VERTICAL_LINE && orient != L_NEG_SLOPE_LINE)
+        return (PTAA *)ERROR_PTR("invalid line orientation", procName, NULL);
+
+    n = boxaGetCount(boxa);
+    ptaa = ptaaCreate(n);
+    for (i = 0; i < n; i++) {
+        box = boxaGetBox(boxa, i, L_CLONE);
+        pta = generatePtaHashBox(box, spacing, width, orient, outline);
+        ptaaAddPta(ptaa, pta, L_INSERT);
+        boxDestroy(&box);
+    }
+
+    return ptaa;
+}
+
+
+/*!
+ *  generatePtaPolyline()
  *
  *      Input:  pta (vertices of polyline)
  *              width
+ *              closeflag (1 to close the contour; 0 otherwise)
  *              removedups  (1 to remove, 0 to leave)
  *      Return: ptad, or null on error
  *
@@ -332,14 +526,15 @@ PTA     *ptad, *ptat, *pta;
  *          removedups = 1.
  */
 PTA  *
-ptaGeneratePolyline(PTA     *ptas,
+generatePtaPolyline(PTA     *ptas,
                     l_int32  width,
+                    l_int32  closeflag,
                     l_int32  removedups)
 {
 l_int32  i, n, x1, y1, x2, y2;
 PTA     *ptad, *ptat, *pta;
 
-    PROCNAME("ptaGeneratePolyline");
+    PROCNAME("generatePtaPolyline");
 
     if (!ptas)
         return (PTA *)ERROR_PTR("ptas not defined", procName, NULL);
@@ -352,18 +547,19 @@ PTA     *ptad, *ptat, *pta;
     ptaGetIPt(ptas, 0, &x1, &y1);
     for (i = 1; i < n; i++) {
         ptaGetIPt(ptas, i, &x2, &y2);
-        pta = ptaGenerateWideLine(x1, y1, x2, y2, width);
+        pta = generatePtaWideLine(x1, y1, x2, y2, width);
         ptaJoin(ptat, pta, 0, 0);
         ptaDestroy(&pta);
         x1 = x2;
         y1 = y2;
     }
 
-        /* Close the contour */
-    ptaGetIPt(ptas, 0, &x2, &y2);
-    pta = ptaGenerateWideLine(x1, y1, x2, y2, width);
-    ptaJoin(ptat, pta, 0, 0);
-    ptaDestroy(&pta);
+    if (closeflag) {
+        ptaGetIPt(ptas, 0, &x2, &y2);
+        pta = generatePtaWideLine(x1, y1, x2, y2, width);
+        ptaJoin(ptat, pta, 0, 0);
+        ptaDestroy(&pta);
+    }
 
     if (removedups)
         ptad = ptaRemoveDuplicates(ptat, 0);
@@ -376,7 +572,7 @@ PTA     *ptad, *ptat, *pta;
 
 
 /*!
- *  ptaGenerateFilledCircle()
+ *  generatePtaFilledCircle()
  *
  *      Input:  radius
  *      Return: pta, or null on error
@@ -389,13 +585,13 @@ PTA     *ptad, *ptat, *pta;
  *          it is to represent a set of pixels in an image.
  */
 PTA  *
-ptaGenerateFilledCircle(l_int32  radius)
+generatePtaFilledCircle(l_int32  radius)
 {
 l_int32    x, y;
 l_float32  radthresh, sqdist;
 PTA       *pta;
 
-    PROCNAME("ptaGenerateFilledCircle");
+    PROCNAME("generatePtaFilledCircle");
 
     if (radius < 1)
         return (PTA *)ERROR_PTR("radius must be >= 1", procName, NULL);
@@ -452,11 +648,9 @@ l_int32  i, n, x, y, w, h, d, maxval;
     if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
         return ERROR_INT("invalid op", procName, 1);
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, &d);
     maxval = 1;
     if (op == L_SET_PIXELS) {
-        d = pixGetDepth(pix);
         switch (d)
         {
         case 2: 
@@ -549,7 +743,7 @@ PIXCMAP  *cmap;
     }
 
     cmap = pixGetColormap(pix);
-    d = pixGetDepth(pix);
+    pixGetDimensions(pix, &w, &h, &d);
     if (cmap) {
         if (pixcmapAddNewColor(cmap, rval, gval, bval, &index))
             return ERROR_INT("colormap is full", procName, 1);
@@ -565,8 +759,6 @@ PIXCMAP  *cmap;
             composeRGBPixel(rval, gval, bval, &val32);
     }
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
     n = ptaGetCount(pta);
     for (i = 0; i < n; i++) {
         ptaGetIPt(pta, i, &x, &y);
@@ -623,8 +815,7 @@ l_float32  frval, fgval, fbval;
         fract = 0.5;
     }
 
-    w = pixGetWidth(pix);
-    h = pixGetHeight(pix);
+    pixGetDimensions(pix, &w, &h, NULL);
     n = ptaGetCount(pta);
     frval = fract * rval;
     fgval = fract * gval;
@@ -685,7 +876,7 @@ PTA  *pta;
     if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
         return ERROR_INT("invalid op", procName, 1);
 
-    if ((pta = ptaGenerateWideLine(x1, y1, x2, y2, width)) == NULL)
+    if ((pta = generatePtaWideLine(x1, y1, x2, y2, width)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPta(pix, pta, op);
     ptaDestroy(&pta);
@@ -725,7 +916,7 @@ PTA  *pta;
         width = 1;
     }
 
-    if ((pta = ptaGenerateWideLine(x1, y1, x2, y2, width)) == NULL)
+    if ((pta = generatePtaWideLine(x1, y1, x2, y2, width)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaArb(pix, pta, rval, gval, bval);
     ptaDestroy(&pta);
@@ -767,7 +958,7 @@ PTA  *pta;
         width = 1;
     }
 
-    if ((pta = ptaGenerateWideLine(x1, y1, x2, y2, width)) == NULL)
+    if ((pta = generatePtaWideLine(x1, y1, x2, y2, width)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaBlend(pix, pta, rval, gval, bval, fract);
     ptaDestroy(&pta);
@@ -801,7 +992,7 @@ PTA  *pta;
     if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
         return ERROR_INT("invalid op", procName, 1);
 
-    if ((pta = ptaGenerateBox(box, width)) == NULL)
+    if ((pta = generatePtaBox(box, width)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPta(pix, pta, op);
     ptaDestroy(&pta);
@@ -835,7 +1026,7 @@ PTA  *pta;
     if (!box)
         return ERROR_INT("box not defined", procName, 1);
 
-    if ((pta = ptaGenerateBox(box, width)) == NULL)
+    if ((pta = generatePtaBox(box, width)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaArb(pix, pta, rval, gval, bval);
     ptaDestroy(&pta);
@@ -872,7 +1063,148 @@ PTA  *pta;
     if (!box)
         return ERROR_INT("box not defined", procName, 1);
 
-    if ((pta = ptaGenerateBox(box, width)) == NULL)
+    if ((pta = generatePtaBox(box, width)) == NULL)
+        return ERROR_INT("pta not made", procName, 1);
+    pixRenderPtaBlend(pix, pta, rval, gval, bval, fract);
+    ptaDestroy(&pta);
+    return 0;
+}
+
+
+/*!
+ *  pixRenderHashBox()
+ *
+ *      Input:  pix
+ *              box
+ *              spacing (spacing between lines; must be > 1)
+ *              width  (thickness of box and hash lines)
+ *              orient  (orientation of lines: L_HORIZONTAL_LINE, ...)
+ *              outline  (0 to skip drawing box outline)
+ *              op  (one of L_SET_PIXELS, L_CLEAR_PIXELS, L_FLIP_PIXELS)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixRenderHashBox(PIX     *pix,
+                 BOX     *box,
+                 l_int32  spacing,
+                 l_int32  width,
+                 l_int32  orient,
+                 l_int32  outline,
+                 l_int32  op)
+{
+PTA  *pta;
+
+    PROCNAME("pixRenderHashBox");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+    if (spacing <= 1)
+        return ERROR_INT("spacing not > 1", procName, 1);
+    if (orient != L_HORIZONTAL_LINE && orient != L_POS_SLOPE_LINE &&
+        orient != L_VERTICAL_LINE && orient != L_NEG_SLOPE_LINE)
+        return ERROR_INT("invalid line orientation", procName, 1);
+    if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
+        return ERROR_INT("invalid op", procName, 1);
+
+    pta = generatePtaHashBox(box, spacing, width, orient, outline);
+    if (!pta)
+        return ERROR_INT("pta not made", procName, 1);
+    pixRenderPta(pix, pta, op);
+    ptaDestroy(&pta);
+    return 0;
+}
+
+
+/*!
+ *  pixRenderBoxArb()
+ *
+ *      Input:  pix
+ *              box
+ *              spacing (spacing between lines; must be > 1)
+ *              width  (thickness of box and hash lines)
+ *              orient  (orientation of lines: L_HORIZONTAL_LINE, ...)
+ *              outline  (0 to skip drawing box outline)
+ *              rval, gval, bval
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixRenderHashBoxArb(PIX     *pix,
+                    BOX     *box,
+                    l_int32  spacing,
+                    l_int32  width,
+                    l_int32  orient,
+                    l_int32  outline,
+                    l_int32  rval,
+                    l_int32  gval,
+                    l_int32  bval)
+{
+PTA  *pta;
+
+    PROCNAME("pixRenderHashBoxArb");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+    if (spacing <= 1)
+        return ERROR_INT("spacing not > 1", procName, 1);
+    if (orient != L_HORIZONTAL_LINE && orient != L_POS_SLOPE_LINE &&
+        orient != L_VERTICAL_LINE && orient != L_NEG_SLOPE_LINE)
+        return ERROR_INT("invalid line orientation", procName, 1);
+
+    pta = generatePtaHashBox(box, spacing, width, orient, outline);
+    if (!pta)
+        return ERROR_INT("pta not made", procName, 1);
+    pixRenderPtaArb(pix, pta, rval, gval, bval);
+    ptaDestroy(&pta);
+    return 0;
+}
+
+
+/*!
+ *  pixRenderHashBoxBlend()
+ *
+ *      Input:  pix
+ *              box
+ *              spacing (spacing between lines; must be > 1)
+ *              width  (thickness of box and hash lines)
+ *              orient  (orientation of lines: L_HORIZONTAL_LINE, ...)
+ *              outline  (0 to skip drawing box outline)
+ *              rval, gval, bval
+ *              fract (in [0.0 - 1.0]; complete transparency (no effect)
+ *                     if 0.0; no transparency if 1.0)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixRenderHashBoxBlend(PIX       *pix,
+                      BOX       *box,
+                      l_int32    spacing,
+                      l_int32    width,
+                      l_int32    orient,
+                      l_int32    outline,
+                      l_int32    rval,
+                      l_int32    gval,
+                      l_int32    bval,
+                      l_float32  fract)
+{
+PTA  *pta;
+
+    PROCNAME("pixRenderHashBoxBlend");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+    if (spacing <= 1)
+        return ERROR_INT("spacing not > 1", procName, 1);
+    if (orient != L_HORIZONTAL_LINE && orient != L_POS_SLOPE_LINE &&
+        orient != L_VERTICAL_LINE && orient != L_NEG_SLOPE_LINE)
+        return ERROR_INT("invalid line orientation", procName, 1);
+
+    pta = generatePtaHashBox(box, spacing, width, orient, outline);
+    if (!pta)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaBlend(pix, pta, rval, gval, bval, fract);
     ptaDestroy(&pta);
@@ -906,7 +1238,7 @@ PTA  *pta;
     if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
         return ERROR_INT("invalid op", procName, 1);
 
-    if ((pta = ptaGenerateBoxa(boxa, width, 0)) == NULL)
+    if ((pta = generatePtaBoxa(boxa, width, 0)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPta(pix, pta, op);
     ptaDestroy(&pta);
@@ -940,7 +1272,7 @@ PTA  *pta;
     if (!boxa)
         return ERROR_INT("boxa not defined", procName, 1);
 
-    if ((pta = ptaGenerateBoxa(boxa, width, 0)) == NULL)
+    if ((pta = generatePtaBoxa(boxa, width, 0)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaArb(pix, pta, rval, gval, bval);
     ptaDestroy(&pta);
@@ -979,7 +1311,7 @@ PTA  *pta;
     if (!boxa)
         return ERROR_INT("boxa not defined", procName, 1);
 
-    if ((pta = ptaGenerateBoxa(boxa, width, removedups)) == NULL)
+    if ((pta = generatePtaBoxa(boxa, width, removedups)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaBlend(pix, pta, rval, gval, bval, fract);
     ptaDestroy(&pta);
@@ -994,6 +1326,7 @@ PTA  *pta;
  *              ptas
  *              width  (thickness of line)
  *              op  (one of L_SET_PIXELS, L_CLEAR_PIXELS, L_FLIP_PIXELS)
+ *              closeflag (1 to close the contour; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  *
  *  Note: this renders a closed contour.
@@ -1002,7 +1335,8 @@ l_int32
 pixRenderPolyline(PIX     *pix,
                   PTA     *ptas,
                   l_int32  width,
-                  l_int32  op)
+                  l_int32  op,
+                  l_int32  closeflag)
 {
 PTA  *pta;
 
@@ -1015,7 +1349,7 @@ PTA  *pta;
     if (op != L_SET_PIXELS && op != L_CLEAR_PIXELS && op != L_FLIP_PIXELS)
         return ERROR_INT("invalid op", procName, 1);
 
-    if ((pta = ptaGeneratePolyline(ptas, width, 0)) == NULL)
+    if ((pta = generatePtaPolyline(ptas, width, closeflag, 0)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPta(pix, pta, op);
     ptaDestroy(&pta);
@@ -1030,6 +1364,7 @@ PTA  *pta;
  *              ptas
  *              width  (thickness of line)
  *              rval, gval, bval
+ *              closeflag (1 to close the contour; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  *
  *  Note: this renders a closed contour.
@@ -1040,7 +1375,8 @@ pixRenderPolylineArb(PIX     *pix,
                      l_int32  width,
                      l_uint8  rval,
                      l_uint8  gval,
-                     l_uint8  bval)
+                     l_uint8  bval,
+                     l_int32  closeflag)
 {
 PTA  *pta;
 
@@ -1051,7 +1387,7 @@ PTA  *pta;
     if (!ptas)
         return ERROR_INT("ptas not defined", procName, 1);
 
-    if ((pta = ptaGeneratePolyline(ptas, width, 0)) == NULL)
+    if ((pta = generatePtaPolyline(ptas, width, closeflag, 0)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaArb(pix, pta, rval, gval, bval);
     ptaDestroy(&pta);
@@ -1068,6 +1404,7 @@ PTA  *pta;
  *              rval, gval, bval
  *              fract (in [0.0 - 1.0]; complete transparency (no effect)
  *                     if 0.0; no transparency if 1.0)
+ *              closeflag (1 to close the contour; 0 otherwise)
  *              removedups  (1 to remove; 0 otherwise)
  *      Return: 0 if OK, 1 on error
  */
@@ -1079,6 +1416,7 @@ pixRenderPolylineBlend(PIX       *pix,
                        l_uint8    gval,
                        l_uint8    bval,
                        l_float32  fract,
+                       l_int32    closeflag,
                        l_int32    removedups)
 {
 PTA  *pta;
@@ -1090,7 +1428,7 @@ PTA  *pta;
     if (!ptas)
         return ERROR_INT("ptas not defined", procName, 1);
 
-    if ((pta = ptaGeneratePolyline(ptas, width, removedups)) == NULL)
+    if ((pta = generatePtaPolyline(ptas, width, closeflag, removedups)) == NULL)
         return ERROR_INT("pta not made", procName, 1);
     pixRenderPtaBlend(pix, pta, rval, gval, bval, fract);
     ptaDestroy(&pta);
@@ -1104,6 +1442,7 @@ PTA  *pta;
  *      Input:  pix (1, 2, 4, 8, 16, 32 bpp)
  *              ptaa
  *              width  (thickness of line)
+ *              closeflag (1 to close the contour; 0 otherwise)
  *      Return: pixd (cmapped, 8 bpp) or null on error
  *
  *  Notes:
@@ -1117,7 +1456,8 @@ PTA  *pta;
 PIX  *
 pixRenderRandomCmapPtaa(PIX     *pix,
                         PTAA    *ptaa,
-                        l_int32  width)
+                        l_int32  width,
+                        l_int32  closeflag)
 {
 l_int32   i, n, index, rval, gval, bval;
 PIXCMAP  *cmap;
@@ -1142,7 +1482,7 @@ PIX      *pixd;
         index = 1 + (i % 255);
         pixcmapGetColor(cmap, index, &rval, &gval, &bval);
         pta = ptaaGetPta(ptaa, i, L_CLONE);
-        ptat = ptaGeneratePolyline(pta, width, 0);
+        ptat = generatePtaPolyline(pta, width, closeflag, 0);
         pixRenderPtaArb(pixd, ptat, rval, gval, bval);
         ptaDestroy(&pta);
         ptaDestroy(&ptat);
@@ -1151,7 +1491,7 @@ PIX      *pixd;
     return pixd;
 }
 
-        
+
 /*------------------------------------------------------------------*
  *             Contour rendering on grayscale images                *
  *------------------------------------------------------------------*/
