@@ -44,7 +44,6 @@
  *     Binary correlation classifier
  *
  *         l_int32     jbClassifyCorrelation()
- *         l_float32   pixCorrelationScore()
  *
  *     Determine the image components we start with
  *
@@ -169,7 +168,8 @@
  *             // Initialize and save data in the classer
  *         JBCLASSER *classer =
  *             jbCorrelationInit(JB_CONN_COMPS, 0, 0, 0.8, 0.7);
- *         SARRAY *safiles = getSortedPathnamesInDirectory(directory, 0, 0);
+ *         SARRAY *safiles = getSortedPathnamesInDirectory(directory,
+ *                                                         NULL, 0, 0);
  *         jbAddPages(classer, safiles);
  *
  *             // Save the data in a data structure for serialization,
@@ -195,14 +195,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <math.h>
 #include "allheaders.h"
 
-#if defined(__MINGW32__) || defined(_WIN32)
-#define snprintf _snprintf
-#endif
-
-    /* MS VC++ can't handle array initialization with static consts ! */
-#define L_BUF_SIZE      512
+    /* MSVC can't handle arrays dimensioned by static const integers */
+#define  L_BUF_SIZE  512
 
     /* For jbClassifyRankHaus(): size of border added around
      * pix of each c.c., to allow further processing.  This
@@ -210,9 +208,10 @@
      * (or MAX_DIFF_WIDTH) and one-half the size of the Sel  */
 static const l_int32  JB_ADDED_PIXELS = 6;
 
-    /* For pixHaustest(); choose these to be 2 or greater */
-static const l_int32  MAX_DIFF_HEIGHT = 2;  /* use at least 2 */
+    /* For pixHaustest(), pixRankHaustest() and pixCorrelationScore():
+     * choose these to be 2 or greater */
 static const l_int32  MAX_DIFF_WIDTH = 2;  /* use at least 2 */
+static const l_int32  MAX_DIFF_HEIGHT = 2;  /* use at least 2 */
 
     /* In initialization, you have the option to discard components
      * (cc, characters or words) that have either width or height larger
@@ -226,16 +225,19 @@ static const l_int32  MAX_CHAR_COMP_WIDTH = 350;  /* default max char width */
 static const l_int32  MAX_WORD_COMP_WIDTH = 1000;  /* default max word width */
 static const l_int32  MAX_COMP_HEIGHT = 120;  /* default max component height */
 
+    /* Max allowed dilation to merge characters into words */
+#define  MAX_ALLOWED_DILATION  14
+
     /* This stores the state of a state machine which fetches
      * similar sized templates */
 struct JbFindTemplatesState
 {
-  JBCLASSER       *classer;    /* classer                               */
-  l_int32          w;          /* desired width                         */
-  l_int32          h;          /* desired height                        */
-  l_int32          i;          /* index into two_by_two step array      */
-  NUMA            *numa;       /* current number array                  */
-  l_int32          n;          /* current element of numa               */
+    JBCLASSER       *classer;    /* classer                               */
+    l_int32          w;          /* desired width                         */
+    l_int32          h;          /* desired height                        */
+    l_int32          i;          /* index into two_by_two step array      */
+    NUMA            *numa;       /* current number array                  */
+    l_int32          n;          /* current element of numa               */
 };
 typedef struct JbFindTemplatesState JBFINDCTX;
 
@@ -253,6 +255,11 @@ static l_int32 finalPositioningForAlignment(PIX *pixs, l_int32 x, l_int32 y,
                              l_int32 idelx, l_int32 idely, PIX *pixt,
                              l_int32 *sumtab, l_int32 *pdx, l_int32 *pdy);
 
+#ifndef NO_CONSOLE_IO
+#define  DEBUG_PLOT_CC             0
+#define  DEBUG_CORRELATION_SCORE   0
+#endif  /* ~NO_CONSOLE_IO */
+
 
 /*----------------------------------------------------------------------*
  *                            Initialization                            *
@@ -669,7 +676,8 @@ SEL        *sel;
                 pix3 = pixaGetPix(pixat, iclass, L_CLONE);
                 pix4 = pixaGetPix(pixatd, iclass, L_CLONE);
                 ptaGetPt(ptact, iclass, &x2, &y2);
-                testval = pixHaustest(pix1, pix2, pix3, pix4, x1 - x2, y1 - y2);
+                testval = pixHaustest(pix1, pix2, pix3, pix4, x1 - x2, y1 - y2,
+                                      MAX_DIFF_WIDTH, MAX_DIFF_HEIGHT);
                 pixDestroy(&pix3);
                 pixDestroy(&pix4);
                 if (testval == 1) {
@@ -731,6 +739,7 @@ SEL        *sel;
                 ptaGetPt(ptact, iclass, &x2, &y2);
                 testval = pixRankHaustest(pix1, pix2, pix3, pix4,
                                           x1 - x2, y1 - y2,
+                                          MAX_DIFF_WIDTH, MAX_DIFF_HEIGHT,
                                           area1, area3, rank, tab8);
                 pixDestroy(&pix3);
                 pixDestroy(&pix4);
@@ -795,6 +804,8 @@ SEL        *sel;
  *              pix4   (exemplar pix, dilated)
  *              delx   (x comp of centroid difference)
  *              dely   (y comp of centroid difference)
+ *              maxdiffw (max width difference of pix1 and pix2)
+ *              maxdiffh (max height difference of pix1 and pix2)
  *      Return: 0 (FALSE) if no match, 1 (TRUE) if the new
  *              pix is in the same class as the exemplar.
  *
@@ -814,7 +825,9 @@ pixHaustest(PIX       *pix1,
             PIX       *pix3,
             PIX       *pix4,
             l_float32  delx,   /* x(1) - x(3) */
-            l_float32  dely)   /* y(1) - y(3) */
+            l_float32  dely,   /* y(1) - y(3) */
+            l_int32    maxdiffw,
+            l_int32    maxdiffh)
 {
 l_int32  wi, hi, wt, ht, delw, delh, idelx, idely, boolmatch;
 PIX     *pixt;
@@ -825,10 +838,10 @@ PIX     *pixt;
     wt = pixGetWidth(pix3);
     ht = pixGetHeight(pix3);
     delw = L_ABS(wi - wt);
-    if (delw > MAX_DIFF_WIDTH)
+    if (delw > maxdiffw)
         return FALSE;
     delh = L_ABS(hi - ht);
-    if (delh > MAX_DIFF_HEIGHT)
+    if (delh > maxdiffh)
         return FALSE;
 
         /* Round difference in centroid location to nearest integer;
@@ -879,6 +892,8 @@ PIX     *pixt;
  *              pix4   (exemplar pix, dilated)
  *              delx   (x comp of centroid difference)
  *              dely   (y comp of centroid difference)
+ *              maxdiffw (max width difference of pix1 and pix2)
+ *              maxdiffh (max height difference of pix1 and pix2)
  *              area1  (fg pixels in pix1)
  *              area3  (fg pixels in pix3)
  *              rank   (rank value of test, each way)
@@ -906,6 +921,8 @@ pixRankHaustest(PIX       *pix1,
                 PIX       *pix4,
                 l_float32  delx,   /* x(1) - x(3) */
                 l_float32  dely,   /* y(1) - y(3) */
+                l_int32    maxdiffw,
+                l_int32    maxdiffh,
                 l_int32    area1,
                 l_int32    area3,
                 l_float32  rank,
@@ -921,10 +938,10 @@ PIX     *pixt;
     wt = pixGetWidth(pix3);
     ht = pixGetHeight(pix3);
     delw = L_ABS(wi - wt);
-    if (delw > MAX_DIFF_WIDTH)
+    if (delw > maxdiffw)
         return FALSE;
     delh = L_ABS(hi - ht);
-    if (delh > MAX_DIFF_HEIGHT)
+    if (delh > maxdiffh)
         return FALSE;
 
         /* Upper bounds in remaining pixels for allowable match */
@@ -989,13 +1006,14 @@ jbClassifyCorrelation(JBCLASSER  *classer,
                       BOXA       *boxa,
                       PIXA       *pixas)
 {
-l_int32     n, nt, i, iclass, wt, ht, found, area, area1, area2, npages;
-l_int32    *sumtab;
-l_float32   x1, y1, x2, y2;
-l_float32   thresh, weight, threshold, score;
+l_int32     n, nt, i, iclass, wt, ht, found, area, area1, area2, npages,
+            overthreshold;
+l_int32    *sumtab, *centtab;
+l_uint32   *row, word;
+l_float32   x1, y1, x2, y2, xsum, ysum;
+l_float32   thresh, weight, threshold;
 BOX        *box;
 NUMA       *naclass, *napage;
-NUMA       *nafg;    /* fg area of all instances */
 NUMA       *nafgt;   /* fg area of all templates */
 NUMA       *naarea;   /* w * h area of all templates */
 JBFINDCTX  *findcontext;
@@ -1004,6 +1022,10 @@ PIX        *pix, *pix1, *pix2;
 PIXA       *pixa, *pixa1, *pixat;
 PIXAA      *pixaa;
 PTA        *pta, *ptac, *ptact;
+l_int32    *pixcts;  /* pixel counts of each pixa */
+l_int32   **pixrowcts;  /* row-by-row pixel counts of each pixa */
+l_int32     x, y, rowcount, downcount, wpl;
+l_uint8     byte;
 
     PROCNAME("jbClassifyCorrelation");
 
@@ -1028,22 +1050,68 @@ PTA        *pta, *ptac, *ptact;
         pixDestroy(&pix);
     }
 
-        /* Get the centroids of all the bordered images.
-         * These are relative to the UL corner of each (bordered) pix.  */
-    pta = pixaCentroids(pixa1);  /* centroids for this page; use here */
-    ptac = classer->ptac;  /* holds centroids of components up to this page */
-    ptaJoin(ptac, pta, 0, 0);  /* save centroids of all components */
-    ptact = classer->ptact;  /* holds centroids of templates */
-        
         /* Use these to save the class and page of each component. */
     naclass = classer->naclass;
     napage = classer->napage;
 
         /* Get the number of fg pixels in each component.  */
-    nafg = pixaCountPixels(pixa1);
     nafgt = classer->nafgt;    /* holds fg areas of the templates */
     sumtab = makePixelSumTab8();
 
+    pixcts = (l_int32 *)CALLOC(n, sizeof(*pixcts));
+    pixrowcts = (l_int32 **)CALLOC(n, sizeof(*pixrowcts));
+    centtab = makePixelCentroidTab8();
+    if (!pixcts || !pixrowcts || !centtab)
+        return ERROR_INT("calloc fail in pix*cts or centtab", procName, 1);
+
+        /* Count the "1" pixels in each row of the pix in pixa1; this
+         * allows pixCorrelationScoreThresholded to abort early if a match
+         * is impossible.  This loop merges three calculations: the total
+         * number of "1" pixels, the number of "1" pixels in each row, and
+         * the centroid.  The centroids are relative to the UL corner of
+         * each (bordered) pix.  The pixrowcts[i][y] are the total number
+         * of fg pixels in pixa[i] below row y. */
+    pta = ptaCreate(n);
+    for (i = 0; i < n; i++) {
+        pix = pixaGetPix(pixa1, i, L_CLONE);
+        pixrowcts[i] = (l_int32 *)CALLOC(pixGetHeight(pix),
+                                         sizeof(**pixrowcts));
+        xsum = 0;
+        ysum = 0;
+        wpl = pixGetWpl(pix);
+        row = pixGetData(pix) + (pixGetHeight(pix) - 1) * wpl;
+        downcount = 0;
+        for (y = pixGetHeight(pix) - 1; y >= 0; y--, row -= wpl) {
+            pixrowcts[i][y] = downcount;
+            rowcount = 0;
+            for (x = 0; x < wpl; x++) {
+                word = row[x];
+                byte = word & 0xff;
+                rowcount += sumtab[byte];
+                xsum += centtab[byte] + (x * 32 + 24) * sumtab[byte];
+                byte = (word >> 8) & 0xff;
+                rowcount += sumtab[byte];
+                xsum += centtab[byte] + (x * 32 + 16) * sumtab[byte];
+                byte = (word >> 16) & 0xff;
+                rowcount += sumtab[byte];
+                xsum += centtab[byte] + (x * 32 + 8) * sumtab[byte];
+                byte = (word >> 24) & 0xff;
+                rowcount += sumtab[byte];
+                xsum += centtab[byte] + x * 32 * sumtab[byte];
+            }
+            downcount += rowcount;
+            ysum += rowcount * y;
+        }
+        pixcts[i] = downcount;
+        ptaAddPt(pta,
+                 xsum / (l_float32)downcount, ysum / (l_float32)downcount);
+        pixDestroy(&pix);
+    }
+
+    ptac = classer->ptac;  /* holds centroids of components up to this page */
+    ptaJoin(ptac, pta, 0, 0);  /* save centroids of all components */
+    ptact = classer->ptact;  /* holds centroids of templates */
+
     /* Store the unbordered pix in a pixaa, in a hierarchical
      * set of arrays.  There is one pixa for each class,
      * and the pix in each pixa are all the instances found
@@ -1084,19 +1152,16 @@ PTA        *pta, *ptac, *ptact;
     nahash = classer->nahash;
     for (i = 0; i < n; i++) {
         pix1 = pixaGetPix(pixa1, i, L_CLONE);
-        numaGetIValue(nafg, i, &area1);
+        area1 = pixcts[i];
         ptaGetPt(pta, i, &x1, &y1);  /* centroid for this instance */
         nt = pixaGetCount(pixat);
         found = FALSE;
         findcontext = findSimilarSizedTemplatesInit(classer, pix1);
         while ( (iclass = findSimilarSizedTemplatesNext(findcontext)) > -1) {
-                /* Find score for this template */
+                /* Get the template */
             pix2 = pixaGetPix(pixat, iclass, L_CLONE);
             numaGetIValue(nafgt, iclass, &area2);
             ptaGetPt(ptact, iclass, &x2, &y2);  /* template centroid */
-            score = pixCorrelationScore(pix1, pix2, area1, area2,
-                        x1 - x2, y1 - y2, sumtab);
-            pixDestroy(&pix2);
 
                 /* Find threshold for this template */
             if (weight > 0.0) {
@@ -1106,7 +1171,45 @@ PTA        *pta, *ptac, *ptact;
             else
                 threshold = thresh;
 
-            if (score >= threshold) {  /* greedy match */
+                /* Find score for this template */
+            overthreshold = pixCorrelationScoreThresholded(pix1, pix2,
+                                                           area1, area2,
+                                                           x1 - x2, y1 - y2,
+                                                           MAX_DIFF_WIDTH,
+                                                           MAX_DIFF_HEIGHT,
+                                                           sumtab,
+                                                           pixrowcts[i],
+                                                           threshold);
+#if DEBUG_CORRELATION_SCORE
+            {
+                l_float32 score, testscore;
+                l_int32 count, testcount;
+                score = pixCorrelationScore(pix1, pix2, area1, area2,
+                                            x1 - x2, y1 - y2,
+                                            MAX_DIFF_WIDTH, MAX_DIFF_HEIGHT,
+                                            sumtab);
+
+                testscore = pixCorrelationScoreSimple(pix1, pix2, area1, area2,
+                                  x1 - x2, y1 - y2, MAX_DIFF_WIDTH,
+                                  MAX_DIFF_HEIGHT, sumtab);
+                count = (l_int32)rint(sqrt(score * area1 * area2));
+                testcount = (l_int32)rint(sqrt(testscore * area1 * area2));
+                if ((score >= threshold) != (testscore >= threshold)) {
+                    fprintf(stderr, "Correlation score mismatch: %d(%g,%d) vs %d(%g,%d) (%g)\n",
+                            count, score, score >= threshold,
+                            testcount, testscore, testscore >= threshold,
+                            score - testscore);
+                }
+
+                if ((score >= threshold) != overthreshold) {
+                    fprintf(stderr, "Mismatch between correlation/threshold comparison: %g(%g,%d) >= %g(%g) vs %s\n",
+                            score, score*area1*area2, count, threshold, threshold*area1*area2, (overthreshold ? "true" : "false"));
+                }
+            }
+#endif  /* DEBUG_CORRELATION_SCORE */
+            pixDestroy(&pix2);
+
+            if (overthreshold) {  /* greedy match */
                 found = TRUE;
                 numaAddNumber(naclass, iclass);
                 numaAddNumber(napage, npages);
@@ -1148,120 +1251,20 @@ PTA        *pta, *ptac, *ptact;
     }
     classer->nclass = pixaGetCount(pixat);
 
+    FREE(pixcts);
+    FREE(centtab);
+    for (i = 0; i < n; i++) {
+        FREE(pixrowcts[i]);
+    }
+    FREE(pixrowcts);
+
     FREE(sumtab);
     ptaDestroy(&pta);
     pixaDestroy(&pixa1);
-    numaDestroy(&nafg);
     return 0;
 }
 
 
-/*!
- *  pixCorrelationScore()
- *
- *      Input:  pix1   (test pix)
- *              pix2   (exemplar pix)
- *              area1  (number of on pixels in pix1)
- *              area2  (number of on pixels in pix2)
- *              delx   (x comp of centroid difference)
- *              dely   (y comp of centroid difference)
- *              tab    (sum tab for byte)
- *      Return: correlation score 
- *
- *  Note: we check first that the two pix are roughly the same size.
- *  Only if they meet that criterion do we compare the bitmaps. 
- *  The centroid difference is used to align the two images to the
- *  nearest integer for the correlation.
- *
- *  The correlation score is the ratio of the square of the number of
- *  pixels in the AND of the two bitmaps to the product of the number
- *  of ON pixels in each.  Denote the number of ON pixels in pix1
- *  by |1|, the number in pix2 by |2|, and the number in the AND
- *  of pix1 and pix2 by |1 & 2|.  The correlation score is then
- *  (|1 & 2|)**2 / (|1|*|2|).
- *
- *  This score is compared with an input threshold, which can
- *  be modified depending on the weight of the template.
- *  The modified threshold is
- *     thresh + (1.0 - thresh) * weight * R
- *  where 
- *     weight is a fixed input factor between 0.0 and 1.0
- *     R = |2| / area(2)
- *  and area(2) is the total number of pixels in 2 (i.e., width x height).
- *
- *  To understand why a weight factor is useful, consider what happens
- *  with thick, sans-serif characters that look similar and have a value
- *  of R near 1.  Different characters can have a high correlation value,
- *  and the classifier will make incorrect substitutions.  The weight
- *  factor raises the threshold for these characters.
- *
- *  Yet another approach to reduce such substitutions is to run the classifier
- *  in a non-greedy way, matching to the template with the highest
- *  score, not the first template with a score satisfying the matching
- *  constraint.  However, this is not particularly effective.
- */
-l_float32
-pixCorrelationScore(PIX       *pix1,
-                    PIX       *pix2,
-                    l_int32    area1,
-                    l_int32    area2,
-                    l_float32  delx,   /* x(1) - x(3) */
-                    l_float32  dely,   /* y(1) - y(3) */
-                    l_int32   *tab)
-{
-l_int32    wi, hi, wt, ht, delw, delh, idelx, idely, count;
-l_float32  score;
-PIX       *pixt;
-
-    PROCNAME("pixCorrelationScore");
-
-    if (!pix1)
-        return (l_float32)ERROR_FLOAT("pix1 not defined", procName, 0.0);
-    if (!pix2)
-        return (l_float32)ERROR_FLOAT("pix2 not defined", procName, 0.0);
-
-        /* eliminate based on size difference */
-    wi = pixGetWidth(pix1);
-    hi = pixGetHeight(pix1);
-    wt = pixGetWidth(pix2);
-    ht = pixGetHeight(pix2);
-    delw = L_ABS(wi - wt);
-    if (delw > MAX_DIFF_WIDTH)
-        return FALSE;
-    delh = L_ABS(hi - ht);
-    if (delh > MAX_DIFF_HEIGHT)
-        return FALSE;
-
-        /* round difference to nearest integer */
-    if (delx >= 0)
-        idelx = (l_int32)(delx + 0.5);
-    else
-        idelx = (l_int32)(delx - 0.5);
-    if (dely >= 0)
-        idely = (l_int32)(dely + 0.5);
-    else
-        idely = (l_int32)(dely - 0.5);
-
-        /*  pixt = pixAnd(NULL, pix1, pix2), including shift.
-         *  To insure that pixels are ON only within the
-         *  intersection of pix1 and the shifted pix2:
-         *  (1) Start with pixt cleared and equal in size to pix1.
-         *  (2) Blit the shifted pix2 onto pixt.  Then all ON pixels
-         *      are within the intersection of pix1 and the shifted pix2.
-         *  (3) AND pix1 with pixt. */
-    pixt = pixCreateTemplate(pix1);
-    pixRasterop(pixt, idelx, idely, wt, ht, PIX_SRC, pix2, 0, 0);
-    pixRasterop(pixt, 0, 0, wi, hi, PIX_SRC & PIX_DST, pix1, 0, 0);
-    pixCountPixels(pixt, &count, tab);
-    pixDestroy(&pixt);
-
-    score = (l_float32)(count * count) / (l_float32)(area1 * area2);
-/*    fprintf(stderr, "score = %7.3f, count = %d, area1 = %d, area2 = %d\n",
-             score, count, area1, area2); */
-    return score;
-}
-
-
 /*----------------------------------------------------------------------*
  *             Determine the image components we start with             *
  *----------------------------------------------------------------------*/
@@ -1290,12 +1293,14 @@ PIXA      *pixa, *pixat;
 
     PROCNAME("jbGetComponents");
 
-    if (!pixs)
-        return ERROR_INT("pixs not defined", procName, 1);
     if (!pboxad)
         return ERROR_INT("&boxad not defined", procName, 1);
+    *pboxad = NULL;
     if (!ppixad)
         return ERROR_INT("&pixad not defined", procName, 1);
+    *ppixad = NULL;
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (components != JB_CONN_COMPS && components != JB_CHARACTERS &&
         components != JB_WORDS)
         return ERROR_INT("invalid components", procName, 1);
@@ -1351,7 +1356,7 @@ PIXA      *pixa, *pixat;
             pixt1 = pixReduceRankBinaryCascade(pixs, 1, 1, 0, 0);
         }
 
-        pixt2 = pixWordMaskByDilation(pixt1, NULL);
+        pixt2 = pixWordMaskByDilation(pixt1, 0, NULL);
 
             /* Expand the optimally dilated word mask to full res. */
         pixt3 = pixExpandReplicate(pixt2, redfactor);
@@ -1375,10 +1380,10 @@ PIXA      *pixa, *pixat;
     }
 
         /* Remove large components, and save the results.  */
-    *ppixad = pixaRemoveLargeComponents(pixa, maxwidth, maxheight,
-                                        L_REMOVE_IF_EITHER, NULL);
-    *pboxad = boxaRemoveLargeComponents(boxa, maxwidth, maxheight,
-                                        L_REMOVE_IF_EITHER, NULL);
+    *ppixad = pixaSelectBySize(pixa, maxwidth, maxheight, L_SELECT_IF_BOTH,
+                               L_SELECT_IF_LTE, NULL);
+    *pboxad = boxaSelectBySize(boxa, maxwidth, maxheight, L_SELECT_IF_BOTH,
+                               L_SELECT_IF_LTE, NULL);
     pixaDestroy(&pixa);
     boxaDestroy(&boxa);
 
@@ -1390,22 +1395,29 @@ PIXA      *pixa, *pixat;
  *  pixWordMaskByDilation()
  *
  *      Input:  pixs (1 bpp; typ. at 75 to 150 ppi)
+ *              maxsize (use 0 for default; not to exceed 14)
  *              &size (<optional return> size of optimal horiz Sel)
  *      Return: pixd (dilated word mask), or null on error
  *
  *  Notes:
- *      (1) In practice, one can usually stop dilating after 7 dilations,
- *          so we cut it off there explicitly.
+ *      (1) For 75 to 150 ppi, the optimal dilation should not exceed 7.
+ *          This is the default size chosen if maxsize <= 0.
+ *      (2) To run this on images at resolution between 200 and 300, it
+ *          is advisable to use a larger maxsize, say between 10 and 14.
+ *      (3) The best size for dilating to get word masks is optionally returned.
  */
 PIX *
 pixWordMaskByDilation(PIX      *pixs,
+                      l_int32   maxsize,
                       l_int32  *psize)
 {
 l_int32  i, diffmin, ndiff, imin;
-l_int32  ncc[12];
+l_int32  ncc[MAX_ALLOWED_DILATION + 1];
 BOXA    *boxa;
-PIX     *pixt1, *pixt2;
+NUMA    *nacc;
+PIX     *pixt1, *pixt2, *pixt3;
 PIXA    *pixa;
+SEL     *sel;
 
     PROCNAME("pixWordMaskbyDilation");
 
@@ -1424,16 +1436,24 @@ PIXA    *pixa;
     pixt1 = pixCopy(NULL, pixs);
     pixaAddPix(pixa, pixt1, L_COPY); 
 
-    for (i = 0; i < 7; i++) {
+    if (maxsize <= 0)
+        maxsize = 7;  /* default */
+    if (maxsize > MAX_ALLOWED_DILATION)
+        maxsize = MAX_ALLOWED_DILATION;
+    nacc = numaCreate(maxsize);
+    for (i = 0; i <= maxsize; i++) {
         if (i == 0)     /* first one not dilated */
             pixt2 = pixCopy(NULL, pixt1); 
         else    /* successive dilation by sel_2h */
             pixt2 = pixMorphSequence(pixt1, "d2.1", 0);
         boxa = pixConnCompBB(pixt2, 4);
         ncc[i] = boxaGetCount(boxa);
+        numaAddNumber(nacc, ncc[i]);
         if (i > 0) {
             ndiff = ncc[i - 1] - ncc[i];
-/*          fprintf(stderr, "ndiff[%d] = %d\n", i - 1, ndiff); */
+#if  DEBUG_PLOT_CC
+            fprintf(stderr, "ndiff[%d] = %d\n", i - 1, ndiff);
+#endif  /* DEBUG_PLOT_CC */
             if (ndiff < diffmin) {
                 imin = i;
                 diffmin = ndiff;
@@ -1446,13 +1466,32 @@ PIXA    *pixa;
     }
     pixDestroy(&pixt1);
 
+#if  DEBUG_PLOT_CC
+    {GPLOT *gplot;
+     NUMA  *naseq;
+        naseq = numaMakeSequence(1, 1, numaGetCount(nacc));
+        gplot = gplotCreate("junk_cc_output", GPLOT_PNG,
+                            "Number of cc vs. horizontal dilation",
+                            "Sel horiz", "Number of cc");
+        gplotAddPlot(gplot, naseq, nacc, GPLOT_LINES, "");
+        gplotMakeOutput(gplot);
+        gplotDestroy(&gplot);
+        numaDestroy(&naseq);
+    }
+#endif  /* DEBUG_PLOT_CC */
+
         /* Save the result of the optimal dilation */
     pixt2 = pixaGetPix(pixa, imin, L_CLONE);
+    sel = selCreateBrick(1, imin, 0, imin - 1, SEL_HIT);
+    pixt3 = pixErode(NULL, pixt2, sel);  /* remove effect of dilation */
+    selDestroy(&sel);
+    pixDestroy(&pixt2);
     pixaDestroy(&pixa);
     if (psize)
         *psize = imin + 1;
 
-    return pixt2;
+    numaDestroy(&nacc);
+    return pixt3;
 }
 
 
@@ -1464,7 +1503,7 @@ PIXA    *pixa;
  *
  *      Input:  pixaa (one pixa for each class)
  *              &pna (<return> number of samples used to build each composite)
- *              &ptad (<return> centroids of bordered composites)
+ *              &ptat (<return> centroids of bordered composites)
  *      Return: pixad (accumulated sum of samples in each class),
  *                     or null on error
  *
@@ -1483,12 +1522,14 @@ PTA       *ptat, *pta;
 
     PROCNAME("jbAccumulateComposites");
 
-    if (!pixaa)
-        return (PIXA *)ERROR_PTR("pixaa not defined", procName, NULL);
     if (!pptat)
         return (PIXA *)ERROR_PTR("&ptat not defined", procName, NULL);
+    *pptat = NULL;
     if (!pna)
         return (PIXA *)ERROR_PTR("&na not defined", procName, NULL);
+    *pna = NULL;
+    if (!pixaa)
+        return (PIXA *)ERROR_PTR("pixaa not defined", procName, NULL);
 
     n = pixaaGetCount(pixaa);
     if ((ptat = ptaCreate(n)) == NULL)
@@ -1757,7 +1798,8 @@ JBDATA  *data;
  *              jbdata
  *      Return: 0 if OK, 1 on error
  *
- *  Note: Writes data in jbdata to file.
+ *  Notes:
+ *      (1) Serialization function that writes data in jbdata to file.
  */
 l_int32
 jbDataWrite(const char  *rootout,
@@ -1835,6 +1877,9 @@ SARRAY   *sa;
     
     PROCNAME("jbDataRead");
 
+    if (!rootname)
+        return (JBDATA *)ERROR_PTR("rootname not defined", procName, NULL);
+
     snprintf(fname, L_BUF_SIZE, "%s%s", rootname, JB_TEMPLATE_EXT);
     if ((pixs = pixRead(fname)) == NULL)
         return (JBDATA *)ERROR_PTR("pix not read", procName, NULL);
@@ -1905,7 +1950,7 @@ SARRAY   *sa;
  *  jbDataRender()
  *
  *      Input:  jbdata
- *              debugflag (if TRUE, writes into 4 bpp pix and adds 
+ *              debugflag (if TRUE, writes into 2 bpp pix and adds 
  *                         component outlines in color)
  *      Return: pixa (reconstruction of original images, using templates) or
  *              null on error
@@ -1926,6 +1971,9 @@ PTA      *ptaul;
     
     PROCNAME("jbDataRender");
 
+    if (!data)
+        return (PIXA *)ERROR_PTR("data not defined", procName, NULL);
+
     npages = data->npages;
     w = data->w;
     h = data->h;
@@ -1993,7 +2041,8 @@ PTA      *ptaul;
  *  jbGetULCorners()
  *
  *      Input:  jbclasser
- *              boxa  (of c.c. bounding rectangles for this page)
+ *              pixs (full res image)
+ *              boxa (of c.c. bounding rectangles for this page)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -2028,6 +2077,8 @@ PTA       *ptac, *ptact, *ptaul;
 
     if (!classer)
         return ERROR_INT("classer not defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
     if (!boxa)
         return ERROR_INT("boxa not defined", procName, 1);
 
@@ -2144,7 +2195,7 @@ PTA       *ptaul, *ptall;
  *----------------------------------------------------------------------*/
 /* When looking for similar matches we check templates whose size is +/- 2 in
  * each direction. This involves 25 possible sizes. This array contains the
- * offsets for each of those positions in a spirial pattern. There are 25 pairs
+ * offsets for each of those positions in a spiral pattern. There are 25 pairs
  * of numbers in this array: even positions are x values. */
 static int two_by_two_walk[50] = {
   0, 0,
@@ -2357,5 +2408,3 @@ BOX     *box;
     *pdy = miny;
     return 0;
 }
-
-
