@@ -22,10 +22,14 @@
  *      useful in applications.  Most are morphological in
  *      nature.
  *
- *      Selective morph sequence operation on each component          *
+ *      Selective morph sequence operation on each component
  *            PIX     *pixMorphSequenceByComponent()
  *            PIXA    *pixaMorphSequenceByComponent()
  *
+ *      Selective morph sequence operation on each region
+ *            PIX     *pixMorphSequenceByRegion()
+ *            PIXA    *pixaMorphSequenceByRegion()
+ *
  *      Selective connected component filling
  *            PIX     *pixSelectiveConnCompFill()
  *
@@ -63,7 +67,7 @@
 /*!
  *  pixMorphSequenceByComponent()
  *
- *      Input:  pixs
+ *      Input:  pixs (1 bpp)
  *              sequence (string specifying sequence)
  *              connectivity (4 or 8)
  *              minw  (minimum width to consider; use 0 or 1 for any width)
@@ -87,7 +91,7 @@ pixMorphSequenceByComponent(PIX         *pixs,
                             l_int32      connectivity,
                             l_int32      minw,
                             l_int32      minh,
-			    BOXA       **pboxa)
+                            BOXA       **pboxa)
 {
 l_int32  n, i, x, y, w, h;
 BOXA    *boxa;
@@ -120,7 +124,7 @@ PIXA    *pixas, *pixad;
     n = pixaGetCount(pixad);
     for (i = 0; i < n; i++) {
         pixaGetBoxGeometry(pixad, i, &x, &y, &w, &h);
-	pix = pixaGetPix(pixad, i, L_CLONE);
+        pix = pixaGetPix(pixad, i, L_CLONE);
         pixRasterop(pixd, x, y, w, h, PIX_PAINT, pix, 0, 0);
         pixDestroy(&pix);
     }
@@ -184,11 +188,11 @@ PIXA    *pixad;
         if (w >= minw && h >= minh) {
             if ((pixt1 = pixaGetPix(pixas, i, L_CLONE)) == NULL)
                 return (PIXA *)ERROR_PTR("pixt1 not found", procName, NULL);
-            if ((pixt2 = pixMorphSequence(pixt1, sequence, 0)) == NULL)
+            if ((pixt2 = pixMorphCompSequence(pixt1, sequence, 0)) == NULL)
                 return (PIXA *)ERROR_PTR("pixt2 not made", procName, NULL);
-	    pixaAddPix(pixad, pixt2, L_INSERT);
-	    box = pixaGetBox(pixas, i, L_COPY);
-	    pixaAddBox(pixad, box, L_INSERT);
+            pixaAddPix(pixad, pixt2, L_INSERT);
+            box = pixaGetBox(pixas, i, L_COPY);
+            pixaAddBox(pixad, box, L_INSERT);
             pixDestroy(&pixt1);
         }
     }
@@ -197,6 +201,168 @@ PIXA    *pixad;
 }
 
 
+/*-----------------------------------------------------------------*
+ *              Morph sequence operation on each region            *
+ *-----------------------------------------------------------------*/
+/*!
+ *  pixMorphSequenceByRegion()
+ *
+ *      Input:  pixs (1 bpp)
+ *              pixm (mask specifying regions)
+ *              sequence (string specifying sequence)
+ *              connectivity (4 or 8, used on mask)
+ *              minw  (minimum width to consider; use 0 or 1 for any width)
+ *              minh  (minimum height to consider; use 0 or 1 for any height)
+ *              &boxa (<optional> return boxa of c.c. in pixm)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) See pixMorphCompSequence() for composing operation sequences.
+ *      (2) This operates separately on the region in pixs corresponding
+ *          to each c.c. in the mask pixm.
+ *      (3) Dilation will NOT increase the region size; the result
+ *          is clipped to the size of the mask region.  This is necessary
+ *          to make regions independent after the operation.
+ *      (4) You can specify that the width and/or height of a region must
+ *          equal or exceed a minimum size for the operation to take place.
+ *      (5) Use NULL for @pboxa to avoid returning the boxa.
+ */
+PIX *
+pixMorphSequenceByRegion(PIX         *pixs,
+                         PIX         *pixm,
+                         const char  *sequence,
+                         l_int32      connectivity,
+                         l_int32      minw,
+                         l_int32      minh,
+                         BOXA       **pboxa)
+{
+l_int32  n, i, x, y, w, h;
+BOXA    *boxa;
+PIX     *pix, *pixd;
+PIXA    *pixam, *pixad;
+
+    PROCNAME("pixMorphSequenceByRegion");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (!pixm)
+        return (PIX *)ERROR_PTR("pixm not defined", procName, NULL);
+    if (pixGetDepth(pixs) != 1 || pixGetDepth(pixm) != 1)
+        return (PIX *)ERROR_PTR("pixs and pixm not both 1 bpp", procName, NULL);
+    if (!sequence)
+        return (PIX *)ERROR_PTR("sequence not defined", procName, NULL);
+
+    if (minw <= 0) minw = 1;
+    if (minh <= 0) minh = 1;
+
+        /* Get the c.c. of the mask */
+    if ((boxa = pixConnComp(pixm, &pixam, connectivity)) == NULL)
+        return (PIX *)ERROR_PTR("boxa not made", procName, NULL);
+
+        /* Operate on each region in pixs independently */
+    pixad = pixaMorphSequenceByRegion(pixs, pixam, sequence, minw, minh);
+    pixaDestroy(&pixam);
+    boxaDestroy(&boxa);
+    if (!pixad)
+        return (PIX *)ERROR_PTR("pixad not made", procName, NULL);
+
+        /* Display the result out into pixd */
+    pixd = pixCreateTemplate(pixs);
+    n = pixaGetCount(pixad);
+    for (i = 0; i < n; i++) {
+        pixaGetBoxGeometry(pixad, i, &x, &y, &w, &h);
+        pix = pixaGetPix(pixad, i, L_CLONE);
+        pixRasterop(pixd, x, y, w, h, PIX_PAINT, pix, 0, 0);
+        pixDestroy(&pix);
+    }
+
+    if (pboxa)
+        *pboxa = pixaGetBoxa(pixad, L_CLONE);
+    pixaDestroy(&pixad);
+    return pixd;
+}
+
+
+/*!
+ *  pixaMorphSequenceByComponent()
+ *
+ *      Input:  pixs (1 bpp)
+ *              pixam (of 1 bpp mask elements)
+ *              sequence (string specifying sequence)
+ *              minw  (minimum width to consider; use 0 or 1 for any width)
+ *              minh  (minimum height to consider; use 0 or 1 for any height)
+ *      Return: pixad, or null on error
+ *
+ *  Notes:
+ *      (1) See pixMorphSequence() for composing operation sequences.
+ *      (2) This operates separately on each region in the input pixs
+ *          defined by the components in pixam.
+ *      (3) You can specify that the width and/or height of a mask
+ *          component must equal or exceed a minimum size for the
+ *          operation to take place.
+ *      (4) The input pixam should have a boxa giving the locations
+ *          of the regions in pixs.
+ */
+PIXA *
+pixaMorphSequenceByRegion(PIX         *pixs,
+                          PIXA        *pixam,
+                          const char  *sequence,
+                          l_int32      minw,
+                          l_int32      minh)
+{
+l_int32  n, i, w, h, d;
+BOX     *box;
+PIX     *pixt1, *pixt2, *pixt3;
+PIXA    *pixad;
+
+    PROCNAME("pixaMorphSequenceByRegion");
+
+    if (!pixs)
+        return (PIXA *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixGetDepth(pixs) != 1)
+        return (PIXA *)ERROR_PTR("pixs not  bpp", procName, NULL);
+    if (!pixam)
+        return (PIXA *)ERROR_PTR("pixam not defined", procName, NULL);
+    pixaGetPixDimensions(pixam, 0, NULL, NULL, &d);
+    if (d != 1)
+        return (PIXA *)ERROR_PTR("mask depth not 1 bpp", procName, NULL);
+    if ((n = pixaGetCount(pixam)) == 0)
+        return (PIXA *)ERROR_PTR("no regions specified", procName, NULL);
+    if (n != pixaGetBoxaCount(pixam))
+        L_WARNING("boxa size != n", procName);
+    if (!sequence)
+        return (PIXA *)ERROR_PTR("sequence not defined", procName, NULL);
+
+    if (minw <= 0) minw = 1;
+    if (minh <= 0) minh = 1;
+
+    if ((pixad = pixaCreate(n)) == NULL)
+        return (PIXA *)ERROR_PTR("pixad not made", procName, NULL);
+
+        /* Use the rectangle to remove the appropriate part of pixs;
+         * then AND with the mask component to get the actual fg
+         * of pixs that is under the mask component. */
+    for (i = 0; i < n; i++) {
+        pixaGetPixDimensions(pixam, i, &w, &h, NULL);
+        if (w >= minw && h >= minh) {
+            if ((pixt1 = pixaGetPix(pixam, i, L_CLONE)) == NULL)
+                return (PIXA *)ERROR_PTR("pixt1 not found", procName, NULL);
+            box = pixaGetBox(pixam, i, L_COPY);
+            pixt2 = pixClipRectangle(pixs, box, NULL);
+            pixAnd(pixt2, pixt2, pixt1);
+            if ((pixt3 = pixMorphCompSequence(pixt2, sequence, 0)) == NULL)
+                return (PIXA *)ERROR_PTR("pixt3 not made", procName, NULL);
+            pixaAddPix(pixad, pixt3, L_INSERT);
+            pixaAddBox(pixad, box, L_INSERT);
+            pixDestroy(&pixt1);
+            pixDestroy(&pixt2);
+        }
+    }
+
+    return pixad;
+}
+
+
 /*-----------------------------------------------------------------*
  *             Selective connected component filling               *
  *-----------------------------------------------------------------*/
@@ -610,7 +776,7 @@ SEL       *sel_2a;
     if ((pixt3 = pixCreateTemplate(pixs)) == NULL)
         return (NUMA *)ERROR_PTR("pix3 not made", procName, NULL);
 
-        /* get pixel counts at different stages of erosion */
+        /* Get pixel counts at different stages of erosion */
     pixCountPixels(pixt1, &count, NULL);
     numaAddNumber(na, count);
     pixErode(pixt2, pixt1, sel_2a);
@@ -623,17 +789,15 @@ SEL       *sel_2a;
         pixErode(pixt2, pixt3, sel_2a);
         pixCountPixels(pixt2, &count, NULL);
         numaAddNumber(na, count);
-/*        if (i == 4) pixWrite("junkoutt", pixt2, IFF_PNG); */
     }
 
-        /* compute length histogram */
+        /* Compute length histogram */
     if ((nah = numaCreate(na->n)) == NULL)
         return (NUMA *)ERROR_PTR("nah not made", procName, NULL);
     numaAddNumber(nah, 0); /* number at length 0 */
     for (i = 1; i < na->n - 1; i++) {
         val = na->array[i+1] - 2 * na->array[i] + na->array[i-1];
         numaAddNumber(nah, val);
-/*        fprintf(stderr, "i = %d, val = %f\n", i, nah->array[i]); */
     }
 
     pixDestroy(&pixt1);
@@ -721,10 +885,10 @@ PIX  *pixd;
 /*!
  *  pixHDome()
  *
- *      Input:  pixs
- *              height (of hdome; must be >= 0)
+ *      Input:  pixs (8 bpp, filling mask)
+ *              height (of seed below the filling maskhdome; must be >= 0)
  *              connectivity (4 or 8)
- *      Return: pixd, or null on error
+ *      Return: pixd (8 bpp), or null on error
  *
  *  Notes:
  *      (1) It is more efficient to use a connectivity of 4 for the fill.
@@ -733,9 +897,14 @@ PIX  *pixd;
  *          not exceeding the size of the structuring element used
  *          in the opening or closing, rsp.  The height of the peak is
  *          irrelevant.  By contrast, for the HDome, the gray seedfill
- *          is be used to extract all peaks that have a height not exceeding
+ *          is used to extract all peaks that have a height not exceeding
  *          a given value, regardless of their width!
- *      (3) The method: the filling mask, pixs, is the image whose peaks
+ *      (3) Slightly more precisely, suppose you set 'height' = 40.
+ *          Then all bumps in pixs with a height greater than or equal
+ *          to 40 become, in pixd, bumps with a max value of exactly 40.
+ *          All shorter bumps have a max value in pixd equal to the height
+ *          of the bump.
+ *      (4) The method: the filling mask, pixs, is the image whose peaks
  *          are to be extracted.  The height of a peak is the distance
  *          between the top of the peak and the highest "leak" to the
  *          outside -- think of a sombrero, where the leak occurs
@@ -753,6 +922,8 @@ PIX  *pixd;
  *            (c) Subtract the filled seed (pixd) from the filling mask (pixs).
  *          Note that in this procedure, everything is done starting
  *          with the filling mask, pixs.
+ *      (5) For segmentation, the resulting image, pixd, can be thresholded
+ *          and used as a seed for another filling operation.
  */
 PIX *
 pixHDome(PIX     *pixs,
@@ -894,11 +1065,11 @@ PIX  *pixg, *pixd;
         vsize++;
     }
 
-        /* optionally smooth first to remove noise.
+        /* Optionally smooth first to remove noise.
          * If smoothing is 0, just get a copy */
     pixg = pixBlockconvGray(pixs, NULL, smoothing, smoothing);
 
-        /* this gives approximately the gradient of a transition */
+        /* This gives approximately the gradient of a transition */
     pixd = pixDilateGray(pixg, hsize, vsize);
     pixSubtractGray(pixd, pixd, pixg);
     pixDestroy(&pixg);
@@ -923,11 +1094,15 @@ PIX  *pixg, *pixd;
 PTA *
 pixaCentroids(PIXA  *pixa)
 {
-l_int32    d, i, j, k, n, w, h, wpl, pixsum, val;
+l_int32    d, i, j, k, n, w, h, wpl, pixsum, rowsum, val;
 l_float32  xsum, ysum, xave, yave;
 l_uint32  *data, *line;
+l_uint32   word;
+l_uint8    byte;
 PIX       *pix;
 PTA       *pta;
+static l_int32 *centtab = NULL;
+static l_int32 *sumtab = NULL;
 
     PROCNAME("pixaCentroids");
 
@@ -944,6 +1119,14 @@ PTA       *pta;
     if ((pta = ptaCreate(n)) == NULL)
         return (PTA *)ERROR_PTR("pta not defined", procName, NULL);
 
+    if ((centtab == NULL) &&
+        ((centtab = makePixelCentroidTab8()) == NULL))
+        return (PTA *)ERROR_PTR("couldn't make centtab", procName, NULL);
+
+    if ((sumtab == NULL) &&
+        ((sumtab = makePixelSumTab8()) == NULL))
+        return (PTA *)ERROR_PTR("couldn't make sumtab", procName, NULL);
+
     for (k = 0; k < n; k++) {
         pix = pixaGetPix(pixa, k, L_CLONE);
         w = pixGetWidth(pix);
@@ -954,14 +1137,41 @@ PTA       *pta;
         pixsum = 0;
         if (d == 1) {
             for (i = 0; i < h; i++) {
+                    /* The body of this loop computes the sum of the set
+                     * (1) bits on this row, weighted by their distance
+                     * from the left edge of pix, and accumulates that into
+                     * xsum; it accumulates their distance from the top
+                     * edge of pix into ysum, and their total count into
+                     * pixsum.  It's equivalent to
+                     * for (j = 0; j < w; j++) {
+                     *     if (GET_DATA_BIT(line, j)) {
+                     *         xsum += j;
+                     *         ysum += i;
+                     *         pixsum++;
+                     *     }
+                     * }
+                     */
                 line = data + wpl * i;
-                for (j = 0; j < w; j++) {
-                    if (GET_DATA_BIT(line, j)) {
-                        xsum += j;
-                        ysum += i;
-                        pixsum++;
+                rowsum = 0;
+                for (j = 0; j < wpl; j++) {
+                    word = line[j];
+                    if (word) {
+                        byte = word & 0xff;
+                        rowsum += sumtab[byte];
+                        xsum += centtab[byte] + (j * 32 + 24) * sumtab[byte];
+                        byte = (word >> 8) & 0xff;
+                        rowsum += sumtab[byte];
+                        xsum += centtab[byte] + (j * 32 + 16) * sumtab[byte];
+                        byte = (word >> 16) & 0xff;
+                        rowsum += sumtab[byte];
+                        xsum += centtab[byte] + (j * 32 + 8) * sumtab[byte];
+                        byte = (word >> 24) & 0xff;
+                        rowsum += sumtab[byte];
+                        xsum += centtab[byte] + j * 32 * sumtab[byte];
                     }
                 }
+                pixsum += rowsum;
+                ysum += rowsum * i;
             }
             if (pixsum == 0) {
                 L_WARNING("no ON pixels in pix", procName);
@@ -998,4 +1208,3 @@ PTA       *pta;
 
     return pta;
 }
-
