@@ -18,6 +18,7 @@
  *
  *      Numa creation, destruction, copy, clone, etc.
  *          NUMA        *numaCreate()
+ *          NUMA        *numaCreateFromIArray()
  *          void        *numaDestroy()
  *          NUMA        *numaCopy()
  *          NUMA        *numaClone()
@@ -28,9 +29,11 @@
  *          l_int32      numaExtendArray()
  *          l_int32      numaInsertNumber()
  *          l_int32      numaRemoveNumber()
+ *          l_int32      numaReplaceNumber()
  *
  *      Numa accessors
  *          l_int32      numaGetCount()
+ *          l_int32      numaSetCount()
  *          l_int32      numaGetIValue()
  *          l_int32      numaGetFValue()
  *          l_int32      numaSetValue()
@@ -38,6 +41,8 @@
  *          l_float32   *numaGetFArray()
  *          l_int32      numaGetRefcount()
  *          l_int32      numaChangeRefcount()
+ *          l_int32      numaGetXParameters()
+ *          l_int32      numaSetXParameters()
  *
  *      Serialize numa for I/O
  *          l_int32      numaRead()
@@ -60,6 +65,12 @@
  *          NUMA        *numaaReplaceNuma()
  *          l_int32      numaaAddNumber()
  *       
+ *      Serialize numaa for I/O
+ *          l_int32      numaaRead()
+ *          l_int32      numaaReadStream()
+ *          l_int32      numaaWrite()
+ *          l_int32      numaaWriteStream()
+ *
  *      Numa2d creation, destruction
  *          NUMA2D      *numa2dCreate()
  *          void        *numa2dDestroy()
@@ -110,6 +121,15 @@
  *        precision for large integers, whereas you must cast (l_int32)
  *        to go from l_float32 --> l_int32 because you're truncating
  *        to the integer value.
+ *
+ *    (5) In situations where the data in a numa correspond to a function,
+ *        y(x), the values can be either at equal spacings in x or at
+ *        arbitrary spacings.  For the former, we can represent all x values
+ *        by two parameters: startx (corresponding to y[0]) and delx
+ *        for the change in x for adjacent values y[i] and y[i+1].
+ *        startx and delx are initialized to 0.0 and 1.0, rsp.
+ *        For arbitrary spacings, we use a second numa, and the two
+ *        numas are typically denoted nay and nax.
  */
 
 #include <stdio.h>
@@ -148,6 +168,39 @@ NUMA  *na;
     na->nalloc = n;
     na->n = 0;
     na->refcount = 1;
+    na->startx = 0.0;
+    na->delx = 1.0;
+
+    return na;
+}
+
+
+/*!
+ *  numaCreateFromIArray()
+ *
+ *      Input:  int array
+ *              size (of the array)
+ *      Return: na, or null on error
+ *
+ *  Notes:
+ *      (1) This just copies the data from the int array into the numa.
+ *      (2) The input array is NOT owned by the numa.
+ */
+NUMA *
+numaCreateFromIArray(l_int32  *array,
+                     l_int32   size)
+{
+l_int32  i;
+NUMA    *na;
+
+    PROCNAME("numaCreateFromIArray");
+
+    if (!array)
+        return (NUMA *)ERROR_PTR("array not defined", procName, NULL);
+
+    na = numaCreate(size);
+    for (i = 0; i < size; i++)
+        numaAddNumber(na, array[i]);
 
     return na;
 }
@@ -210,6 +263,8 @@ NUMA    *cna;
 
     if ((cna = numaCreate(na->nalloc)) == NULL)
         return (NUMA *)ERROR_PTR("cna not made", procName, NULL);
+    cna->startx = na->startx;
+    cna->delx = na->delx;
 
     for (i = 0; i < na->n; i++)
         numaAddNumber(cna, na->array[i]);
@@ -227,7 +282,6 @@ NUMA    *cna;
 NUMA *
 numaClone(NUMA  *na)
 {
-
     PROCNAME("numaClone");
 
     if (!na)
@@ -252,7 +306,6 @@ numaClone(NUMA  *na)
 l_int32
 numaEmpty(NUMA  *na)
 {
-
     PROCNAME("numaEmpty");
 
     if (!na)
@@ -303,7 +356,6 @@ l_int32  n;
 l_int32
 numaExtendArray(NUMA  *na)
 {
-
     PROCNAME("numaExtendArray");
 
     if (!na)
@@ -392,6 +444,34 @@ l_int32  i, n;
 }
 
 
+/*!
+ *  numaReplaceNumber()
+ *
+ *      Input:  na
+ *              index (element to be replaced)
+ *              val (new value to replace old one)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+numaReplaceNumber(NUMA      *na,
+                  l_int32    index,
+                  l_float32  val)
+{
+l_int32  n;
+
+    PROCNAME("numaReplaceNumber");
+
+    if (!na)
+        return ERROR_INT("na not defined", procName, 1);
+    n = numaGetCount(na);
+    if (index < 0 || index >= n)
+        return ERROR_INT("index not in {0...n - 1}", procName, 1);
+
+    na->array[index] = val;
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
  *                            Numa accessors                            *
  *----------------------------------------------------------------------*/
@@ -404,7 +484,6 @@ l_int32  i, n;
 l_int32
 numaGetCount(NUMA  *na)
 {
-
     PROCNAME("numaGetCount");
 
     if (!na)
@@ -413,6 +492,40 @@ numaGetCount(NUMA  *na)
 }
         
 
+/*!
+ *  numaSetCount()
+ *
+ *      Input:  na
+ *              newcount
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) If newcount <= na->nalloc, this resets na->n.
+ *          Using newcount = 0 is equivalent to numaEmpty().
+ *      (2) If newcount > na->nalloc, this causes a realloc
+ *          to a size na->nalloc = newcount.
+ *      (3) All the previously unused values in na are set to 0.0.
+ */
+l_int32
+numaSetCount(NUMA    *na,
+             l_int32  newcount)
+{
+    PROCNAME("numaSetCount");
+
+    if (!na)
+        return ERROR_INT("na not defined", procName, 1);
+    if (newcount > na->nalloc) {
+        if ((na->array = (l_float32 *)reallocNew((void **)&na->array,
+                         sizeof(l_float32) * na->nalloc,
+                         sizeof(l_float32) * newcount)) == NULL)
+            return ERROR_INT("new ptr array not returned", procName, 1);
+        na->nalloc = newcount;
+    }
+    na->n = newcount;
+    return 0;
+}
+        
+
 /*!
  *  numaGetFValue()
  *
@@ -484,7 +597,6 @@ numaSetValue(NUMA      *na,
              l_int32    index,
              l_float32  val)
 {
-
     PROCNAME("numaSetValue");
 
     if (!na)
@@ -505,7 +617,11 @@ numaSetValue(NUMA      *na,
  *              by rounding, or null on error
  *  Notes:
  *      (1) A copy is always made, because we need to generate an
- *          integer array from the bare float array.
+ *          integer array from the bare float array.  The caller
+ *          is responsible for freeing the array.
+ *      (2) This function is provided to simplify calculations
+ *          using the bare internal array (rather than continually
+ *          calling accessors on the numa).
  */
 l_int32 *
 numaGetIArray(NUMA  *na)
@@ -535,11 +651,19 @@ l_int32  *array;
  *
  *      Input:  na
  *              copyflag (L_NOCOPY or L_COPY)
- *      Return: the bare internal array, or null on error
+ *      Return: either the bare internal array or a copy of it,
+ *              or null on error
  *
  *  Notes:
  *      (1) If copyflag == L_COPY, it makes a copy which the caller
- *          is responsible for destroying.
+ *          is responsible for freeing.  Otherwise, it operates
+ *          directly on the bare array of the numa.
+ *      (2) Very important: for L_NOCOPY, any writes to the array
+ *          will be in the numa.  Do not write beyond the size of
+ *          the count field, because it will not be accessable
+ *          from the numa!  If necessary, be sure to set the count
+ *          the count field to a larger number (such as the alloc
+ *          size) BEFORE calling this function.
  */
 l_float32 *
 numaGetFArray(NUMA    *na,
@@ -553,10 +677,10 @@ l_float32  *array;
     if (!na)
         return (l_float32 *)ERROR_PTR("na not defined", procName, NULL);
 
-    n = numaGetCount(na);
     if (copyflag == L_NOCOPY)
         array = na->array;
     else {  /* copyflag == L_COPY */
+        n = numaGetCount(na);
         if ((array = (l_float32 *)CALLOC(n, sizeof(l_float32))) == NULL)
             return (l_float32 *)ERROR_PTR("array not made", procName, NULL);
         for (i = 0; i < n; i++)
@@ -604,8 +728,58 @@ numaChangeRefcount(NUMA    *na,
 }
 
 
+/*!
+ *  numaGetXParameters()
+ *
+ *      Input:  na
+ *              &startx (<optional return> startx)
+ *              &delx (<optional return> delx)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+numaGetXParameters(NUMA       *na,
+                   l_float32  *pstartx,
+                   l_float32  *pdelx)
+{
+    PROCNAME("numaGetXParameters");
+
+    if (!na)
+        return ERROR_INT("na not defined", procName, 1);
+
+    if (pstartx) *pstartx = na->startx;
+    if (pdelx) *pdelx = na->delx;
+    return 0;
+}
+
+
+/*!
+ *  numaSetXParameters()
+ *
+ *      Input:  na
+ *              startx (x value corresponding to na[0])
+ *              delx (difference in x values for the situation where the
+ *                    elements of na correspond to the evaulation of a
+ *                    function at equal intervals of size @delx)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+numaSetXParameters(NUMA      *na,
+                   l_float32  startx,
+                   l_float32  delx)
+{
+    PROCNAME("numaSetXParameters");
+
+    if (!na)
+        return ERROR_INT("na not defined", procName, 1);
+
+    na->startx = startx;
+    na->delx = delx;
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
- *                        Serialize for I/O                             *
+ *                       Serialize numa for I/O                         *
  *----------------------------------------------------------------------*/
 /*!
  *  numaRead()
@@ -647,7 +821,7 @@ NUMA *
 numaReadStream(FILE  *fp)
 {
 l_int32    i, n, index, ret, version;
-l_float32  val;
+l_float32  val, startx, delx;
 NUMA      *na;
 
     PROCNAME("numaReadStream");
@@ -671,6 +845,10 @@ NUMA      *na;
         numaAddNumber(na, val);
     }
 
+        /* Optional data */
+    if ((fscanf(fp, "startx = %f, delx = %f\n", &startx, &delx)) == 2)
+        numaSetXParameters(na, startx, delx);
+
     return na;
 }
         
@@ -714,7 +892,8 @@ l_int32
 numaWriteStream(FILE  *fp,
                 NUMA  *na)
 {
-l_int32  i, n;
+l_int32    i, n;
+l_float32  startx, delx;
 
     PROCNAME("numaWriteStream");
 
@@ -730,6 +909,11 @@ l_int32  i, n;
         fprintf(fp, "  [%d] = %f\n", i, na->array[i]);
     fprintf(fp, "\n");
 
+        /* Optional data */
+    numaGetXParameters(na, &startx, &delx);
+    if (startx != 0.0 || delx != 1.0)
+        fprintf(fp, "startx = %f, delx = %f\n", startx, delx);
+
     return 0;
 }
         
@@ -855,7 +1039,6 @@ NUMA    *nac;
 l_int32
 numaaExtendArray(NUMAA  *naa)
 {
-
     PROCNAME("numaaExtendArray");
 
     if (!naa)
@@ -1019,6 +1202,142 @@ NUMA    *na;
 }
 
 
+/*----------------------------------------------------------------------*
+ *                      Serialize numaa for I/O                         *
+ *----------------------------------------------------------------------*/
+/*!
+ *  numaaRead()
+ *
+ *      Input:  filename
+ *      Return: naa, or null on error
+ */
+NUMAA *
+numaaRead(const char  *filename)
+{
+FILE   *fp;
+NUMAA  *naa;
+
+    PROCNAME("numaaRead");
+
+    if (!filename)
+        return (NUMAA *)ERROR_PTR("filename not defined", procName, NULL);
+
+    if ((fp = fopenReadStream(filename)) == NULL)
+        return (NUMAA *)ERROR_PTR("stream not opened", procName, NULL);
+
+    if ((naa = numaaReadStream(fp)) == NULL) {
+        fclose(fp);
+        return (NUMAA *)ERROR_PTR("naa not read", procName, NULL);
+    }
+
+    fclose(fp);
+    return naa;
+}
+        
+
+/*!
+ *  numaaReadStream()
+ *
+ *      Input:  stream
+ *      Return: naa, or null on error
+ */
+NUMAA *
+numaaReadStream(FILE  *fp)
+{
+l_int32    i, n, index, ret, version;
+NUMA      *na;
+NUMAA     *naa;
+
+    PROCNAME("numaaReadStream");
+
+    if (!fp)
+        return (NUMAA *)ERROR_PTR("stream not defined", procName, NULL);
+
+    ret = fscanf(fp, "\nNumaa Version %d\n", &version);
+    if (ret != 1)
+        return (NUMAA *)ERROR_PTR("not a numa file", procName, NULL);
+    if (version != NUMA_VERSION_NUMBER)
+        return (NUMAA *)ERROR_PTR("invalid numaa version", procName, NULL);
+    fscanf(fp, "Number of numa = %d\n\n", &n);
+    if ((naa = numaaCreate(n)) == NULL)
+        return (NUMAA *)ERROR_PTR("naa not made", procName, NULL);
+
+    for (i = 0; i < n; i++) {
+        fscanf(fp, "Numa[%d]:", &index);
+        if ((na = numaReadStream(fp)) == NULL)
+            return (NUMAA *)ERROR_PTR("na not made", procName, NULL);
+        numaaAddNuma(naa, na, L_INSERT);
+    }
+
+    return naa;
+}
+        
+
+/*!
+ *  numaaWrite()
+ *
+ *      Input:  filename, naa
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+numaaWrite(const char  *filename,
+           NUMAA       *naa)
+{
+FILE  *fp;
+
+    PROCNAME("numaaWrite");
+
+    if (!filename)
+        return ERROR_INT("filename not defined", procName, 1);
+    if (!naa)
+        return ERROR_INT("naa not defined", procName, 1);
+
+    if ((fp = fopen(filename, "w")) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    if (numaaWriteStream(fp, naa))
+        return ERROR_INT("naa not written to stream", procName, 1);
+    fclose(fp);
+
+    return 0;
+}
+        
+
+/*!
+ *  numaaWriteStream()
+ *
+ *      Input:  stream, naa
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+numaaWriteStream(FILE   *fp,
+                 NUMAA  *naa)
+{
+l_int32    i, n;
+l_float32  startx, delx;
+NUMA      *na;
+
+    PROCNAME("numaaWriteStream");
+
+    if (!fp)
+        return ERROR_INT("stream not defined", procName, 1);
+    if (!naa)
+        return ERROR_INT("naa not defined", procName, 1);
+
+    n = numaaGetCount(naa);
+    fprintf(fp, "\nNumaa Version %d\n", NUMA_VERSION_NUMBER);
+    fprintf(fp, "Number of numa = %d\n\n", n);
+    for (i = 0; i < n; i++) {
+        if ((na = numaaGetNuma(naa, i, L_CLONE)) == NULL)
+            return ERROR_INT("na not found", procName, 1);
+        fprintf(fp, "Numa[%d]:", i);
+        numaWriteStream(fp, na);
+        numaDestroy(&na);
+    }
+
+    return 0;
+}
+        
+
 /*--------------------------------------------------------------------------*
  *                      Numa2d creation, destruction                        *
  *--------------------------------------------------------------------------*/
@@ -1180,7 +1499,8 @@ NUMA  *na;
  *              col of 2d array
  *      Return: na (a clone of the numa if it exists) or null if it doesn't
  *
- *  Note: this does not give an error if the index is out of bounds.
+ *  Notes:
+ *      (1) This does not give an error if the index is out of bounds.
  */
 NUMA *
 numa2dGetNuma(NUMA2D     *na2d,
