@@ -13,8 +13,8 @@
  -  or altered from any source or modified source distribution.
  *====================================================================*/
 
-#ifndef  PIX_H_INCLUDED
-#define  PIX_H_INCLUDED
+#ifndef  LEPTONICA_PIX_H
+#define  LEPTONICA_PIX_H
 
 /*
  *   pix.h
@@ -31,6 +31,7 @@
  *       struct Pta
  *       struct Ptaa
  *       struct Pixacc
+ *       struct PixTiling
  *
  *   Contains definitions for:
  *       colors for RGB
@@ -42,10 +43,18 @@
  *       graphics pixel setting flags
  *       size filtering flags
  *       rotation and shear flags
+ *       affine transform order flags                        *
  *       grayscale filling flags
  *       dithering flags
  *       distance flags
+ *       statistical measures
+ *       set selection flags
  *       text orientation flags
+ *       edge orientation flags
+ *       line orientation flags
+ *       thinning flags
+ *       runlength flags
+ *       edge filter flags
  */
 
 
@@ -99,9 +108,19 @@ typedef struct RGBA_Quad  RGBA_QUAD;
 /*-------------------------------------------------------------------------*
  *                             Colors for 32 bpp                           *
  *-------------------------------------------------------------------------*/
-/* Note: colors are used in 32 bpp images.  The 4th byte, typically
- *       known as the "alpha channel", can be used for blending.
- *       It's not explicitly used in leptonica.  */
+/*  Notes:
+ *      (1) These are the byte indices for colors in 32 bpp images.
+ *          They are used through the GET/SET_DATA_BYTE accessors.
+ *          The 4th byte, typically known as the "alpha channel" and used
+ *          for blending, is not explicitly used in leptonica.
+ *      (2) If you redefine these values, a lot of functions that
+ *          have been made more efficient by assuming them implicitly
+ *          will break.  These functions are labelled with "***"
+ *          next to their names at the top of the files in which they
+ *          are defined.  Advice: Do not change these values!
+ *      (3) The shifts to extract the red, green and blue components
+ *          from a 32 bit pixel are defined in terms of these colors.
+ */
 enum {
     COLOR_RED = 0,
     COLOR_GREEN = 1,
@@ -109,6 +128,15 @@ enum {
     L_ALPHA_CHANNEL = 3
 };
 
+static const l_int32  L_RED_SHIFT = 
+       8 * (sizeof(l_uint32) - 1 - COLOR_RED);           /* 24 */
+static const l_int32  L_GREEN_SHIFT = 
+       8 * (sizeof(l_uint32) - 1 - COLOR_GREEN);         /* 16 */
+static const l_int32  L_BLUE_SHIFT = 
+       8 * (sizeof(l_uint32) - 1 - COLOR_BLUE);          /*  8 */
+static const l_int32  L_ALPHA_SHIFT = 
+       8 * (sizeof(l_uint32) - 1 - L_ALPHA_CHANNEL);     /*  0 */
+
 
 /*-------------------------------------------------------------------------*
  *                        Flags for colormap conversion                    *
@@ -264,6 +292,16 @@ enum {
  *           of the pix is decremented.  The pix is only destroyed
  *           when the reference count goes to zero.
  *
+ *       (6) The version numbers (below) are used in the serialization
+ *           of these data structures.  They are placed in the files,
+ *           and rarely (if ever) change.  No provision is made for
+ *           backward compatibility in either reading or writing.
+ *
+ *       (7) The PIXA_VERSION_NUMBER is applied to pixa, pixaa, boxa,
+ *           and boxaa, and must be changed if any of these serializations
+ *           are changed.  That is because pixaa depends on pixa and boxa,
+ *           and pixa and boxaa depend on boxa.
+ *
  *-------------------------------------------------------------------------*/
 
 
@@ -289,6 +327,7 @@ struct Pixaa
     l_int32             n;            /* number of Pixa in ptr array       */
     l_int32             nalloc;       /* number of Pixa ptrs allocated     */
     struct Pixa       **pixa;         /* array of ptrs to pixa             */
+    struct Boxa        *boxa;         /* array of boxes                    */
 };
 typedef struct Pixaa PIXAA;
 
@@ -297,8 +336,6 @@ typedef struct Pixaa PIXAA;
 /*-------------------------------------------------------------------------*
  *                    Basic rectangle and rectangle arrays                 *
  *-------------------------------------------------------------------------*/
-#define  BOXA_VERSION_NUMBER      2
-
 struct Box
 {
     l_int32            x;
@@ -369,6 +406,21 @@ struct Pixacc
 typedef struct Pixacc PIXACC;
 
 
+/*-------------------------------------------------------------------------*
+ *                              Pix tiling                                 *
+ *-------------------------------------------------------------------------*/
+struct PixTiling
+{
+    struct Pix          *pix;         /* input pix (a clone)               */
+    l_int32              nx;          /* number of tiles horizontally      */
+    l_int32              ny;          /* number of tiles vertically        */
+    l_int32              w;           /* tile width                        */
+    l_int32              h;           /* tile height                       */
+    l_int32              overlap;     /* amount of overlap on each side    */
+};
+typedef struct PixTiling PIXTILING;
+
+
 /*-------------------------------------------------------------------------*
  *                         Access and storage flags                        *
  *-------------------------------------------------------------------------*/
@@ -465,10 +517,17 @@ enum {
  *                           Size filter flags                             *
  *-------------------------------------------------------------------------*/
 enum {
-    L_REMOVE_IF_EITHER = 1,       /* remove component if either constraint */
-                                  /* is not met                            */
-    L_REMOVE_IF_BOTH = 2          /* remove component only if both         */
-                                  /* constraints are not met               */
+    L_SELECT_WIDTH = 1,           /* width must satisfy constraint         */
+    L_SELECT_HEIGHT = 2,          /* height must satisfy constraint        */
+    L_SELECT_IF_EITHER = 3,       /* either width or height can satisfy    */
+    L_SELECT_IF_BOTH = 4          /* both width and height must satisfy    */
+};
+
+enum {
+    L_SELECT_IF_LT = 1,           /* save if value is less than threshold  */
+    L_SELECT_IF_GT = 2,           /* save if value is more than threshold  */
+    L_SELECT_IF_LTE = 3,          /* save if value is <= to the threshold  */
+    L_SELECT_IF_GTE = 4           /* save if value is >= to the threshold  */
 };
 
 
@@ -486,6 +545,19 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                     Affine transform order flags                        *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_TR_SC_RO = 1,              /* translate, scale, rotate               */
+    L_SC_RO_TR = 2,              /* scale, rotate, translate               */
+    L_RO_TR_SC = 3,              /* rotate, translate, scale               */
+    L_TR_RO_SC = 4,              /* translate, rotate, scale               */
+    L_RO_SC_TR = 5,              /* rotate, scale, translate               */
+    L_SC_TR_RO = 6               /* scale, translate, rotate               */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                         Grayscale fill flags                            *
  *-------------------------------------------------------------------------*/
@@ -518,11 +590,22 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                            Average measures                             *
+ *                         Statistical measures                            *
  *-------------------------------------------------------------------------*/
 enum {
     L_MEAN_ABSVAL = 1,           /* average of abs values                  */
-    L_ROOT_MEAN_SQUARE = 2       /* rms of values                          */
+    L_ROOT_MEAN_SQUARE = 2,      /* rms of values                          */
+    L_STANDARD_DEVIATION = 3,    /* standard deviation from mean           */
+    L_VARIANCE = 4               /* variance of values                     */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                          Set selection flags                            *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_CHOOSE_CONSECUTIVE = 1,    /* select 'n' consecutive                 */
+    L_CHOOSE_SKIP_BY = 2         /* select at intervals of 'n'             */
 };
 
 
@@ -538,5 +621,53 @@ enum {
 };
 
 
-#endif  /* PIX_H_INCLUDED */
+/*-------------------------------------------------------------------------*
+ *                         Edge orientation flags                          *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_HORIZONTAL_EDGES = 0,     /* filters for horizontal edges            */
+    L_VERTICAL_EDGES = 1,       /* filters for vertical edges              */
+    L_ALL_EDGES = 2             /* filters for all edges                   */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                         Line orientation flags                          *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_HORIZONTAL_LINE = 0,     /* horizontal line                          */
+    L_POS_SLOPE_LINE = 1,      /* 45 degree line with positive slope       */
+    L_VERTICAL_LINE = 2,       /* vertical line                            */
+    L_NEG_SLOPE_LINE = 3       /* 45 degree line with negative slope       */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                             Thinning flags                              *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_THIN_FG = 1,               /* thin foreground of 1 bpp image         */
+    L_THIN_BG = 2                /* thin background of 1 bpp image         */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                            Runlength flags                              *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_HORIZONTAL_RUNS = 0,     /* determine runlengths of horizontal runs  */
+    L_VERTICAL_RUNS = 1        /* determine runlengths of vertical runs    */
+};
+
+
+/*-------------------------------------------------------------------------*
+ *                          Edge filter flags                              *
+ *-------------------------------------------------------------------------*/
+enum {
+    L_SOBEL_EDGE = 1,          /* Sobel edge filter                        */
+    L_TWO_SIDED_EDGE = 2       /* Two-sided edge filter                    */
+};
+
+
+#endif  /* LEPTONICA_PIX_H */
 
