@@ -16,17 +16,23 @@
 /*
  *  pix1.c
  *
- *    The pixN.c {N = 1,2,3} files are sorted by the type of operation.
+ *    The pixN.c {N = 1,2,3,4} files are sorted by the type of operation.
  *    The primary functions in these files are:
  *
  *        pix1.c: constructors, destructors and field accessors
  *        pix2.c: pixel poking of image, pad and border pixels
- *        pix3.c: logical and mask ops; counting; histograms
+ *        pix3.c: masking and logical ops, counting, mirrored tiling
+ *        pix4.c: histograms, fg/bg estimation, rectangle extraction
  *
  *
  *    This file has the basic constructors, destructors and field accessors
  *
- *      Pix creation
+ *    Pix memory management
+ *          static void  *pix_malloc()
+ *          static void   pix_free()
+ *          void          setPixMemoryManager()
+ *
+ *    Pix creation
  *          PIX          *pixCreate()
  *          PIX          *pixCreateNoInit()
  *          PIX          *pixCreateTemplate()
@@ -34,16 +40,17 @@
  *          PIX          *pixCreateHeader()
  *          PIX          *pixClone()
  *
- *      Pix destruction
+ *    Pix destruction
  *          void          pixDestroy()
  *          void          pixFree()
  *
- *      Pix copy
+ *    Pix copy
  *          PIX          *pixCopy()
+ *          l_int32       pixResizeImageData()
  *          l_int32       pixCopyColormap()
  *          l_int32       pixSizesEqual()
  *
- *      Pix accessors
+ *    Pix accessors
  *          l_int32       pixGetWidth()
  *          l_int32       pixSetWidth()
  *          l_int32       pixGetHeight()
@@ -74,7 +81,7 @@
  *          l_uint32     *pixGetData()
  *          l_int32       pixSetData()
  *
- *      Pix debug
+ *    Pix debug
  *          l_int32       pixPrintStreamInfo()
  */
 
@@ -84,6 +91,85 @@
 #include "allheaders.h"
 
 
+/*-------------------------------------------------------------------------*
+ *                        Pix Memory Management                            *
+ *                                                                         *
+ *  These functions give you the freedom to specify at compile or run      *
+ *  time the allocator and deallocator to be used for pix.  It has no      *
+ *  effect on memory management for other data structs, which are          *
+ *  controlled by the #defines in environ.h.  Likewise, the #defines       *
+ *  in environ.h have no effect on the pix memory management.              *
+ *  The default functions are malloc and free.  Use setPixMemoryManager()  *
+ *  to specify other functions to use.                                     *
+ *-------------------------------------------------------------------------*/
+struct PixMemoryManager
+{
+    void     *(*allocator)(size_t);
+    void      (*deallocator)(void *);
+};
+
+static struct PixMemoryManager  pix_mem_manager = {
+    &malloc,
+    &free
+};
+    
+static void *
+pix_malloc(size_t  size)
+{
+#ifndef COMPILER_MSVC
+    return (*pix_mem_manager.allocator)(size);
+#else  /* COMPILER_MSVC */
+    /* Under MSVC++, pix_mem_manager is initialized after a call
+     * to pix_malloc.  Just ignore the custom allocator feature. */
+    return malloc(size);
+#endif  /* COMPILER_MSVC */
+}
+
+static void
+pix_free(void  *ptr)
+{
+#ifndef COMPILER_MSVC
+    return (*pix_mem_manager.deallocator)(ptr);
+#else  /* COMPILER_MSVC */
+    /* Under MSVC++, pix_mem_manager is initialized after a call
+     * to pix_malloc.  Just ignore the custom allocator feature. */
+    return free(ptr);
+#endif  /* COMPILER_MSVC */
+}
+
+/*!
+ *  setPixMemoryManager()
+ *
+ *      Input: allocator (<optional>; use null to skip)
+ *             deallocator (<optional>; use null to skip)
+ *      Return: void
+ *
+ *  Notes:
+ *      (1) Use this to change the alloc and/or dealloc functions;
+ *          e.g., setPixMemoryManager(my_malloc, my_free).
+ */
+#ifndef COMPILER_MSVC
+void
+setPixMemoryManager(void  *(allocator(size_t)),
+                    void  (deallocator(void *)))
+{
+    if (allocator) pix_mem_manager.allocator = allocator;
+    if (deallocator) pix_mem_manager.deallocator = deallocator;
+    return;
+}
+#else  /* COMPILER_MSVC */
+    /* MSVC++ wants type (*fun)(types...) syntax */
+void
+setPixMemoryManager(void  *((*allocator)(size_t)),
+                    void  ((*deallocator)(void *)))
+{
+    if (allocator) pix_mem_manager.allocator = allocator;
+    if (deallocator) pix_mem_manager.deallocator = deallocator;
+    return;
+}
+#endif /* COMPILER_MSVC */
+
+
 /*--------------------------------------------------------------------*
  *                              Pix Creation                          *
  *--------------------------------------------------------------------*/
@@ -116,6 +202,10 @@ PIX       *pixd;
  *      Input:  width, height, depth
  *      Return: pixd (with data allocated but not initialized),
  *                    or null on error
+ *
+ *  Notes:
+ *      (1) Must set pad bits to avoid reading unitialized data, because
+ *          some optimized routines (e.g., pixConnComp()) read from pad bits.
  */
 PIX *
 pixCreateNoInit(l_int32  width,
@@ -130,9 +220,10 @@ l_uint32  *data;
     pixd = pixCreateHeader(width, height, depth);
     if (!pixd) return NULL;
     wpl = pixGetWpl(pixd);
-    if ((data = (l_uint32 *)MALLOC(4 * wpl * height)) == NULL)
+    if ((data = (l_uint32 *)pix_malloc(4 * wpl * height)) == NULL)
         return (PIX *)ERROR_PTR("MALLOC fail for data", procName, NULL);
     pixSetData(pixd, data);
+    pixSetPadBits(pixd, 0);
     return pixd;
 }
 
@@ -245,21 +336,21 @@ PIX       *pixd;
  *      Input:  pix
  *      Return: same pix (ptr), or null on error
  *
- *  Note: Why is this here?  We make a "clone", which
- *        is just another handle to an existing pix, because
- *        (1) images can be large and hence expensive to copy,
- *        (2) extra handles to a data structure that are made
- *        without some control are dangerous because if you
- *        have two (say), how do you know which one should be
- *        used to destroy the pix, and, conversely, if you
- *        destroy the pix, how do you remember that the other
- *        handle is no longer valid?  This is solved by a
- *        simple protocol: (1) whenever you want a new handle
- *        to an existing image, call pixClone(), which just
- *        bumps a ref count, and (2) always call pixDestroy()
- *        on ALL handles, which decrements the ref count,
- *        nulls the handle, and only destroys the pix when
- *        pixDestroy() has been called on all handles.
+ *  Notes:
+ *      (1) A "clone" is simply a handle (ptr) to an existing pix.
+ *          It is implemented because (a) images can be large and
+ *          hence expensive to copy, and (b) extra handles to a data
+ *          structure need to be made with a simple policy to avoid
+ *          both double frees and memory leaks.  Pix are reference
+ *          counted.  The side effect of pixClone() is an increase
+ *          by 1 in the ref count.
+ *      (2) The protocol to be used is:
+ *          (a) Whenever you want a new handle to an existing image,
+ *              call pixClone(), which just bumps a ref count.
+ *          (b) Always call pixDestroy() on all handles.  This
+ *              decrements the ref count, nulls the handle, and
+ *              only destroys the pix when pixDestroy() has been
+ *              called on all handles.
  */
 PIX *
 pixClone(PIX  *pixs)
@@ -329,7 +420,7 @@ char      *text;
     pixChangeRefcount(pix, -1);
     if (pixGetRefcount(pix) <= 0) {
         if ((data = pixGetData(pix)))
-            FREE(data);
+            pix_free(data);
         if ((text = pixGetText(pix)))
             FREE(text);
         pixDestroyColormap(pix);
@@ -345,16 +436,29 @@ char      *text;
 /*!
  *  pixCopy()
  *
- *      Input:  pixd (<optional>)
+ *      Input:  pixd (<optional>; can be null, or equal to pixs,
+ *                    or different from pixs)
  *              pixs
  *      Return: pixd, or null on error
  *
  *  Notes:
- *      (1) If pixd = NULL, this makes a new copy, with refcount of 1.
- *          If pixd != NULL, this makes sure pixs and pixd are the same
- *          size, and then copies the image data, leaving the refcounts
- *          of pixs and pixd unchanged.
- *      (2) This operation, like all others that may involve a pre-existing
+ *      (1) There are three cases:
+ *            (a) pixd == null  (makes a new pix; refcount = 1)
+ *            (b) pixd == pixs  (no-op)
+ *            (c) pixd != pixs  (data copy; no change in refcount)
+ *          If the refcount of pixd > 1, case (c) will side-effect
+ *          these handles.
+ *      (2) The general pattern of use is:
+ *             pixd = pixCopy(pixd, pixs);
+ *          This will work for all three cases.
+ *          For clarity when the case is known, you can use:
+ *            (a) pixd = pixCopy(NULL, pixs);
+ *            (c) pixCopy(pixd, pixs);
+ *      (3) For case (c), we check if pixs and pixd are the same
+ *          size (w,h,d).  If so, the data is copied directly.
+ *          Otherwise, the data is reallocated to the correct size
+ *          and the copy proceeds.  The refcount of pixd is unchanged.
+ *      (4) This operation, like all others that may involve a pre-existing
  *          pixd, will side-effect any existing clones of pixd.
  */
 PIX *
@@ -367,11 +471,9 @@ l_uint32  *datas, *datad;
     PROCNAME("pixCopy");
 
     if (!pixs)
-        return (PIX *)ERROR_PTR("pixs not defined", procName, pixd);
-    if (pixs == pixd) {
-        L_WARNING("pix copied to itself", procName);
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (pixs == pixd)
         return pixd;
-    }
 
         /* Total bytes in image data */
     bytes = 4 * pixGetWpl(pixs) * pixGetHeight(pixs);
@@ -386,23 +488,62 @@ l_uint32  *datas, *datad;
         return pixd;
     }
 
-        /* Check sizes */
-    if (!pixSizesEqual(pixs, pixd))
-        return (PIX *)ERROR_PTR("pix sizes not equal", procName, pixd);
+        /* Reallocate image data if sizes are different */
+    pixResizeImageData(pixd, pixs);
+
+        /* Copy non-image data fields */
     pixCopyColormap(pixd, pixs);
     pixCopyResolution(pixd, pixs);
     pixCopyInputFormat(pixd, pixs);
     pixCopyText(pixd, pixs);
 
-        /* Copy the data */
+        /* Copy image data */
     datas = pixGetData(pixs);
     datad = pixGetData(pixd);
     memcpy((char*)datad, (char*)datas, bytes);
-
     return pixd;
 }
 
 
+/*!
+ *  pixResizeImageData()
+ *
+ *      Input:  pixd, pixs
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixResizeImageData(PIX  *pixd,
+                   PIX  *pixs)
+{
+l_int32    w, h, d, wpl, bytes;
+l_uint32  *data;
+
+    PROCNAME("pixResizeImageData");
+
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+    if (!pixd)
+        return ERROR_INT("pixd not defined", procName, 1);
+
+    if (pixSizesEqual(pixs, pixd))  /* nothing to do */
+        return 0;
+
+    pixGetDimensions(pixs, &w, &h, &d);
+    wpl = pixGetWpl(pixs);
+    pixSetWidth(pixd, w);
+    pixSetHeight(pixd, h);
+    pixSetDepth(pixd, d);
+    pixSetWpl(pixd, wpl);
+    bytes = 4 * wpl * h;
+    if ((data = pixGetData(pixd)))
+        pix_free(data);
+    if ((data = (l_uint32 *)pix_malloc(bytes)) == NULL)
+        return ERROR_INT("MALLOC fail for data", procName, 1);
+    pixSetData(pixd, data);
+    return 0;
+}
+
+
 /*!
  *  pixCopyColormap()
  *
@@ -436,8 +577,8 @@ PIXCMAP  *cmaps, *cmapd;
 /*!
  *  pixSizesEqual()
  *
- *      Input:  two Pix
- *      Return: 1 if the two Pix have same {h, w, d}; 0 otherwise.
+ *      Input:  two pix
+ *      Return: 1 if the two pix have same {h, w, d}; 0 otherwise.
  */
 l_int32
 pixSizesEqual(PIX  *pix1,
@@ -445,10 +586,11 @@ pixSizesEqual(PIX  *pix1,
 {
     PROCNAME("pixSizesEqual");
 
-    if (!pix1)
-        return ERROR_INT("pix1 not defined", procName, 0);
-    if (!pix2)
-        return ERROR_INT("pix2 not defined", procName, 0);
+    if (!pix1 || !pix2)
+        return ERROR_INT("pix1 and pix2 not both defined", procName, 0);
+
+    if (pix1 == pix2)
+        return 1;
 
     if ((pixGetWidth(pix1) != pixGetWidth(pix2)) ||
         (pixGetHeight(pix1) != pixGetHeight(pix2)) ||
@@ -958,4 +1100,3 @@ PIXCMAP  *cmap;
 
     return 0;
 }
-
