@@ -18,13 +18,16 @@
  *
  *    This file has these basic operations:
  *
- *      (1) Get and set of image pixels, pad pixels, border pixels,
- *          and color components for RGB
- *      (2) Endian byte swaps
+ *      (1) Get and set: individual pixels, full image, rectangular region,
+ *          pad pixels, border pixels, and color components for RGB
+ *      (2) Add and remove border pixels
+ *      (3) Endian byte swaps
  *
  *      Pixel poking
  *           l_int32     pixGetPixel()
  *           l_int32     pixSetPixel()
+ *           l_int32     pixGetRGBPixel()
+ *           l_int32     pixSetRGBPixel()
  *           l_int32     pixClearPixel()
  *           l_int32     pixFlipPixel()
  *           void        setPixelLow()
@@ -33,30 +36,44 @@
  *           l_int32     pixClearAll()
  *           l_int32     pixSetAll()
  *           l_int32     pixSetAllArbitrary()
+ *
+ *      Rectangular region clear/set/set-to-arbitrary-value
+ *           l_int32     pixClearInRect()
+ *           l_int32     pixSetInRect()
+ *           l_int32     pixSetInRectArbitrary()
+ *
+ *      Set pad bits
  *           l_int32     pixSetPadBits()
  *           l_int32     pixSetPadBitsBand()
  *
  *      Set border pixels to arbitrary value
  *           l_int32     pixSetOrClearBorder()
  *           l_int32     pixSetBorderVal()
+ *           l_int32     pixSetMirroredBorder()
  *
  *      Add and remove border
  *           PIX        *pixAddBorder()
- *           PIX        *pixRemoveBorder()
  *           PIX        *pixAddBorderGeneral()
+ *           PIX        *pixRemoveBorder()
  *           PIX        *pixRemoveBorderGeneral()
+ *           PIX        *pixAddMirroredBorder()
  *
  *      Color sample setting and extraction
  *           PIX        *pixCreateRGBImage()
  *           PIX        *pixGetRGBComponent()
  *           l_int32     pixSetRGBComponent()
+ *           PIX        *pixGetRGBComponentCmap()
  *           l_int32     composeRGBPixel()
+ *           void        extractRGBValues()
  *           l_int32     pixGetRGBLine()
  *
  *      Conversion between big and little endians
  *           PIX        *pixEndianByteSwapNew()
  *           l_int32     pixEndianByteSwap()
+ *           PIX        *pixEndianTwoByteSwapNew()
  *           l_int32     pixEndianTwoByteSwap()
+ *
+ *      *** indicates implicit assumption about RGB component ordering
  */
 
 
@@ -208,6 +225,91 @@ l_uint32  *line, *data;
 }
 
 
+/*!
+ *  pixGetRGBPixel()
+ *
+ *      Input:  pix
+ *              (x,y) pixel coords
+ *              &rval (<optional return> red component)
+ *              &gval (<optional return> green component)
+ *              &bval (<optional return> blue component)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixGetRGBPixel(PIX      *pix,
+               l_int32   x,
+               l_int32   y,
+               l_int32  *prval,
+               l_int32  *pgval,
+               l_int32  *pbval)
+{
+l_int32    w, h, d, wpl;
+l_uint32  *data, *ppixel;
+
+    PROCNAME("pixGetRGBPixel");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    pixGetDimensions(pix, &w, &h, &d);
+    if (d != 32)
+        return ERROR_INT("pix not 32 bpp", procName, 1);
+    if (x < 0 || x >= w)
+        return ERROR_INT("x out of bounds", procName, 1);
+    if (y < 0 || y >= h)
+        return ERROR_INT("y out of bounds", procName, 1);
+
+    wpl = pixGetWpl(pix);
+    data = pixGetData(pix);
+    ppixel = data + y * wpl + x;
+    if (prval) *prval = GET_DATA_BYTE(ppixel, COLOR_RED);
+    if (pgval) *pgval = GET_DATA_BYTE(ppixel, COLOR_GREEN);
+    if (pbval) *pbval = GET_DATA_BYTE(ppixel, COLOR_BLUE);
+    return 0;
+}
+
+
+/*!
+ *  pixSetRGBPixel()
+ *
+ *      Input:  pix
+ *              (x,y) pixel coords
+ *              rval (red component)
+ *              gval (green component)
+ *              bval (blue component)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixSetRGBPixel(PIX     *pix,
+               l_int32  x,
+               l_int32  y,
+               l_int32  rval,
+               l_int32  gval,
+               l_int32  bval)
+{
+l_int32    w, h, d, wpl;
+l_uint32   pixel;
+l_uint32  *data, *line;
+
+    PROCNAME("pixSetRGBPixel");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    pixGetDimensions(pix, &w, &h, &d);
+    if (d != 32)
+        return ERROR_INT("pix not 32 bpp", procName, 1);
+    if (x < 0 || x >= w)
+        return ERROR_INT("x out of bounds", procName, 1);
+    if (y < 0 || y >= h)
+        return ERROR_INT("y out of bounds", procName, 1);
+
+    wpl = pixGetWpl(pix);
+    data = pixGetData(pix);
+    line = data + y * wpl;
+    composeRGBPixel(rval, gval, bval, &pixel);
+    *(line + x) = pixel;
+    return 0;
+}
+
 
 /*!
  *  pixClearPixel()
@@ -385,7 +487,6 @@ setPixelLow(l_uint32  *line,
 }
 
 
-
 /*-------------------------------------------------------------*
  *     Full image clear/set/set-to-arbitrary-value/invert      *
  *-------------------------------------------------------------*/
@@ -487,6 +588,160 @@ l_uint32  *data, *line;
 }
 
 
+/*-------------------------------------------------------------*
+ *     Rectangular region clear/set/set-to-arbitrary-value     *
+ *-------------------------------------------------------------*/
+/*!
+ *  pixClearInRect()
+ *
+ *      Input:  pix (all depths)
+ *              box (in which all pixels will be cleared)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixClearInRect(PIX  *pix,
+               BOX  *box)
+{
+l_int32  x, y, w, h;
+
+    PROCNAME("pixClearInRect");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+
+    boxGetGeometry(box, &x, &y, &w, &h);
+    pixRasterop(pix, x, y, w, h, PIX_CLR, NULL, 0, 0);
+    return 0;
+}
+
+
+/*!
+ *  pixSetInRect()
+ *
+ *      Input:  pix (all depths)
+ *              box (in which all pixels will be set)
+ *      Return: 0 if OK, 1 on error
+ */
+l_int32
+pixSetInRect(PIX  *pix,
+             BOX  *box)
+{
+l_int32  x, y, w, h;
+
+    PROCNAME("pixSetInRect");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+
+    boxGetGeometry(box, &x, &y, &w, &h);
+    pixRasterop(pix, x, y, w, h, PIX_SET, NULL, 0, 0);
+    return 0;
+}
+
+
+/*!
+ *  pixSetInRectArbitrary()
+ *
+ *      Input:  pix (all depths)
+ *              box (in which all pixels will be set to val)
+ *              val  (value to set all pixels)
+ *      Return: 0 if OK; 1 on error
+ */
+l_int32
+pixSetInRectArbitrary(PIX      *pix,
+                      BOX      *box,
+                      l_uint32  val)
+{
+l_int32    x, y, xstart, xend, ystart, yend, bw, bh, w, h, d, wpl, maxval;
+l_uint32  *data, *line;
+BOX       *boxc;
+
+    PROCNAME("pixSetInRectArbitrary");
+
+    if (!pix)
+        return ERROR_INT("pix not defined", procName, 1);
+    if (!box)
+        return ERROR_INT("box not defined", procName, 1);
+    pixGetDimensions(pix, &w, &h, &d);
+    if (d != 1 && d != 2 && d != 4 && d !=8 && d != 16 && d != 32)
+        return ERROR_INT("depth must be in {1,2,4,8,16,32} bpp", procName, 1);
+
+    if (d == 32)
+        maxval = 0xffffffff;
+    else
+        maxval = (1 << d) - 1;
+    if (val < 0) {
+        L_WARNING("invalid pixel value; set to 0", procName);
+        val = 0;
+    }
+    if (val > maxval) {
+        L_WARNING_INT("invalid pixel val; set to maxval = %d",
+                      procName, maxval);
+        val = maxval;
+    }
+
+        /* Handle the simple cases: the min and max values */
+    if (val == 0) {
+        pixClearInRect(pix, box);
+        return 0;
+    }
+    if (d == 1 ||
+        (d == 2 && val == 3) ||
+        (d == 4 && val == 0xf) ||
+        (d == 8 && val == 0xff) ||
+        (d == 16 && val == 0xffff) ||
+        (d == 32 && ((val ^ 0xffffff00) >> 8 == 0))) {
+        pixSetInRect(pix, box);
+        return 0;
+    }
+
+        /* Find the overlap of box with the input pix */
+    if ((boxc = boxClipToRectangle(box, w, h)) == NULL)
+        return ERROR_INT("no overlap of box with image", procName, 1);
+    boxGetGeometry(boxc, &xstart, &ystart, &bw, &bh);
+    xend = xstart + bw - 1;
+    yend = ystart + bh - 1;
+    boxDestroy(&boxc);
+
+    wpl = pixGetWpl(pix);
+    data = pixGetData(pix);
+    for (y = ystart; y <= yend; y++) {
+        line = data + y * wpl;
+        for (x = xstart; x <= xend; x++) {
+            switch(d)
+            {
+            case 2:
+                SET_DATA_DIBIT(line, x, val);
+                break;
+            case 4:
+                SET_DATA_QBIT(line, x, val);
+                break;
+            case 8:
+                SET_DATA_BYTE(line, x, val);
+                break;
+            case 16:
+                SET_DATA_TWO_BYTES(line, x, val);
+                break;
+            case 32:
+                line[x] = val;
+                break;
+            default:
+                return ERROR_INT("depth not 2|4|8|16|32 bpp", procName, 1);
+            }
+        }
+    }
+
+    return 0;
+}
+
+
+/*-------------------------------------------------------------*
+ *                         Set pad bits                        *
+ *-------------------------------------------------------------*/
 /*!
  *  pixSetPadBits()
  *
@@ -618,7 +873,7 @@ l_uint32  *data, *pword;
  *  pixSetOrClearBorder()
  *
  *      Input:  pixs (all depths)
- *              leftpix, rightpix, toppix, bottompix
+ *              left, right, top, bot (amount to set or clear)
  *              operation (PIX_SET or PIX_CLR)
  *      Return: 0 if OK; 1 on error
  *
@@ -634,10 +889,10 @@ l_uint32  *data, *pword;
  */
 l_int32
 pixSetOrClearBorder(PIX     *pixs,
-                    l_int32  leftpix,
-                    l_int32  rightpix,
-                    l_int32  toppix,
-                    l_int32  bottompix,
+                    l_int32  left,
+                    l_int32  right,
+                    l_int32  top,
+                    l_int32  bot,
                     l_int32  op)
 {
 l_int32  w, h;
@@ -650,10 +905,10 @@ l_int32  w, h;
         return ERROR_INT("op must be PIX_SET or PIX_CLR", procName, 1);
 
     pixGetDimensions(pixs, &w, &h, NULL);
-    pixRasterop(pixs, 0, 0, leftpix, h, op, NULL, 0, 0);
-    pixRasterop(pixs, w - rightpix, 0, rightpix, h, op, NULL, 0, 0);
-    pixRasterop(pixs, 0, 0, w, toppix, op, NULL, 0, 0);
-    pixRasterop(pixs, 0, h - bottompix, w, bottompix, op, NULL, 0, 0);
+    pixRasterop(pixs, 0, 0, left, h, op, NULL, 0, 0);
+    pixRasterop(pixs, w - right, 0, right, h, op, NULL, 0, 0);
+    pixRasterop(pixs, 0, 0, w, top, op, NULL, 0, 0);
+    pixRasterop(pixs, 0, h - bot, w, bot, op, NULL, 0, 0);
 
     return 0;
 }
@@ -662,8 +917,8 @@ l_int32  w, h;
 /*!
  *  pixSetBorderVal()
  *
- *      Input:  pixs (8 or 32 bpp)
- *              leftpix, rightpix, toppix, bottompix
+ *      Input:  pixs (8, 16 or 32 bpp)
+ *              left, right, top, bot (amount to set)
  *              val (value to set at each border pixel)
  *      Return: 0 if OK; 1 on error
  *
@@ -677,14 +932,14 @@ l_int32  w, h;
  *          you're setting the border to either black or white.
  *      (3) If d != 32, the input value should be masked off
  *          to the appropriate number of least significant bits.
- *      (4) The code is easily generalized for 2, 4 or 16 bpp.
+ *      (4) The code is easily generalized for 2 or 4 bpp.
  */
 l_int32
 pixSetBorderVal(PIX      *pixs,
-                l_int32   leftpix,
-                l_int32   rightpix,
-                l_int32   toppix,
-                l_int32   bottompix,
+                l_int32   left,
+                l_int32   right,
+                l_int32   top,
+                l_int32   bot,
                 l_uint32  val)
 {
 l_int32    w, h, d, wpls, i, j, bstart, rstart;
@@ -695,23 +950,23 @@ l_uint32  *datas, *lines;
     if (!pixs)
         return ERROR_INT("pixs not defined", procName, 1);
     pixGetDimensions(pixs, &w, &h, &d);
-    if (d != 8 && d != 32)
-        return ERROR_INT("depth must be 8 or 32 bpp", procName, 1);
+    if (d != 8 && d != 16 && d != 32)
+        return ERROR_INT("depth must be 8, 16 or 32 bpp", procName, 1);
 
     datas = pixGetData(pixs);
     wpls = pixGetWpl(pixs);
     if (d == 8) {
         val &= 0xff;
-        for (i = 0; i < toppix; i++) {
+        for (i = 0; i < top; i++) {
             lines = datas + i * wpls;
             for (j = 0; j < w; j++)
                 SET_DATA_BYTE(lines, j, val);
         }
-        rstart = w - rightpix;
-        bstart = h - bottompix;
-        for (i = toppix; i < bstart; i++) {
+        rstart = w - right;
+        bstart = h - bot;
+        for (i = top; i < bstart; i++) {
             lines = datas + i * wpls;
-            for (j = 0; j < leftpix; j++)
+            for (j = 0; j < left; j++)
                 SET_DATA_BYTE(lines, j, val);
             for (j = rstart; j < w; j++)
                 SET_DATA_BYTE(lines, j, val);
@@ -722,17 +977,39 @@ l_uint32  *datas, *lines;
                 SET_DATA_BYTE(lines, j, val);
         }
     }
+    else if (d == 16) {
+        val &= 0xffff;
+        for (i = 0; i < top; i++) {
+            lines = datas + i * wpls;
+            for (j = 0; j < w; j++)
+                SET_DATA_TWO_BYTES(lines, j, val);
+        }
+        rstart = w - right;
+        bstart = h - bot;
+        for (i = top; i < bstart; i++) {
+            lines = datas + i * wpls;
+            for (j = 0; j < left; j++)
+                SET_DATA_TWO_BYTES(lines, j, val);
+            for (j = rstart; j < w; j++)
+                SET_DATA_TWO_BYTES(lines, j, val);
+        }
+        for (i = bstart; i < h; i++) {
+            lines = datas + i * wpls;
+            for (j = 0; j < w; j++)
+                SET_DATA_TWO_BYTES(lines, j, val);
+        }
+    }
     else {   /* d == 32 */
-        for (i = 0; i < toppix; i++) {
+        for (i = 0; i < top; i++) {
             lines = datas + i * wpls;
             for (j = 0; j < w; j++)
                 *(lines + j) = val;
         }
-        rstart = w - rightpix;
-        bstart = h - bottompix;
-        for (i = toppix; i < bstart; i++) {
+        rstart = w - right;
+        bstart = h - bot;
+        for (i = top; i < bstart; i++) {
             lines = datas + i * wpls;
-            for (j = 0; j < leftpix; j++)
+            for (j = 0; j < left; j++)
                 *(lines + j) = val;
             for (j = rstart; j < w; j++)
                 *(lines + j) = val;
@@ -748,109 +1025,90 @@ l_uint32  *datas, *lines;
 }
         
 
+/*!
+ *  pixSetMirroredBorder()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot (number of pixels to set)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This applies what is effectively mirror boundary conditions
+ *          to a border region in the image.  It is in-place.
+ *      (2) This is useful for setting pixels near the border to a
+ *          value representative of the near pixels to the interior.
+ *      (3) The general pixRasterop() is used for an in-place operation here
+ *          because there is no overlap between the src and dest rectangles.
+ */
+l_int32
+pixSetMirroredBorder(PIX     *pixs,
+                     l_int32  left,
+                     l_int32  right,
+                     l_int32  top,
+                     l_int32  bot)
+{
+l_int32  i, j, w, h;
+
+    PROCNAME("pixSetMirroredBorder");
+
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    for (j = 0; j < left; j++)
+        pixRasterop(pixs, left - 1 - j, top, 1, h - top - bot, PIX_SRC,
+                    pixs, left + j, top);
+    for (j = 0; j < right; j++)
+        pixRasterop(pixs, w - right + j, top, 1, h - top - bot, PIX_SRC,
+                    pixs, w - right - 1 - j, top);
+    for (i = 0; i < top; i++)
+        pixRasterop(pixs, 0, top - 1 - i, w, 1, PIX_SRC,
+                    pixs, 0, top + i);
+    for (i = 0; i < bot; i++)
+        pixRasterop(pixs, 0, h - bot + i, w, 1, PIX_SRC,
+                    pixs, 0, h - bot - 1 - i);
+
+    return 0;
+}
+     
+
+
 /*-------------------------------------------------------------*
  *                     Add and remove border                   *
  *-------------------------------------------------------------*/
 /*!
  *  pixAddBorder()
  *
- *      Input:  pix
+ *      Input:  pixs (all depths; colormap ok)
  *              npix (number of pixels to be added to each side)
  *              val  (value of added border pixels)
- *      Return: pix with the input pix centered, or null on error.
+ *      Return: pixd (with the input pixs centered), or null on error
  *
  *  Notes:
- *      (1) For binary images:
- *             white:  val = 0
- *             black:  val = 1
- *      (2) For grayscale images:
- *             white:  val = 2 ** d - 1
- *             black:  val = 0
- *      (3) For rgb color images:
- *             white:  val = 0xffffff00
- *             black:  val = 0
+ *      (1) See pixAddBorderGeneral() for values of white & black pixels.
  */
 PIX *
 pixAddBorder(PIX      *pixs,
              l_int32   npix,
              l_uint32  val)
 {
-l_int32  ws, hs, wd, hd, d;
-PIX     *pixd;
-
     PROCNAME("pixAddBorder");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-
     if (npix == 0)
         return pixClone(pixs);
-
-    pixGetDimensions(pixs, &ws, &hs, &d);
-    wd = ws + 2 * npix;
-    hd = hs + 2 * npix;
-    if ((pixd = pixCreate(wd, hd, d)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    pixCopyResolution(pixd, pixs);
-    pixCopyColormap(pixd, pixs);
-
-    pixSetAllArbitrary(pixd, val);   /* a little extra writing ! */
-
-    pixRasterop(pixd, npix, npix, ws, hs, PIX_SRC, pixs, 0, 0);
-
-    return pixd;
-}
-
-
-/*!
- *  pixRemoveBorder()
- *
- *      Input:  pixs
- *              npix (number to be removed from each of the 4 sides)
- *      Return: pixd, or null on error
- */
-PIX *
-pixRemoveBorder(PIX     *pixs,
-                l_int32  npix)
-{
-l_int32  ws, hs, wd, hd, d;
-PIX     *pixd;
-
-    PROCNAME("pixRemoveBorder");
-
-    if (!pixs)
-        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
-
-    if (npix == 0)
-        return pixClone(pixs);
-
-    pixGetDimensions(pixs, &ws, &hs, &d);
-    wd = ws - 2 * npix;
-    hd = hs - 2 * npix;
-    if (wd <= 0)
-        return (PIX *)ERROR_PTR("width must be > 0", procName, NULL);
-    if (hd <= 0)
-        return (PIX *)ERROR_PTR("height must be > 0", procName, NULL);
-    if ((pixd = pixCreate(wd, hd, d)) == NULL)
-        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
-    pixCopyResolution(pixd, pixs);
-    pixCopyColormap(pixd, pixs);
-
-        /* Rasterop from the center */
-    pixRasterop(pixd, 0, 0, wd, hd, PIX_SRC, pixs, npix, npix);
-    
-    return pixd;
+    return pixAddBorderGeneral(pixs, npix, npix, npix, npix, val);
 }
 
 
 /*!
  *  pixAddBorderGeneral()
  *
- *      Input:  pix
- *              leftpix, rightpix, toppix, bottompix  (number of pixels
- *                   to be added to each side)
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot  (number of pixels added)
  *              val   (value of added border pixels)
- *      Return: pix with the input pix placed properly, or null on error
+ *      Return: pixd (with the input pixs inserted), or null on error
  *
  *  Notes:
  *      (1) For binary images:
@@ -865,50 +1123,87 @@ PIX     *pixd;
  */
 PIX *
 pixAddBorderGeneral(PIX      *pixs,
-                    l_int32   leftpix,
-                    l_int32   rightpix,
-                    l_int32   toppix,
-                    l_int32   bottompix,
+                    l_int32   left,
+                    l_int32   right,
+                    l_int32   top,
+                    l_int32   bot,
                     l_uint32  val)
 {
-l_int32  ws, hs, wd, hd, d;
+l_int32  ws, hs, wd, hd, d, op;
 PIX     *pixd;
 
     PROCNAME("pixAddBorderGeneral");
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (left < 0 || right < 0 || top < 0 || bot < 0)
+        return (PIX *)ERROR_PTR("negative border added!", procName, NULL);
 
     pixGetDimensions(pixs, &ws, &hs, &d);
-    wd = ws + leftpix + rightpix;
-    hd = hs + toppix + bottompix;
-    if ((pixd = pixCreate(wd, hd, d)) == NULL)
+    wd = ws + left + right;
+    hd = hs + top + bot;
+    if ((pixd = pixCreateNoInit(wd, hd, d)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     pixCopyResolution(pixd, pixs);
     pixCopyColormap(pixd, pixs);
 
-    pixSetAllArbitrary(pixd, val);   /* a little extra writing ! */
-
-    pixRasterop(pixd, leftpix, toppix, ws, hs, PIX_SRC, pixs, 0, 0);
+        /* Set the new border pixels */
+    op = UNDEF;
+    if (val == 0)
+        op = PIX_CLR;
+    else if ((d == 1 && val == 1) || (d == 2 && val == 3) ||
+             (d == 4 && val == 0xf) || (d == 8 && val == 0xff) || 
+             (d == 32 && (val >> 8) == 0xffffff))
+        op = PIX_SET;
+    if (op == UNDEF)
+        pixSetAllArbitrary(pixd, val);   /* a little extra writing ! */
+    else {
+        pixRasterop(pixd, 0, 0, left, hd, op, NULL, 0, 0); 
+        pixRasterop(pixd, wd - right, 0, right, hd, op, NULL, 0, 0); 
+        pixRasterop(pixd, 0, 0, wd, top, op, NULL, 0, 0); 
+        pixRasterop(pixd, 0, hd - bot, wd, bot, op, NULL, 0, 0); 
+    }
 
+        /* Copy pixs into the interior */
+    pixRasterop(pixd, left, top, ws, hs, PIX_SRC, pixs, 0, 0);
     return pixd;
 }
 
 
+/*!
+ *  pixRemoveBorder()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              npix (number to be removed from each of the 4 sides)
+ *      Return: pixd (with pixels removed around border), or null on error
+ */
+PIX *
+pixRemoveBorder(PIX     *pixs,
+                l_int32  npix)
+{
+    PROCNAME("pixRemoveBorder");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (npix == 0)
+        return pixClone(pixs);
+    return pixRemoveBorderGeneral(pixs, npix, npix, npix, npix);
+}
+
+
 /*!
  *  pixRemoveBorderGeneral()
  *
- *      Input:  pixs
- *              leftpix, rightpix, toppix, bottompix  (number of pixels
- *                   to be removed from each side)
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot  (number of pixels added)
  *      Return: pixd (with pixels removed around border), or null on error
  */
 PIX *
 pixRemoveBorderGeneral(PIX     *pixs,
-                       l_int32  leftpix,
-                       l_int32  rightpix,
-                       l_int32  toppix,
-                       l_int32  bottompix)
+                       l_int32  left,
+                       l_int32  right,
+                       l_int32  top,
+                       l_int32  bot)
 {
 l_int32  ws, hs, wd, hd, d;
 PIX     *pixd;
@@ -917,25 +1212,83 @@ PIX     *pixd;
 
     if (!pixs)
         return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if (left < 0 || right < 0 || top < 0 || bot < 0)
+        return (PIX *)ERROR_PTR("negative border removed!", procName, NULL);
 
     pixGetDimensions(pixs, &ws, &hs, &d);
-    wd = ws - leftpix - rightpix;
-    hd = hs - toppix - bottompix;
+    wd = ws - left - right;
+    hd = hs - top - bot;
     if (wd <= 0)
         return (PIX *)ERROR_PTR("width must be > 0", procName, NULL);
     if (hd <= 0)
         return (PIX *)ERROR_PTR("height must be > 0", procName, NULL);
-    if ((pixd = pixCreate(wd, hd, d)) == NULL)
+    if ((pixd = pixCreateNoInit(wd, hd, d)) == NULL)
         return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
     pixCopyResolution(pixd, pixs);
     pixCopyColormap(pixd, pixs);
 
-    pixRasterop(pixd, 0, 0, wd, hd, PIX_SRC, pixs, leftpix, toppix);
-    
+    pixRasterop(pixd, 0, 0, wd, hd, PIX_SRC, pixs, left, top);
     return pixd;
 }
 
 
+/*!
+ *  pixAddMirroredBorder()
+ *
+ *      Input:  pixs (all depths; colormap ok)
+ *              left, right, top, bot (number of pixels added)
+ *      Return: pixd, or null on error
+ *
+ *  Notes:
+ *      (1) This applies what is effectively mirror boundary conditions.
+ *          For the added border pixels in pixd, the pixels in pixs
+ *          near the border are mirror-copied into the border region.
+ *      (2) This is useful for avoiding special operations near
+ *          boundaries when doing image processing operations
+ *          such as rank filters and convolution.  In use, one first
+ *          adds mirrored pixels to each side of the image.  The number
+ *          of pixels added on each side is half the filter dimension.
+ *          Then the image processing operations proceed over a
+ *          region equal to the size of the original image, and
+ *          write directly into a dest pix of the same size as pixs.
+ *      (3) The general pixRasterop() is used for an in-place operation here
+ *          because there is no overlap between the src and dest rectangles.
+ */
+PIX  *
+pixAddMirroredBorder(PIX      *pixs,
+                      l_int32  left,
+                      l_int32  right,
+                      l_int32  top,
+                      l_int32  bot)
+{
+l_int32  i, j, w, h;
+PIX     *pixd;
+
+    PROCNAME("pixAddMirroredBorder");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    pixd = pixAddBorderGeneral(pixs, left, right, top, bot, 0);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    for (j = 0; j < left; j++)
+        pixRasterop(pixd, left - 1 - j, top, 1, h, PIX_SRC,
+                    pixd, left + j, top);
+    for (j = 0; j < right; j++)
+        pixRasterop(pixd, left + w + j, top, 1, h, PIX_SRC,
+                    pixd, left + w - 1 - j, top);
+    for (i = 0; i < top; i++)
+        pixRasterop(pixd, 0, top - 1 - i, left + w + right, 1, PIX_SRC,
+                    pixd, 0, top + i);
+    for (i = 0; i < bot; i++)
+        pixRasterop(pixd, 0, top + h + i, left + w + right, 1, PIX_SRC,
+                    pixd, 0, top + h - 1 - i);
+
+    return pixd;
+}
+     
+
+
 /*-------------------------------------------------------------*
  *                Color sample setting and extraction          *
  *-------------------------------------------------------------*/
@@ -1108,12 +1461,95 @@ l_uint32  *datas, *datad;
 }
 
 
+/*!
+ *  pixGetRGBComponentCmap()
+ *
+ *      Input:  pixs  (colormapped)
+ *              color  (one of {COLOR_RED, COLOR_GREEN, COLOR_BLUE})
+ *      Return: pixd  (the selected 8 bpp component image of the
+ *                     input cmapped image), or null on error
+ */
+PIX *
+pixGetRGBComponentCmap(PIX     *pixs,
+                       l_int32  color)
+{
+l_int32     i, j, w, h, val, index;
+l_int32     wplc, wpld;
+l_uint32   *linec, *lined;
+l_uint32   *datac, *datad;
+PIX        *pixc, *pixd;
+PIXCMAP    *cmap;
+RGBA_QUAD  *cta;
+
+    PROCNAME("pixGetRGBComponentCmap");
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+    if ((cmap = pixGetColormap(pixs)) == NULL)
+        return (PIX *)ERROR_PTR("pixs not cmapped", procName, NULL);
+    if (color != COLOR_RED && color != COLOR_GREEN &&
+        color != COLOR_BLUE)
+        return (PIX *)ERROR_PTR("invalid color", procName, NULL);
+
+        /* If not 8 bpp, make a cmapped 8 bpp pix */
+    if (pixGetDepth(pixs) == 8)
+        pixc = pixClone(pixs);
+    else
+        pixc = pixConvertTo8(pixs, TRUE);
+
+    pixGetDimensions(pixs, &w, &h, NULL);
+    if ((pixd = pixCreateNoInit(w, h, 8)) == NULL)
+        return (PIX *)ERROR_PTR("pixd not made", procName, NULL);
+    pixCopyResolution(pixd, pixs);
+    wplc = pixGetWpl(pixc);
+    wpld = pixGetWpl(pixd);
+    datac = pixGetData(pixc);
+    datad = pixGetData(pixd);
+    cta = (RGBA_QUAD *)cmap->array;
+
+    for (i = 0; i < h; i++) {
+        linec = datac + i * wplc;
+        lined = datad + i * wpld;
+        if (color == COLOR_RED) {
+            for (j = 0; j < w; j++) {
+                index = GET_DATA_BYTE(linec, j);
+                val = cta[index].red;
+                SET_DATA_BYTE(lined, j, val);
+            }
+        }
+        else if (color == COLOR_GREEN) {
+            for (j = 0; j < w; j++) {
+                index = GET_DATA_BYTE(linec, j);
+                val = cta[index].green;
+                SET_DATA_BYTE(lined, j, val);
+            }
+        }
+        else if (color == COLOR_BLUE) {
+            for (j = 0; j < w; j++) {
+                index = GET_DATA_BYTE(linec, j);
+                val = cta[index].green;
+                SET_DATA_BYTE(lined, j, val);
+            }
+        }
+    }
+
+    pixDestroy(&pixc);
+    return pixd;
+}
+
+
 /*!
  *  composeRGBPixel()
  *
  *      Input:  rval, gval, bval
  *              &rgbpixel  (<return> 32-bit pixel)
  *      Return: 0 if OK; 1 on error
+ *
+ *  Notes:
+ *      (1) A slower implementation uses macros:
+ *            SET_DATA_BYTE(ppixel, COLOR_RED, rval);
+ *            SET_DATA_BYTE(ppixel, COLOR_GREEN, gval);
+ *            SET_DATA_BYTE(ppixel, COLOR_BLUE, bval);
  */
 l_int32
 composeRGBPixel(l_int32    rval,
@@ -1126,14 +1562,40 @@ composeRGBPixel(l_int32    rval,
     if (!ppixel)
         return ERROR_INT("&pixel not defined", procName, 1);
 
-    *ppixel = 0;  /* want the alpha byte to be 0 */
-    SET_DATA_BYTE(ppixel, COLOR_RED, rval);
-    SET_DATA_BYTE(ppixel, COLOR_GREEN, gval);
-    SET_DATA_BYTE(ppixel, COLOR_BLUE, bval);
+    *ppixel = (rval << L_RED_SHIFT) | (gval << L_GREEN_SHIFT) |
+              (bval << L_BLUE_SHIFT);
     return 0;
 }
 
 
+/*!
+ *  extractRGBValues()
+ *
+ *      Input:  pixel (32 bit)
+ *              &rval (<optional return> red component)
+ *              &gval (<optional return> green component)
+ *              &bval (<optional return> blue component)
+ *      Return: void
+ *
+ *  Notes:
+ *      (1) A slower implementation uses macros:
+ *             *prval = GET_DATA_BYTE(&pixel, COLOR_RED);
+ *             *pgval = GET_DATA_BYTE(&pixel, COLOR_GREEN);
+ *             *pbval = GET_DATA_BYTE(&pixel, COLOR_BLUE);
+ */
+void
+extractRGBValues(l_uint32  pixel,
+                 l_int32  *prval,
+                 l_int32  *pgval,
+                 l_int32  *pbval)
+{
+    if (prval) *prval = (pixel >> L_RED_SHIFT) & 0xff;
+    if (pgval) *pgval = (pixel >> L_GREEN_SHIFT) & 0xff;
+    if (pbval) *pbval = (pixel >> L_BLUE_SHIFT) & 0xff;
+    return;
+}
+
+
 /*!
  *  pixGetRGBLine()
  *
@@ -1253,7 +1715,7 @@ PIX       *pixd;
 /*!
  *  pixEndianByteSwap()
  *
- *      Input:  pix
+ *      Input:  pixs
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -1271,7 +1733,7 @@ PIX       *pixd;
  *          order when serialized, so no byte flipping is required.
  */
 l_int32
-pixEndianByteSwap(PIX  *pix)
+pixEndianByteSwap(PIX  *pixs)
 {
 l_uint32  *data;
 l_int32    i, j, h, wpl;
@@ -1285,12 +1747,12 @@ l_uint32   word;
 
 #else   /* L_LITTLE_ENDIAN */
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
 
-    data = pixGetData(pix);
-    wpl = pixGetWpl(pix);
-    h = pixGetHeight(pix);
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+    h = pixGetHeight(pixs);
     for (i = 0; i < h; i++) {
         for (j = 0; j < wpl; j++, data++) {
             word = *data;
@@ -1308,10 +1770,65 @@ l_uint32   word;
 }
 
 
+/*!
+ *  pixEndianTwoByteSwapNew()
+ *
+ *      Input:  pixs
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) This is used on little-endian platforms to swap the
+ *          2-byte entities within a 32-bit word.
+ *      (2) This is equivalent to a full byte swap, as performed
+ *          by pixEndianByteSwap(), followed by byte swaps in
+ *          each of the 16-bit entities separately.
+ *      (3) Unlike pixEndianTwoByteSwap(), which swaps the shorts in-place,
+ *          this returns a new pix (or a clone).  We provide this
+ *          to avoid having to swap twice in situations where the input
+ *          pix must be restored to canonical little-endian order.
+ */
+PIX *
+pixEndianTwoByteSwapNew(PIX  *pixs)
+{
+l_uint32  *datas, *datad;
+l_int32    i, j, h, wpl;
+l_uint32   word;
+PIX       *pixd;
+
+    PROCNAME("pixEndianTwoByteSwapNew");
+        
+#ifdef L_BIG_ENDIAN
+
+    return pixClone(pixs);
+
+#else   /* L_LITTLE_ENDIAN */
+
+    if (!pixs)
+        return (PIX *)ERROR_PTR("pixs not defined", procName, NULL);
+
+    datas = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+    h = pixGetHeight(pixs);
+    pixd = pixCreateTemplate(pixs);
+    datad = pixGetData(pixd);
+    for (i = 0; i < h; i++) {
+        for (j = 0; j < wpl; j++, datas++, datad++) {
+            word = *datas;
+            *datad = (word << 16) | (word >> 16);
+        }
+    }
+
+    return pixd;
+
+#endif   /* L_BIG_ENDIAN */
+
+}
+
+
 /*!
  *  pixEndianTwoByteSwap()
  *
- *      Input:  pix
+ *      Input:  pixs
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -1322,7 +1839,7 @@ l_uint32   word;
  *          each of the 16-bit entities separately.
  */
 l_int32
-pixEndianTwoByteSwap(PIX  *pix)
+pixEndianTwoByteSwap(PIX  *pixs)
 {
 l_uint32  *data;
 l_int32    i, j, h, wpl;
@@ -1336,17 +1853,16 @@ l_uint32   word;
 
 #else   /* L_LITTLE_ENDIAN */
 
-    if (!pix)
-        return ERROR_INT("pix not defined", procName, 1);
+    if (!pixs)
+        return ERROR_INT("pixs not defined", procName, 1);
 
-    data = pixGetData(pix);
-    wpl = pixGetWpl(pix);
-    h = pixGetHeight(pix);
+    data = pixGetData(pixs);
+    wpl = pixGetWpl(pixs);
+    h = pixGetHeight(pixs);
     for (i = 0; i < h; i++) {
         for (j = 0; j < wpl; j++, data++) {
             word = *data;
-            *data = ((word << 16) & 0xffff0000) |
-                    ((word >> 16) & 0x0000ffff);
+            *data = (word << 16) | (word >> 16);
         }
     }
 
