@@ -16,16 +16,19 @@
 /*
  *  pngio.c
  *                     
- *    Reading png:
+ *    Read png from file
  *          PIX        *pixReadStreamPng()
  *          l_int32     readHeaderPng()
  *          l_int32     freadHeaderPng()
  *          l_int32     sreadHeaderPng()
  *
- *    Writing png:
+ *    Write png to file
  *          l_int32     pixWritePng()  [ special top level ]
  *          l_int32     pixWriteStreamPng()
  *          
+ *    Read/write to memory   [not on windows]
+ *          PIX        *pixReadMemPng()
+ *          l_int32     pixWriteMemPng()
  *
  *    Documentation: libpng.txt and example.c
  *
@@ -34,16 +37,38 @@
  *    3 component color images are read into a 32 bpp Pix with
  *    rgb samples.  On output (compression to file), palette color
  *    images are written as 8 bpp with the colormap, and 32 bpp
- *    full color images are written compressed as a set of three
- *    8 bpp (rgb) images.
+ *    full color images are written compressed as a 24 bpp,
+ *    3 component color image.
+ *
+ *    Note these particular limitations, which are invoked by
+ *    two transform flags in pixReadStreamPng():
+ *
+ *       bpc == bit/component
+ *       cpp == component/pixel
+ *       bpp == bits/pixel of image in Pix (memory)
+ *
+ *    (1) 16 bpc input images are read into a Pix with 8 bpc.
+ *        - For 16 bpc rgb (16 bpc, 3 cpp) --> 32 bpp rgb Pix
+ *        - For 16 bpc gray (16 bpc, 1 cpp) --> 8 bpp grayscale Pix
+ *    (2) The alpha layer is stripped out
+ *        - For 8 bpc rgba (8 bpc, 4 cpp) --> 32 bpp rgb Pix
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include "allheaders.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config_auto.h"
+#endif  /* HAVE_CONFIG_H */
+
+/* --------------------------------------------*/
+#if  HAVE_LIBPNG   /* defined in environ.h */
+/* --------------------------------------------*/
+
+#include "png.h"
+
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG     0
 #endif  /* ~NO_CONSOLE_IO */
@@ -58,7 +83,7 @@
  *      Input:  stream
  *      Return: pix, or null on error
  *
- *  Usage notes:
+ *  Notes:
  *      (1) If called from pixReadStream(), the stream is positioned
  *          at the beginning of the file.
  *      (2) To do sequential reads of png format images from a stream,
@@ -68,11 +93,11 @@ PIX *
 pixReadStreamPng(FILE  *fp)
 {
 l_uint8      rval, gval, bval;
-l_int32             i, j, k;
+l_int32      i, j, k;
 l_int32      wpl, d, spp, cindex;
 l_uint32     png_transforms;
 l_uint32    *data, *line, *ppixel;
-int          num_palette;
+int          num_palette, num_text;
 png_byte     bit_depth, color_type, channels;
 png_uint_32  w, h, rowbytes;
 png_uint_32  xres, yres;
@@ -81,6 +106,7 @@ png_bytep   *row_pointers;
 png_structp  png_ptr;
 png_infop    info_ptr, end_info;
 png_colorp   palette;
+png_textp    text_ptr;  /* ptr to text_chunk */
 PIX         *pix;
 PIXCMAP     *cmap;
 
@@ -117,7 +143,7 @@ PIXCMAP     *cmap;
          * DO NOT invert binary using PNG_TRANSFORM_INVERT_MONO!!
          * To remove alpha channel, use PNG_TRANSFORM_STRIP_ALPHA
          * To strip 16 --> 8 bit depth, use PNG_TRANSFORM_STRIP_16 */
-#if 0  /* this does both */
+#if 1  /* this does both */
     png_transforms = PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_STRIP_ALPHA;
 #else  /* this just strips alpha */
     png_transforms = PNG_TRANSFORM_STRIP_ALPHA;
@@ -133,7 +159,6 @@ PIXCMAP     *cmap;
     rowbytes = png_get_rowbytes(png_ptr, info_ptr);
     color_type = png_get_color_type(png_ptr, info_ptr);
     channels = png_get_channels(png_ptr, info_ptr);
-
     spp = channels;
 
     if (spp == 1)
@@ -242,8 +267,12 @@ PIXCMAP     *cmap;
     pixSetXRes(pix, (l_int32)((l_float32)xres / 39.37 + 0.5));  /* to ppi */
     pixSetYRes(pix, (l_int32)((l_float32)yres / 39.37 + 0.5));  /* to ppi */
 
-    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+        /* Get the text if there is any */
+    png_get_text(png_ptr, info_ptr, &text_ptr, &num_text);
+    if (num_text && text_ptr)
+        pixSetText(pix, text_ptr->text);
 
+    png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
     return pix;
 }
 
@@ -256,10 +285,11 @@ PIXCMAP     *cmap;
  *              &height (<return>)
  *              &bpc (<return>, bits/component)
  *              &cpp (<return>, components/pixel)
- *              &cmap (<optional return>; input NULL to ignore)
+ *              &iscmap (<optional return>; input NULL to ignore)
  *      Return: 0 if OK, 1 on error
  *
- *  Note: if there is a colormap, cmap is returned as 1; else 0.
+ *  Notes:
+ *      (1) If there is a colormap, iscmap is returned as 1; else 0.
  */
 l_int32
 readHeaderPng(const char *filename,
@@ -267,7 +297,7 @@ readHeaderPng(const char *filename,
               l_int32    *pheight,
               l_int32    *pbpc,
               l_int32    *pcpp,
-              l_int32    *pcmap)
+              l_int32    *piscmap)
 {
 l_int32  ret;
 FILE    *fp;
@@ -280,7 +310,7 @@ FILE    *fp;
         return ERROR_INT("input ptr(s) not defined", procName, 1);
     if ((fp = fopenReadStream(filename)) == NULL)
         return ERROR_INT("image file not found", procName, 1);
-    ret = freadHeaderPng(fp, pwidth, pheight, pbpc, pcpp, pcmap);
+    ret = freadHeaderPng(fp, pwidth, pheight, pbpc, pcpp, piscmap);
     fclose(fp);
     return ret;
 }
@@ -294,10 +324,11 @@ FILE    *fp;
  *              &height (<return>)
  *              &bpc (<return>, bits/component)
  *              &cpp (<return>, components/pixel)
- *              &cmap (<optional return>; input NULL to ignore)
+ *              &iscmap (<optional return>; input NULL to ignore)
  *      Return: 0 if OK, 1 on error
  *
- *  Note: if there is a colormap, cmap is returned as 1; else 0.
+ *  Notes:
+ *      (1) If there is a colormap, iscmap is returned as 1; else 0.
  */
 l_int32
 freadHeaderPng(FILE     *fp,
@@ -305,7 +336,7 @@ freadHeaderPng(FILE     *fp,
                l_int32  *pheight,
                l_int32  *pbpc,
                l_int32  *pcpp,
-               l_int32  *pcmap)
+               l_int32  *piscmap)
 {
 l_int32   nbytes, ret;
 l_uint8  *data;
@@ -317,13 +348,13 @@ l_uint8  *data;
     if (!pwidth || !pheight || !pbpc || !pcpp)
         return ERROR_INT("input ptr(s) not defined", procName, 1);
     
-    nbytes = nbytesInFile(fp);
+    nbytes = fnbytesInFile(fp);
     if (nbytes < 40)
         return ERROR_INT("file too small to be png", procName, 1);
     if ((data = (l_uint8 *)CALLOC(40, sizeof(l_uint8))) == NULL)
         return ERROR_INT("CALLOC fail for data", procName, 1);
     fread(data, 40, 1, fp);
-    ret = sreadHeaderPng(data, pwidth, pheight, pbpc, pcpp, pcmap);
+    ret = sreadHeaderPng(data, pwidth, pheight, pbpc, pcpp, piscmap);
     FREE(data);
     return ret;
 }
@@ -337,10 +368,11 @@ l_uint8  *data;
  *              &height (<return>)
  *              &bpc (<return>, bits/component)
  *              &cpp (<return>, components/pixel)
- *              &cmap (<optional return>; input NULL to ignore)
+ *              &iscmap (<optional return>; input NULL to ignore)
  *      Return: 0 if OK, 1 on error
  *
- *  Note: if there is a colormap, cmap is returned as 1; else 0.
+ *  Notes:
+ *      (1) If there is a colormap, iscmap is returned as 1; else 0.
  */
 l_int32
 sreadHeaderPng(const l_uint8  *data,
@@ -348,7 +380,7 @@ sreadHeaderPng(const l_uint8  *data,
                l_int32        *pheight,
                l_int32        *pbpc,
                l_int32        *pcpp,
-               l_int32        *pcmap)
+               l_int32        *piscmap)
 {
 l_uint8    colortype, bpc;
 l_uint16   twobytes;
@@ -362,8 +394,8 @@ l_uint32  *pword;
     if (!pwidth || !pheight || !pbpc || !pcpp)
         return ERROR_INT("input ptr(s) not defined", procName, 1);
     *pwidth = *pheight = *pbpc = *pcpp = 0;
-    if (pcmap)
-      *pcmap = 0;
+    if (piscmap)
+      *piscmap = 0;
     
         /* Check password */
     if (data[0] != 137 || data[1] != 80 || data[2] != 78 ||
@@ -386,11 +418,11 @@ l_uint32  *pword;
         *pcpp = 4;
     else   /* palette or gray */
         *pcpp = 1;
-    if (pcmap) {
+    if (piscmap) {
         if (colortype & 1)  /* palette: see png.h, PNG_COLOR_TYPE_... */
-            *pcmap = 1;
+            *piscmap = 1;
         else
-            *pcmap = 0;
+            *piscmap = 0;
     }
 
     return 0;
@@ -447,7 +479,7 @@ FILE  *fp;
  *              gamma (use 0.0 if gamma is not defined)
  *      Return: 0 if OK; 1 on error
  *
- *  Usage notes:
+ *  Notes:
  *      (1) If called from pixWriteStream(), the stream is positioned
  *          at the beginning of the file.
  *      (2) To do sequential writes of png format images to a stream,
@@ -494,7 +526,7 @@ pixWriteStreamPng(FILE      *fp,
                   PIX       *pix,
                   l_float32  gamma)
 {
-l_int32             i, j, k;
+l_int32      i, j, k;
 l_int32      wpl, d, cmflag;
 l_int32      ncolors;
 l_int32     *rmap, *gmap, *bmap;
@@ -588,7 +620,6 @@ char        *text;
         }
 
         png_set_PLTE(png_ptr, info_ptr, palette, (int)ncolors);
-
         FREE(rmap);
         FREE(gmap);
         FREE(bmap);
@@ -613,6 +644,7 @@ char        *text;
 #endif
         png_set_text(png_ptr, info_ptr, &text_chunk, 1);
     }
+
         /* Write header and palette info */
     png_write_info(png_ptr, info_ptr);
 
@@ -688,4 +720,111 @@ char        *text;
 }
 
 
+/*---------------------------------------------------------------------*
+ *                         Read/write to memory                        *
+ *---------------------------------------------------------------------*/
+#if HAVE_FMEMOPEN || \
+ (!defined(__MINGW32__) && !defined(_CYGWIN_ENVIRON) && !defined(_STANDARD_C_))
+
+extern FILE *open_memstream(char **data, size_t *size);
+extern FILE *fmemopen(void *data, size_t size, const char *mode);
+
+/*!
+ *  pixReadMemPng()
+ *
+ *      Input:  cdata (const; png-encoded)
+ *              size (of data)
+ *      Return: pix, or null on error
+ *
+ *  Notes:
+ *      (1) The @size byte of @data must be a null character.
+ */
+PIX *
+pixReadMemPng(const l_uint8  *cdata,
+              size_t          size)
+{
+l_uint8  *data;
+FILE     *fp;
+PIX      *pix;
+
+    PROCNAME("pixReadMemPng");
+
+    if (!cdata)
+        return (PIX *)ERROR_PTR("cdata not defined", procName, NULL);
+
+    data = (l_uint8 *)cdata;  /* we're really not going to change this */
+    if ((fp = fmemopen(data, size, "r")) == NULL)
+        return (PIX *)ERROR_PTR("stream not opened", procName, NULL);
+    pix = pixReadStreamPng(fp);
+    fclose(fp);
+    return pix;
+}
+
+
+/*!
+ *  pixWriteMemPng()
+ *
+ *      Input:  &data (<return> data of tiff compressed image)
+ *              &size (<return> size of returned data)
+ *              pix
+ *              gamma (use 0.0 if gamma is not defined)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) See pixWriteStreamPng() for usage.  This version writes to
+ *          memory instead of to a file stream.
+ */
+l_int32
+pixWriteMemPng(l_uint8  **pdata,
+               size_t    *psize,
+               PIX       *pix,
+               l_float32  gamma)
+{
+l_int32  ret;
+FILE    *fp;
+
+    PROCNAME("pixWriteMemPng");
+
+    if (!pdata)
+        return ERROR_INT("&data not defined", procName, 1 );
+    if (!psize)
+        return ERROR_INT("&size not defined", procName, 1 );
+    if (!pix)
+        return ERROR_INT("&pix not defined", procName, 1 );
+
+    if ((fp = open_memstream((char **)pdata, psize)) == NULL)
+        return ERROR_INT("stream not opened", procName, 1);
+    ret = pixWriteStreamPng(fp, pix, gamma);
+    fclose(fp);
+    return ret;
+}
+
+#else
+
+PIX *
+pixReadMemPng(const l_uint8  *cdata,
+              size_t          size)
+{
+    return (PIX *)ERROR_PTR(
+        "png read from memory not implemented on this platform",
+        "pixReadMemPng", NULL);
+}
+
+
+l_int32
+pixWriteMemPng(l_uint8  **pdata,
+               size_t    *psize,
+               PIX       *pix,
+               l_float32  gamma)
+{
+    return ERROR_INT(
+        "png write to memory not implemented on this platform",
+        "pixWriteMemPng", 1);
+}
+
+#endif  /* HAVE_FMEMOPEN || (!defined(__MINGW32__) && etc ) */
+
+/* --------------------------------------------*/
+#endif  /* HAVE_LIBPNG */
+/* --------------------------------------------*/
 
