@@ -13,31 +13,46 @@
  -  or altered from any source or modified source distribution.
  *====================================================================*/
 
-#include <stdio.h>
-#include "allheaders.h"
-
 /*
  *  pnmiostub.c
  *
  *     Stubs for pnmio.c functions
  */
 
-PIX * pixReadStreamPnm ( FILE *fp )
+#include <stdio.h>
+#include "allheaders.h"
+
+/* --------------------------------------------*/
+#if  !USE_PNMIO   /* defined in environ.h */
+/* --------------------------------------------*/
+
+PIX * pixReadStreamPnm(FILE *fp)
 {
-    PROCNAME("pixReadStreamPnm");
-    return (PIX * )ERROR_PTR("function not present", procName, NULL);
+    return (PIX * )ERROR_PTR("function not present", "pixReadStreamPnm", NULL);
 }
 
-l_int32 pixWriteStreamPnm ( FILE *fp, PIX *pix )
+l_int32 pixWriteStreamPnm(FILE *fp, PIX *pix)
 {
-    PROCNAME("pixWriteStreamPnm");
-    return ERROR_INT("function not present", procName, 1);
+    return ERROR_INT("function not present", "pixWriteStreamPnm", 1);
 }
 
-l_int32 pixWriteStreamAsciiPnm ( FILE *fp, PIX *pix )
+l_int32 pixWriteStreamAsciiPnm(FILE *fp, PIX *pix)
 {
-    PROCNAME("pixWriteStreamAsciiPnm");
-    return ERROR_INT("function not present", procName, 1);
+    return ERROR_INT("function not present", "pixWriteStreamAsciiPnm", 1);
+}
+
+PIX * pixReadMemPnm(const l_uint8 *cdata, size_t size)
+{
+    return (PIX * )ERROR_PTR("function not present", "pixReadMemPnm", NULL);
 }
 
 
+l_int32 pixWriteMemPnm(l_uint8 **pdata, size_t *psize, PIX *pix)
+{
+    return ERROR_INT("function not present", "pixWritememPnm", 1);
+}
+
+/* --------------------------------------------*/
+#endif  /* !USE_PNMIO */
+/* --------------------------------------------*/
+
