@@ -69,11 +69,13 @@
  *
  *      In use
  *           BOX      *ptaGetExtent()
+ *           PTA      *ptaGetInsideBox()
  *           PTA      *pixFindCornerPixels()
  *           l_int32   pixPlotAlongPta()
  *           l_int32   ptaContainsPt()
  *           l_int32   ptaTestIntersection()
  *           PTA      *ptaTransform()
+ *           PTA      *ptaSubsample()
  *           l_int32   ptaGetLinearLSF()
  *           PTA      *ptaGetPixelsFromPix()
  *           PIX      *pixGenerateFromPta()
@@ -81,6 +83,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include "allheaders.h"
 
 static const l_int32  INITIAL_PTR_ARRAYSIZE = 20;   /* n'import quoi */
@@ -171,7 +174,7 @@ ptaCopy(PTA  *pta)
 {
 l_int32    i;
 l_float32  x, y;
-PTA  *npta;
+PTA       *npta;
 
     PROCNAME("ptaCopy");
 
@@ -1248,6 +1251,42 @@ l_int32  n, i, x, y, minx, maxx, miny, maxy;
 }
 
 
+/*!
+ *  ptaGetInsideBox()
+ *
+ *      Input:  ptas (input pts)
+ *              box
+ *      Return: ptad (of pts in ptas that are inside the box),
+ *              or null on error
+ */
+PTA *
+ptaGetInsideBox(PTA  *ptas,
+                BOX  *box)
+{
+PTA       *ptad;
+l_int32    n, i, contains;
+l_float32  x, y;
+
+    PROCNAME("ptaGetInsideBox");
+
+    if (!ptas)
+        return (PTA *)ERROR_PTR("ptas not defined", procName, NULL);
+    if (!box)
+        return (PTA *)ERROR_PTR("box not defined", procName, NULL);
+
+    n = ptaGetCount(ptas);
+    ptad = ptaCreate(0);
+    for (i = 0; i < n; i++) {
+        ptaGetPt(ptas, i, &x, &y);
+        boxContainsPt(box, x, y, &contains);
+        if (contains)
+            ptaAddPt(ptad, x, y);
+    }
+
+    return ptad;
+}
+
+
 /*!
  *  pixFindCornerPixels()
  *
@@ -1549,6 +1588,40 @@ PTA     *ptad;
 }
 
 
+/*!
+ *  ptaSubsample()
+ *
+ *      Input:  ptas 
+ *              subfactor (subsample factor, >= 1)
+ *      Return: ptad (evenly sampled pt values from ptas, or null on error
+ */
+PTA *
+ptaSubsample(PTA     *ptas,
+             l_int32  subfactor)
+{
+l_int32    n, i;
+l_float32  x, y;
+PTA       *ptad;
+
+    PROCNAME("pixSubsample");
+
+    if (!ptas)
+        return (PTA *)ERROR_PTR("ptas not defined", procName, NULL);
+    if (subfactor < 1)
+        return (PTA *)ERROR_PTR("subfactor < 1", procName, NULL);
+
+    ptad = ptaCreate(0);
+    n = ptaGetCount(ptas);
+    for (i = 0; i < n; i++) {
+        if (i % subfactor != 0) continue;
+        ptaGetPt(ptas, i, &x, &y);
+        ptaAddPt(ptad, x, y);
+    }
+
+    return ptad;
+}
+
+
 /*!
  *  ptaGetLinearLSF()
  *
