@@ -17,15 +17,20 @@
 /*
  *  readfile.c:  reads image on file into memory
  *
- *      High level read functions
- *           PIXA      *pixaReadFiles()
+ *      Top-level functions for reading images from file
+ *           PIXA      *pixaReadFiles()     [unix only]
+ *           PIXA      *pixaReadFilesSA()
  *           PIX       *pixRead()
  *           PIX       *pixReadWithHint()
  *           PIX       *pixReadStream()
  *
- *      Format helper
+ *      Format finders
  *           l_int32    findFileFormat()
  *           l_int32    findFileFormatBuffer()
+ *           l_int32    fileFormatIsTiff()
+ *
+ *      Read from memory
+ *           PIX       *pixReadMem()
  *
  *      Test function for I/O with different formats 
  *           l_int32    ioFormatTest()
@@ -46,86 +51,90 @@ enum {
 
     /* Output files for ioFormatTest().
      * Note that the test for jpeg is not yet implemented */
-static const char *FILE_BMP =  "/usr/tmp/junkout.bmp";
-static const char *FILE_PNG =  "/usr/tmp/junkout.png";
-static const char *FILE_PNM =  "/usr/tmp/junkout.pnm";
-static const char *FILE_G4 =   "/usr/tmp/junkout_g4.tif";
-static const char *FILE_TIFF = "/usr/tmp/junkout.tif";
-static const char *FILE_JPG =  "/usr/tmp/junkout.jpg"; 
+static const char *FILE_BMP  =  "/usr/tmp/junkout.bmp";
+static const char *FILE_PNG  =  "/usr/tmp/junkout.png";
+static const char *FILE_PNM  =  "/usr/tmp/junkout.pnm";
+static const char *FILE_G3   =  "/usr/tmp/junkout_g3.tif";
+static const char *FILE_G4   =  "/usr/tmp/junkout_g4.tif";
+static const char *FILE_RLE  =  "/usr/tmp/junkout_rle.tif";
+static const char *FILE_PB   =  "/usr/tmp/junkout_packbits.tif";
+static const char *FILE_LZW  =  "/usr/tmp/junkout_lzw.tif";
+static const char *FILE_ZIP  =  "/usr/tmp/junkout_zip.tif";
+static const char *FILE_TIFF =  "/usr/tmp/junkout.tif";
+static const char *FILE_JPG  =  "/usr/tmp/junkout.jpg"; 
 
 
 /*---------------------------------------------------------------------*
- *         Top-level procedures for reading images from file           *
+ *          Top-level functions for reading images from file           *
  *---------------------------------------------------------------------*/
-
-#ifndef CYGWIN_ENVIRON  /* this uses unix-type directory lookup */
-
 /*!
  *  pixaReadFiles()
  *
  *      Input:  dirname
- *              root
+ *              substr (<optional> substring filter on filenames; can be NULL)
  *      Return: pixa, or NULL on error
  *
  *  Notes:
  *      (1) 'dirname' is the full path for the directory.
- *      (2) 'root' is the part of the file name (excluding
+ *      (2) 'substr' is the part of the file name (excluding
  *          the directory) that is to be matched.  All matching
- *          filenames are read into the Pixa.
+ *          filenames are read into the Pixa.  If substr is NULL,
+ *          all filenames are read into the Pixa.
+ *      (3) This is unix only; it does not work on Windows.
  */
 PIXA *
 pixaReadFiles(const char  *dirname,
-              const char  *root)
+              const char  *substr)
 {
-char    *str;
-l_int32  i, n1, n2, len;
-PIX     *pix;
 PIXA    *pixa;
-SARRAY  *sa1, *sa2, *sa3;
+SARRAY  *sa;
 
     PROCNAME("pixaReadFiles");
 
     if (!dirname)
         return (PIXA *)ERROR_PTR("dirname not defined", procName, NULL);
-    if (!root)
-        return (PIXA *)ERROR_PTR("rootname not defined", procName, NULL);
-    len = strlen(root);
 
-    if ((sa1 = getFilenamesInDirectory(dirname)) == NULL)
-        return (PIXA *)ERROR_PTR("sa1 not made", procName, NULL);
+    if ((sa = getSortedPathnamesInDirectory(dirname, substr, 0, 0)) == NULL)
+        return (PIXA *)ERROR_PTR("sa not made", procName, NULL);
 
-        /* Save the matching files */
-    sa2 = sarrayCreate(0);
-    n1 = sarrayGetCount(sa1);
-    for (i = 0; i < n1; i++) {
-        str = sarrayGetString(sa1, i, L_NOCOPY);
-        if (!strncmp(str, root, len))
-            sarrayAddString(sa2, str, L_COPY);
-    }
-    sarrayDestroy(&sa1);
-
-    if ((n2 = sarrayGetCount(sa2)) == 0) {
-        L_WARNING("no matching filenames", procName);
-	return pixaCreate(1);   /* empty */
-    }
-    
-        /* Sort the filenames and read the files */
-    sa3 = sarraySort(NULL, sa2, L_SORT_INCREASING);
-    if ((pixa = pixaCreate(n2)) == NULL)
-        return (PIXA *)ERROR_PTR("pixa not made", procName, NULL);
-    for (i = 0; i < n2; i++) {
-        str = sarrayGetString(sa3, i, L_NOCOPY);
-        if ((pix = pixRead(str)) == NULL)
-            L_WARNING("pix not read from file", procName);
-	pixaAddPix(pixa, pix, L_INSERT);
-    }
-
-    sarrayDestroy(&sa2);
-    sarrayDestroy(&sa3);
+    pixa = pixaReadFilesSA(sa);
+    sarrayDestroy(&sa);
     return pixa;
 }
 
-#endif  /* ~CYGWIN_ENVIRON */
+
+/*!
+ *  pixaReadFilesSA()
+ *
+ *      Input:  sarray (full pathnames for all files)
+ *      Return: pixa, or null on error
+ */
+PIXA *
+pixaReadFilesSA(SARRAY  *sa)
+{
+char    *str;
+l_int32  i, n;
+PIX     *pix;
+PIXA    *pixa;
+
+    PROCNAME("pixaReadFilesSA");
+
+    if (!sa)
+        return (PIXA *)ERROR_PTR("sa not defined", procName, NULL);
+
+    n = sarrayGetCount(sa);
+    pixa = pixaCreate(n);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        if ((pix = pixRead(str)) == NULL) {
+            L_WARNING_STRING("pix not read from file %s", procName, str);
+            continue;
+        }
+	pixaAddPix(pixa, pix, L_INSERT);
+    }
+
+    return pixa;
+}
 
 
 /*!
@@ -205,6 +214,7 @@ PIX     *pix;
 
     if (!fp)
         return (PIX *)ERROR_PTR("stream not defined", procName, NULL);
+    pix = NULL;
 
     format = findFileFormat(fp);
 
@@ -227,6 +237,12 @@ PIX     *pix;
         break;
 
     case IFF_TIFF:
+    case IFF_TIFF_PACKBITS:
+    case IFF_TIFF_RLE:
+    case IFF_TIFF_G3:
+    case IFF_TIFF_G4:
+    case IFF_TIFF_LZW:
+    case IFF_TIFF_ZIP:
         if ((pix = pixReadStreamTiff(fp, 0)) == NULL)  /* page 0 by default */
             return (PIX *)ERROR_PTR("tiff: no pix returned", procName, NULL);
         break;
@@ -236,18 +252,26 @@ PIX     *pix;
             return (PIX *)ERROR_PTR("pnm: no pix returned", procName, NULL);
         break;
 
+    case IFF_GIF:
+        if ((pix = pixReadStreamGif(fp)) == NULL)
+            return (PIX *)ERROR_PTR("gif: no pix returned", procName, NULL);
+        break;
+
     case IFF_UNKNOWN:
         return (PIX *)ERROR_PTR( "Unknown format: no pix returned",
                 procName, NULL);
         break;
     }
 
-    pixSetInputFormat(pix, format);
+    if (pix)
+        pixSetInputFormat(pix, format);
     return pix;
 }
 
 
-
+/*---------------------------------------------------------------------*
+ *                            Format finders                           *
+ *---------------------------------------------------------------------*/
 /*!
  *  findFileFormat()
  *
@@ -259,7 +283,8 @@ PIX     *pix;
 l_int32
 findFileFormat(FILE  *fp)
 {
-l_uint8   firstbytes[12]; 
+l_uint8  firstbytes[12]; 
+l_int32  format, ret;
 
     PROCNAME("findFileFormat");
 
@@ -267,13 +292,20 @@ l_uint8   firstbytes[12];
         return ERROR_INT("stream not defined", procName, 0);
 
     rewind(fp);
-    if (nbytesInFile(fp) < 12)
+    if (fnbytesInFile(fp) < 12)
         return ERROR_INT("truncated file", procName, 0);
 
-    fread((char *)&firstbytes, 1, 12, fp);
+    ret = fread((char *)&firstbytes, 1, 12, fp);
+    if (ret != 12)
+        return ERROR_INT("failed to read first 12 bytes of file", procName, 0);
     rewind(fp);
 
-    return findFileFormatBuffer(firstbytes);
+    format = findFileFormatBuffer(firstbytes);
+    if (format == IFF_TIFF) {
+        findTiffCompression(fp, &format);
+        rewind(fp);
+    }
+    return format;
 }
 
 
@@ -283,9 +315,11 @@ l_uint8   firstbytes[12];
  *      Input:  byte buffer (at least 12 bytes in size; we can't check) 
  *      Return: format integer; 0 on error or if format not recognized
  *
- *  Note: this allows you to determine the file format from the first
- *        12 bytes in the compressed data stream, which are stored
- *        in memory.
+ *  Notes:
+ *      (1) This determines the file format from the first 12 bytes in
+ *          the compressed data stream, which are stored in memory.
+ *      (2) For tiff files, this returns IFF_TIFF.  The specific tiff
+ *          compression is then determined using findTiffCompression().
  */
 l_int32
 findFileFormatBuffer(const l_uint8  *buf)
@@ -297,7 +331,7 @@ l_uint16  twobytepw;
     if (!buf)
         return ERROR_INT("byte buffer not defined", procName, 0);
 
-        /* check the bmp and tiff 2-byte header ids */
+        /* Check the bmp and tiff 2-byte header ids */
     ((char *)(&twobytepw))[0] = buf[0];
     ((char *)(&twobytepw))[1] = buf[1];
 
@@ -307,7 +341,7 @@ l_uint16  twobytepw;
     if (twobytepw == TIFF_BIGEND_ID || twobytepw == TIFF_LITTLEEND_ID)
         return IFF_TIFF;
 
-        /* check for the p*m 2-byte header ids */
+        /* Check for the p*m 2-byte header ids */
     if ((buf[0] == 'P' && buf[1] == '4') || /* newer packed */
         (buf[0] == 'P' && buf[1] == '1'))   /* old format */
             return IFF_PNM;
@@ -334,34 +368,163 @@ l_uint16  twobytepw;
     if (buf[0] == 0xff && buf[1] == 0xd8)
         return IFF_JFIF_JPEG;
 
-        /*  check for the 8 byte PNG signature (png_signature in png.c):
-         *        {137, 80, 78, 71, 13, 10, 26, 10}      */
+        /* Check for the 8 byte PNG signature (png_signature in png.c):
+         *       {137, 80, 78, 71, 13, 10, 26, 10}      */
     if (buf[0] == 137 && buf[1] == 80  && buf[2] == 78  && buf[3] == 71  &&
         buf[4] == 13  && buf[5] == 10  && buf[6] == 26  && buf[7] == 10)
             return IFF_PNG;
 
-        /* format header not found */
+        /* Look for "GIF87a" or "GIF89a" */
+    if (buf[0] == 'G' && buf[1] == 'I' && buf[2] == 'F' && buf[3] == '8' &&
+        (buf[4] == '7' || buf[4] == '9') && buf[5] == 'a') {
+        return IFF_GIF;
+    }
+
+        /* Format header not found */
     return IFF_UNKNOWN;
 }
 
 
-/*
+/*!
+ *  fileFormatIsTiff()
+ *
+ *      Input:  fp
+ *      Return: 1 if file is tiff; 0 otherwise or on error
+ */
+l_int32
+fileFormatIsTiff(FILE  *fp)
+{
+l_int32  format;
+
+    PROCNAME("fileFormatIsTiff");
+
+    if (!fp)
+        return ERROR_INT("stream not defined", procName, 0);
+
+    format = findFileFormat(fp);
+    if (format == IFF_TIFF || format == IFF_TIFF_PACKBITS ||
+        format == IFF_TIFF_RLE || format == IFF_TIFF_G3 ||
+        format == IFF_TIFF_G4 || format == IFF_TIFF_LZW ||
+        format == IFF_TIFF_ZIP)
+        return 1;
+    else
+        return 0;
+}
+
+
+/*---------------------------------------------------------------------*
+ *                            Read from memory                         *
+ *---------------------------------------------------------------------*/
+/*!
+ *  pixReadMem()
+ *
+ *      Input:  data (const; encoded)
+ *              datasize (size of data)
+ *      Return: pix, or null on error
+ *
+ *  Notes:
+ *      (1) This is a variation of pixReadStream(), where the data is read
+ *          from a memory buffer rather than a file.
+ *      (2) On windows, this will only read tiff formatted files from
+ *          memory.  For other formats, it requires fmemopen(3).
+ *          Attempts to read those formats will fail at runtime.
+ */
+PIX *
+pixReadMem(const l_uint8  *data,
+           size_t          size)
+{
+l_int32  format;
+PIX     *pix;
+
+    PROCNAME("pixReadMem");
+
+    if (!data)
+        return (PIX *)ERROR_PTR("data not defined", procName, NULL);
+    if (size < 12)
+        return (PIX *)ERROR_PTR("size < 12", procName, NULL);
+    pix = NULL;
+
+    format = findFileFormatBuffer(data);
+
+    switch (format)
+    {
+    case IFF_BMP:
+        if ((pix = pixReadMemBmp(data, size)) == NULL )
+            return (PIX *)ERROR_PTR( "bmp: no pix returned", procName, NULL);
+        break;
+
+    case IFF_JFIF_JPEG:
+        if ((pix = pixReadMemJpeg(data, size, READ_24_BIT_COLOR, 1, NULL, 0))
+                == NULL)
+            return (PIX *)ERROR_PTR( "jpeg: no pix returned", procName, NULL);
+        break;
+
+    case IFF_PNG:
+        if ((pix = pixReadMemPng(data, size)) == NULL)
+            return (PIX *)ERROR_PTR("png: no pix returned", procName, NULL);
+        break;
+
+    case IFF_TIFF:
+    case IFF_TIFF_PACKBITS:
+    case IFF_TIFF_RLE:
+    case IFF_TIFF_G3:
+    case IFF_TIFF_G4:
+    case IFF_TIFF_LZW:
+    case IFF_TIFF_ZIP:
+            /* Reading page 0 by default */
+        if ((pix = pixReadMemTiff(data, size, 0)) == NULL)
+            return (PIX *)ERROR_PTR("tiff: no pix returned", procName, NULL);
+        break;
+
+    case IFF_PNM:
+        if ((pix = pixReadMemPnm(data, size)) == NULL)
+            return (PIX *)ERROR_PTR("pnm: no pix returned", procName, NULL);
+        break;
+
+    case IFF_GIF:
+        if ((pix = pixReadMemGif(data, size)) == NULL)
+            return (PIX *)ERROR_PTR("gif: no pix returned", procName, NULL);
+        break;
+
+    case IFF_UNKNOWN:
+        return (PIX *)ERROR_PTR("Unknown format: no pix returned",
+                procName, NULL);
+        break;
+    }
+
+    if (pix)
+        pixSetInputFormat(pix, format);
+    return pix;
+}
+
+
+/*---------------------------------------------------------------------*
+ *             Test function for I/O with different formats            *
+ *---------------------------------------------------------------------*/
+/*!
  *  ioFormatTest()
  *
  *      Input:  filename (input file)
- *      Return: 0 if OK; 1 on error
+ *      Return: 0 if OK; 1 on error or if the test fails
  *
- *  Note: This writes and reads a set of output files in different formats
- *        to /usr/tmp/, and tests that the result before and after
- *        is unchanged.  It should work properly on input images of
- *        any depth, with and without colormaps.
+ *  Notes:
+ *      (1) This writes and reads a set of output files losslessly
+ *          in different formats to /usr/tmp/, and tests that the
+ *          result before and after is unchanged.
+ *      (2) This should work properly on input images of any depth,
+ *          with and without colormaps.
+ *      (3) All supported formats are tested for bmp, png, tiff and
+ *          non-ascii pnm.  Ascii pnm also works (but who'd ever want
+ *          to use it?)   We allow 2 bpp bmp, although it's not
+ *          supported elsewhere.  And we don't support reading
+ *          16 bpp png, although this can be turned on in pngio.c.
  */
 l_int32
 ioFormatTest(const char  *filename)
 {
-l_int32      d, equal, problems;
-PIX         *pixs, *pixc, *pixt, *pixt2;
-PIXCMAP     *cmap;
+l_int32   d, equal, problems;
+PIX      *pixs, *pixc, *pixt, *pixt2;
+PIXCMAP  *cmap;
 
     PROCNAME("ioFormatTest");
 
@@ -369,25 +532,28 @@ PIXCMAP     *cmap;
         return ERROR_INT("filename not defined", procName, 1);
 
     if ((pixs = pixRead(filename)) == NULL)
-        return ERROR_INT("pix not made", procName, 1);
+        return ERROR_INT("pixs not made", procName, 1);
 
         /* Note that the reader automatically removes colormaps
          * from 1 bpp BMP images, but not from 8 bpp BMP images.
          * Therefore, if our 8 bpp image initially doesn't have a
          * colormap, we are going to need to remove it from any
-         * pix read from a BMP file!  */
+         * pix read from a BMP file. */
     pixc = pixClone(pixs);  /* laziness */
     cmap = pixGetColormap(pixc);  /* colormap; can be NULL */
     d = pixGetDepth(pixc);
 
     problems = FALSE;
-    switch (d)
-    {
-    case 1:
-    case 8:
-    case 32:
-            /* BMP always writes colormaps for 1 and 8 bpp, so we
-             * remove it if the input image doesn't have a colormap */
+
+        /* ----------------------- BMP -------------------------- */
+
+        /* BMP works for 1, 2, 4, 8 and 32 bpp images.
+         * It always writes colormaps for 1 and 8 bpp, so we must
+         * remove it after readback if the input image doesn't have
+         * a colormap.  Although we can write/read 2 bpp BMP, nobody
+         * else can read them! */
+    if (d == 1 || d == 8) {
+        L_INFO("write/read bmp", procName);
         pixWrite(FILE_BMP, pixc, IFF_BMP);
         pixt = pixRead(FILE_BMP);
         if (!cmap)
@@ -396,114 +562,154 @@ PIXCMAP     *cmap;
             pixt2 = pixClone(pixt);
         pixEqual(pixc, pixt2, &equal);
         if (!equal) {
-            fprintf(stderr, "bad bmp image\n");
+            L_INFO("   **** bad bmp image ****", procName);
             problems = TRUE;
         }
         pixDestroy(&pixt);
         pixDestroy(&pixt2);
+    }
 
-        pixWrite(FILE_PNG, pixc, IFF_PNG);
-        pixt = pixRead(FILE_PNG);
-        pixEqual(pixc, pixt, &equal);
-        if (!equal) {
-            fprintf(stderr, "bad png image\n");
-            problems = TRUE;
-        }
-        pixDestroy(&pixt);
-
-            /* Because 8 bpp tiff always writes 256 entry colormaps, the
-             * colormap sizes may be different for 8 bpp images with
-             * colormap; we are testing if the image content is the same */
-        pixWrite(FILE_TIFF, pixc, IFF_TIFF);
-        pixt = pixRead(FILE_TIFF);
-        pixEqual(pixc, pixt, &equal);
-        if (!equal) {
-            fprintf(stderr, "bad tiff uncompressed image\n");
-            problems = TRUE;
-        }
-        pixDestroy(&pixt);
-
-        if (d == 1) {
-            pixWrite(FILE_G4, pixc, IFF_TIFF_G4);
-            pixt = pixRead(FILE_G4);
-            pixEqual(pixc, pixt, &equal);
-            if (!equal) {
-                fprintf(stderr, "bad tiff g4 image\n");
-                problems = TRUE;
-            }
-            pixDestroy(&pixt);
-        }
-
-            /* pnm doesn't have colormaps, so when we write colormapped
-             * pix out as pnm, the colormap is removed.  Thus for the test,
-             * we must remove the colormap from pixc before testing.  */
-        pixWrite(FILE_PNM, pixc, IFF_PNM);
-        pixt = pixRead(FILE_PNM);
-        if (cmap)
-            pixt2 = pixRemoveColormap(pixc, REMOVE_CMAP_BASED_ON_SRC);
-        else
-            pixt2 = pixClone(pixc);
-        pixEqual(pixt, pixt2, &equal);
-        if (!equal) {
-            fprintf(stderr, "bad pnm image\n");
-            problems = TRUE;
-        }
-        pixDestroy(&pixt);
-        pixDestroy(&pixt2);
-        break;
-    case 2:
-    case 4:
-        pixWrite(FILE_PNG, pixc, IFF_PNG);
-        pixt = pixRead(FILE_PNG);
-        pixEqual(pixc, pixt, &equal);
-        if (!equal) {
-            fprintf(stderr, "bad png 2 or 4 bpp image\n");
-            problems = TRUE;
-        }
-        pixDestroy(&pixt);
-
-            /* I believe the 2 and 4 bpp tiff images with colormaps
-             * have colormap sizes 4 and 16, rsp.  This test should
-             * work properly on the content, regardless of the number
-             * of color entries in pixc. */
-        pixWrite(FILE_TIFF, pixc, IFF_TIFF);
-        pixt = pixRead(FILE_TIFF);
-        pixEqual(pixc, pixt, &equal);
-        if (!equal) {
-            fprintf(stderr, "bad tiff uncompressed image\n");
-            problems = TRUE;
-        }
-        pixDestroy(&pixt);
-
-            /* Note: this works in this test, but nobody else can
-             * read 2 bpp colormapped bmp files that are generated here! */
+    if (d == 2 || d == 4 || d == 32) {
+        L_INFO("write/read bmp", procName);
         pixWrite(FILE_BMP, pixc, IFF_BMP);
         pixt = pixRead(FILE_BMP);
         pixEqual(pixc, pixt, &equal);
         if (!equal) {
-            fprintf(stderr, "bad bmp 2 or 4 bpp image\n");
+            L_INFO("   **** bad bmp image ****", procName);
             problems = TRUE;
         }
         pixDestroy(&pixt);
-        break;
-    case 16:
+    }
+
+        /* ----------------------- PNG -------------------------- */
+
+        /* PNG works for all depths, but here, because we strip
+         * 16 --> 8 bpp on reading, we don't test png for 16 bpp. */
+    if (d != 16) {
+        L_INFO("write/read png", procName);
         pixWrite(FILE_PNG, pixc, IFF_PNG);
         pixt = pixRead(FILE_PNG);
         pixEqual(pixc, pixt, &equal);
         if (!equal) {
-            fprintf(stderr, "bad png 16 bpp image\n");
+            L_INFO("   **** bad png image ****", procName);
             problems = TRUE;
         }
         pixDestroy(&pixt);
-    default:
-        return ERROR_INT("d not in {1,2,4,8,16,32}", procName, 1);
     }
 
+        /* ----------------------- TIFF -------------------------- */
+
+        /* TIFF works for 1, 2, 4, 8, 16 and 32 bpp images.
+         * Because 8 bpp tiff always writes 256 entry colormaps, the
+         * colormap sizes may be different for 8 bpp images with
+         * colormap; we are testing if the image content is the same.
+         * Likewise, the 2 and 4 bpp tiff images with colormaps
+         * have colormap sizes 4 and 16, rsp.  This test should
+         * work properly on the content, regardless of the number
+         * of color entries in pixc. */
+
+        /* tiff uncompressed works for all pixel depths */
+    L_INFO("write/read uncompressed tiff", procName);
+    pixWrite(FILE_TIFF, pixc, IFF_TIFF);
+    pixt = pixRead(FILE_TIFF);
+    pixEqual(pixc, pixt, &equal);
+    if (!equal) {
+        L_INFO("   **** bad tiff uncompressed image ****", procName);
+        problems = TRUE;
+    }
+    pixDestroy(&pixt);
+
+        /* tiff lzw works for all pixel depths */
+    L_INFO("write/read lzw compressed tiff", procName);
+    pixWrite(FILE_LZW, pixc, IFF_TIFF_LZW);
+    pixt = pixRead(FILE_LZW);
+    pixEqual(pixc, pixt, &equal);
+    if (!equal) {
+        L_INFO("   **** bad tiff lzw compressed image ****", procName);
+        problems = TRUE;
+    }
+    pixDestroy(&pixt);
+
+        /* tiff adobe deflate (zip) works for all pixel depths */
+    L_INFO("write/read zip compressed tiff", procName);
+    pixWrite(FILE_ZIP, pixc, IFF_TIFF_ZIP);
+    pixt = pixRead(FILE_ZIP);
+    pixEqual(pixc, pixt, &equal);
+    if (!equal) {
+        L_INFO("   **** bad tiff zip compressed image ****", procName);
+        problems = TRUE;
+    }
+    pixDestroy(&pixt);
+
+        /* tiff g4, g3, rle and packbits work for 1 bpp */
+    if (d == 1) {
+        L_INFO("write/read g4 compressed tiff", procName);
+        pixWrite(FILE_G4, pixc, IFF_TIFF_G4);
+        pixt = pixRead(FILE_G4);
+        pixEqual(pixc, pixt, &equal);
+        if (!equal) {
+            L_INFO("   **** bad tiff g4 image ****", procName);
+            problems = TRUE;
+        }
+        pixDestroy(&pixt);
+
+        L_INFO("write/read g3 compressed tiff", procName);
+        pixWrite(FILE_G3, pixc, IFF_TIFF_G3);
+        pixt = pixRead(FILE_G3);
+        pixEqual(pixc, pixt, &equal);
+        if (!equal) {
+            L_INFO("   **** bad tiff g3 image ****", procName);
+            problems = TRUE;
+        }
+        pixDestroy(&pixt);
+
+        L_INFO("write/read rle compressed tiff", procName);
+        pixWrite(FILE_RLE, pixc, IFF_TIFF_RLE);
+        pixt = pixRead(FILE_RLE);
+        pixEqual(pixc, pixt, &equal);
+        if (!equal) {
+            L_INFO("   **** bad tiff rle image ****", procName);
+            problems = TRUE;
+        }
+        pixDestroy(&pixt);
+
+        L_INFO("write/read packbits compressed tiff", procName);
+        pixWrite(FILE_PB, pixc, IFF_TIFF_PACKBITS);
+        pixt = pixRead(FILE_PB);
+        pixEqual(pixc, pixt, &equal);
+        if (!equal) {
+            L_INFO("   **** bad tiff packbits image ****", procName);
+            problems = TRUE;
+        }
+        pixDestroy(&pixt);
+    }
+
+        /* ----------------------- PNM -------------------------- */
+
+        /* pnm works for 1, 2, 4, 8, 16 and 32 bpp.
+         * pnm doesn't have colormaps, so when we write colormapped
+         * pix out as pnm, the colormap is removed.  Thus for the test,
+         * we must remove the colormap from pixc before testing.  */
+    L_INFO("write/read pnm", procName);
+    pixWrite(FILE_PNM, pixc, IFF_PNM);
+    pixt = pixRead(FILE_PNM);
+    if (cmap)
+        pixt2 = pixRemoveColormap(pixc, REMOVE_CMAP_BASED_ON_SRC);
+    else
+        pixt2 = pixClone(pixc);
+    pixEqual(pixt, pixt2, &equal);
+    if (!equal) {
+        L_INFO("   **** bad pnm image ****", procName);
+        problems = TRUE;
+    }
+    pixDestroy(&pixt);
+    pixDestroy(&pixt2);
+
     if (problems == FALSE)
-        L_INFO("all formats read and written OK", procName);
+        L_INFO("All formats read and written OK!", procName);
 
     pixDestroy(&pixc);
     pixDestroy(&pixs);
-    return 0;
+    return problems;
 }
 
