@@ -44,6 +44,7 @@
  *
  *      Concatenate 2 sarrays
  *          l_int32    sarrayConcatenate()
+ *          l_int32    sarrayAppendRange()
  *
  *      Convert word sarray to (formatted) line sarray
  *          SARRAY    *sarrayConvertWordsToLines()
@@ -51,6 +52,10 @@
  *      Split string on separator list
  *          SARRAY    *sarraySplitString()
  *
+ *      Filter sarray
+ *          SARRAY    *sarraySelectBySubstring()
+ *          l_int32    sarrayParseRange()
+ *
  *      Sort
  *          SARRAY    *sarraySort()
  *          l_int32    stringCompareLexical()
@@ -465,12 +470,13 @@ l_int32  i;
     PROCNAME("sarrayClear");
 
     if (!sa)
-        return ERROR_INT("sa not defined", procName, 0);
+        return ERROR_INT("sa not defined", procName, 1);
     for (i = 0; i < sa->n; i++) {  /* free strings and null ptrs */
         FREE(sa->array[i]);
         sa->array[i] = NULL;
     }
     sa->n = 0;
+    return 0;
 }
 
         
@@ -656,6 +662,8 @@ sarrayToString(SARRAY  *sa,
  *          all white space.
  *      (2) If addnlflag != 0, adds either a '\n' or a ' ' after
  *          each substring.
+ *      (3) If the sarray is empty, this returns a string with just
+ *          the character corresponding to @addnlflag.
  */
 char *
 sarrayToStringRange(SARRAY  *sa,
@@ -670,8 +678,25 @@ l_int32  n, i, last, size, index, len;
 
     if (!sa)
         return (char *)ERROR_PTR("sa not defined", procName, NULL);
+    if (addnlflag != 0 && addnlflag != 1 && addnlflag != 2)
+        return (char *)ERROR_PTR("invalid addnlflag", procName, NULL);
 
     n = sarrayGetCount(sa);
+
+        /* Empty sa; return char corresponding to addnlflag only */
+    if (n == 0) {
+        if (first == 0) {
+            if (addnlflag == 0)
+                return stringNew("");
+            if (addnlflag == 1)
+                return stringNew("\n");
+	    else  /* addnlflag == 2) */
+                return stringNew(" ");
+        }
+        else
+            return (char *)ERROR_PTR("first not valid", procName, NULL);
+    }
+
     if (first < 0 || first >= n)
         return (char *)ERROR_PTR("first not valid", procName, NULL);
     if (nstrings == 0 || (nstrings > n - first))
@@ -711,8 +736,8 @@ l_int32  n, i, last, size, index, len;
 /*!
  *  sarrayConcatenate()
  *
- *      Input:  sarray1  (to be added to)
- *              sarray2  (append to sarray1)
+ *      Input:  sa1  (to be added to)
+ *              sa2  (append to sa1)
  *      Return: 0 if OK, 1 on error
  *
  *  Notes:
@@ -722,7 +747,7 @@ l_int32
 sarrayConcatenate(SARRAY  *sa1,
                   SARRAY  *sa2)
 {
-char    *string;
+char    *str;
 l_int32  n, i;
 
     PROCNAME("sarrayConcatenate");
@@ -734,8 +759,53 @@ l_int32  n, i;
 
     n = sarrayGetCount(sa2);
     for (i = 0; i < n; i++) {
-        string = sarrayGetString(sa2, i, L_NOCOPY);
-        sarrayAddString(sa1, string, L_COPY);
+        str = sarrayGetString(sa2, i, L_NOCOPY);
+        sarrayAddString(sa1, str, L_COPY);
+    }
+
+    return 0;
+}
+
+
+/*!
+ *  sarrayAppendRange()
+ *
+ *      Input:  sa1  (to be added to)
+ *              sa2  (append specified range of strings in sa2 to sa1)
+ *              start (index of first string of sa2 to append)
+ *              end (index of last string of sa2 to append)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) Copies of the strings in sarray2 are added to sarray1.
+ *      (2) The [start ... end] range is truncated if necessary.
+ */
+l_int32
+sarrayAppendRange(SARRAY  *sa1,
+                  SARRAY  *sa2,
+		  l_int32  start,
+		  l_int32  end)
+{
+char    *str;
+l_int32  n, i;
+
+    PROCNAME("sarrayAppendRange");
+
+    if (!sa1)
+        return ERROR_INT("sa1 not defined", procName, 1);
+    if (!sa2)
+        return ERROR_INT("sa2 not defined", procName, 1);
+    if (start < 0)
+        start = 0;
+    n = sarrayGetCount(sa2);
+    if (end >= n)
+        end = n - 1;
+    if (start > end)
+        return ERROR_INT("start > end", procName, 1);
+
+    for (i = start; i <= end; i++) {
+        str = sarrayGetString(sa2, i, L_NOCOPY);
+        sarrayAddString(sa1, str, L_COPY);
     }
 
     return 0;
@@ -878,6 +948,162 @@ char  *cstr, *substr, *saveptr;
 }
 
 
+/*----------------------------------------------------------------------*
+ *                              Filter sarray                           *
+ *----------------------------------------------------------------------*/
+/*!
+ *  sarraySelectBySubstring()
+ *
+ *      Input:  sain (input sarray)
+ *              substr (<optional> substring for matching; can be NULL)
+ *      Return: saout (output sarray, filtered with substring) or null on error
+ *
+ *  Notes:
+ *      (1) This selects all strings in sain that have substr as a substring.
+ *          Note that we can't use strncmp() because we're looking for
+ *          a match to the substring anywhere within each filename.
+ *      (2) If substr == NULL, returns a copy of the sarray.
+ */
+SARRAY *
+sarraySelectBySubstring(SARRAY      *sain,
+                        const char  *substr)
+{
+char    *str;
+l_int32  n, i, offset, found;
+SARRAY  *saout;
+
+    PROCNAME("sarraySelectBySubstring");
+
+    if (!sain)
+        return (SARRAY *)ERROR_PTR("sain not defined", procName, NULL);
+
+    n = sarrayGetCount(sain);
+    if (!substr || n == 0)
+        return sarrayCopy(sain);
+
+    saout = sarrayCreate(n);
+    for (i = 0; i < n; i++) {
+        str = sarrayGetString(sain, i, L_NOCOPY);
+        arrayFindSequence((l_uint8 *)str, strlen(str), (l_uint8 *)substr,
+                          strlen(substr), &offset, &found);
+        if (found)
+            sarrayAddString(saout, str, L_COPY);
+    }
+
+    return saout;
+}
+
+
+/*!
+ *  sarrayParseRange()
+ *
+ *      Input:  sa (input sarray)
+ *              start (index to start range search)
+ *             &actualstart (<return> index of actual start; may be > 'start')
+ *             &end (<return> index of end)
+ *             &newstart (<return> index of start of next range)
+ *              substr (substring for matching at beginning of string)
+ *              loc (byte offset within the string for the pattern; use
+ *                   -1 if the location does not matter);
+ *      Return: 0 if valid range found; 1 otherwise
+ *
+ *  Notes:
+ *      (1) This finds the range of the next set of strings in SA,
+ *          beginning the search at 'start', that does NOT have
+ *          the substring 'substr' either at the indicated location
+ *          in the string or anywhere in the string.  The input
+ *          variable 'loc' is the specified offset within the string;
+ *          use -1 to indicate 'anywhere in the string'.
+ *      (2) Always check the return value to verify that a valid range
+ *          was found.
+ *      (3) If a valid range is not found, the values of actstart,
+ *          end and newstart are all set to the size of sa.
+ *      (4) If this is the last valid range, newstart returns the value n.
+ *          In use, this should be tested before calling the function.
+ *      (5) Usage example.  To find all the valid ranges in a file
+ *          where the invalid lines begin with two dashes, copy each
+ *          line in the file to a string in an sarray, and do:
+ *             start = 0;
+ *             while (!sarrayParseRange(sa, start, &actstart, &end, &start,
+ *                    "--", 0))
+ *                 fprintf(stderr, "start = %d, end = %d\n", actstart, end);
+ */
+l_int32
+sarrayParseRange(SARRAY      *sa,
+                 l_int32      start,
+                 l_int32     *pactualstart,
+                 l_int32     *pend,
+                 l_int32     *pnewstart,
+                 const char  *substr,
+		 l_int32      loc)
+{
+char    *str;
+l_int32  n, i, offset, found;
+
+    PROCNAME("sarrayParseRange");
+
+    if (!sa)
+        return ERROR_INT("sa not defined", procName, 1);
+    if (!pactualstart || !pend || !pnewstart)
+        return ERROR_INT("not all range addresses defined", procName, 1);
+    n = sarrayGetCount(sa);
+    *pactualstart = *pend = *pnewstart = n;
+    if (!substr)
+        return ERROR_INT("substr not defined", procName, 1);
+
+        /* Look for the first string without the marker */
+    if (start < 0 || start >= n)
+        return 1;
+    for (i = start; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        arrayFindSequence((l_uint8 *)str, strlen(str), (l_uint8 *)substr,
+                          strlen(substr), &offset, &found);
+	if (loc < 0) {
+            if (!found) break;
+	} else {
+            if (!found || offset != loc) break;
+	}
+    }
+    start = i;
+    if (i == n)  /* couldn't get started */
+        return 1;
+
+        /* Look for the last string without the marker */
+    *pactualstart = start;
+    for (i = start + 1; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        arrayFindSequence((l_uint8 *)str, strlen(str), (l_uint8 *)substr,
+                          strlen(substr), &offset, &found);
+	if (loc < 0) {
+            if (found) break;
+	} else {
+            if (found && offset == loc) break;
+	}
+    }
+    *pend = i - 1;
+    start = i;
+    if (i == n)  /* no further range */
+        return 0;
+
+        /* Look for the first string after *pend without the marker.
+         * This will start the next run of strings, if it exists. */
+    for (i = start; i < n; i++) {
+        str = sarrayGetString(sa, i, L_NOCOPY);
+        arrayFindSequence((l_uint8 *)str, strlen(str), (l_uint8 *)substr,
+                          strlen(substr), &offset, &found);
+	if (loc < 0) {
+            if (!found) break;
+	} else {
+            if (!found || offset != loc) break;
+	}
+    }
+    if (i < n)
+        *pnewstart = i;
+
+    return 0;
+}
+
+
 /*----------------------------------------------------------------------*
  *                                   Sort                               *
  *----------------------------------------------------------------------*/
