@@ -84,11 +84,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include "allheaders.h"
 
 #ifndef  NO_CONSOLE_IO
 #define  DEBUG_OVERFLOW   0
+#define  DEBUG_UNROLLING  0
 #endif  /* ~NO_CONSOLE_IO */
 
 
@@ -105,8 +105,6 @@
  *  and weighting each pixel value by this fractional area.
  *
  *  P3 speed is about 7 x 10^6 dst pixels/sec/GHz
- *
- *  *** Warning: explicit assumption about RGB component ordering ***
  */
 void
 scaleColorLILow(l_uint32  *datad,
@@ -165,18 +163,18 @@ l_float32  scx, scy;
             area10 = xf * (16 - yf);
             area01 = (16 - xf) * yf;
             area11 = xf * yf;
-            v00r = area00 * (pixels1 >> 24);
-            v00g = area00 * ((pixels1 >> 16) & 0xff);
-            v00b = area00 * ((pixels1 >> 8) & 0xff);
-            v10r = area10 * (pixels2 >> 24);
-            v10g = area10 * ((pixels2 >> 16) & 0xff);
-            v10b = area10 * ((pixels2 >> 8) & 0xff);
-            v01r = area01 * (pixels3 >> 24);
-            v01g = area01 * ((pixels3 >> 16) & 0xff);
-            v01b = area01 * ((pixels3 >> 8) & 0xff);
-            v11r = area11 * (pixels4 >> 24);
-            v11g = area11 * ((pixels4 >> 16) & 0xff);
-            v11b = area11 * ((pixels4 >> 8) & 0xff);
+            v00r = area00 * ((pixels1 >> L_RED_SHIFT) & 0xff);
+            v00g = area00 * ((pixels1 >> L_GREEN_SHIFT) & 0xff);
+            v00b = area00 * ((pixels1 >> L_BLUE_SHIFT) & 0xff);
+            v10r = area10 * ((pixels2 >> L_RED_SHIFT) & 0xff);
+            v10g = area10 * ((pixels2 >> L_GREEN_SHIFT) & 0xff);
+            v10b = area10 * ((pixels2 >> L_BLUE_SHIFT) & 0xff);
+            v01r = area01 * ((pixels3 >> L_RED_SHIFT) & 0xff);
+            v01g = area01 * ((pixels3 >> L_GREEN_SHIFT) & 0xff);
+            v01b = area01 * ((pixels3 >> L_BLUE_SHIFT) & 0xff);
+            v11r = area11 * ((pixels4 >> L_RED_SHIFT) & 0xff);
+            v11g = area11 * ((pixels4 >> L_GREEN_SHIFT) & 0xff);
+            v11b = area11 * ((pixels4 >> L_BLUE_SHIFT) & 0xff);
             pixel = (((v00r + v10r + v01r + v11r + 128) << 16) & 0xff000000) |
                     (((v00g + v10g + v01g + v11g + 128) << 8) & 0x00ff0000) |
                     ((v00b + v10b + v01b + v11b + 128) & 0x0000ff00);
@@ -356,13 +354,13 @@ l_uint32  *lines, *lined;
  *              lastlineflag  (1 if last src line; 0 otherwise)
  *      Return: void
  *
- *  *** Warning: explicit assumption about RGB component ordering ***
+ *  *** Warning: implicit assumption about RGB component ordering ***
  */
 void
 scaleColor2xLILineLow(l_uint32  *lined,
                       l_int32    wpld,
-                       l_uint32  *lines,
-                       l_int32    ws,
+                      l_uint32  *lines,
+                      l_int32    ws,
                       l_int32    wpls,
                       l_int32    lastlineflag)
 {
@@ -408,17 +406,17 @@ l_uint32  *linesp, *linedp;
                 /* save dest values */
             pixel = (rval1 << 24 | gval1 << 16 | bval1 << 8);
             *(lined + jd) = pixel;                               /* pix 1 */
-            pixel = (((rval1 + rval2) << 23) & 0xff000000 |
-                     ((gval1 + gval2) << 15) & 0x00ff0000 |
-                     ((bval1 + bval2) << 7) & 0x0000ff00);
+            pixel = ((((rval1 + rval2) << 23) & 0xff000000) |
+                     (((gval1 + gval2) << 15) & 0x00ff0000) |
+                     (((bval1 + bval2) << 7) & 0x0000ff00));
             *(lined + jd + 1) = pixel;                           /* pix 2 */
-            pixel = (((rval1 + rval3) << 23) & 0xff000000 |
-                     ((gval1 + gval3) << 15) & 0x00ff0000 |
-                     ((bval1 + bval3) << 7) & 0x0000ff00);
+            pixel = ((((rval1 + rval3) << 23) & 0xff000000) |
+                     (((gval1 + gval3) << 15) & 0x00ff0000) |
+                     (((bval1 + bval3) << 7) & 0x0000ff00));
             *(linedp + jd) = pixel;                              /* pix 3 */
-            pixel = (((rval1 + rval2 + rval3 + rval4) << 22) & 0xff000000 | 
-                     ((gval1 + gval2 + gval3 + gval4) << 14) & 0x00ff0000 |
-                     ((bval1 + bval2 + bval3 + bval4) << 6) & 0x0000ff00);
+            pixel = ((((rval1 + rval2 + rval3 + rval4) << 22) & 0xff000000) | 
+                     (((gval1 + gval2 + gval3 + gval4) << 14) & 0x00ff0000) |
+                     (((bval1 + bval2 + bval3 + bval4) << 6) & 0x0000ff00));
             *(linedp + jd + 1) = pixel;                          /* pix 4 */
         }  
             /* last src pixel on line */
@@ -431,9 +429,9 @@ l_uint32  *linesp, *linedp;
         pixel = (rval1 << 24 | gval1 << 16 | bval1 << 8);
         *(lined + 2 * wsm) = pixel;                        /* pix 1 */
         *(lined + 2 * wsm + 1) = pixel;                    /* pix 2 */
-        pixel = (((rval1 + rval3) << 23) & 0xff000000 |
-                 ((gval1 + gval3) << 15) & 0x00ff0000 |
-                 ((bval1 + bval3) << 7) & 0x0000ff00);
+        pixel = ((((rval1 + rval3) << 23) & 0xff000000) |
+                 (((gval1 + gval3) << 15) & 0x00ff0000) |
+                 (((bval1 + bval3) << 7) & 0x0000ff00));
         *(linedp + 2 * wsm) = pixel;                       /* pix 3 */
         *(linedp + 2 * wsm + 1) = pixel;                   /* pix 4 */
     }
@@ -454,9 +452,9 @@ l_uint32  *linesp, *linedp;
             pixel = (rval1 << 24 | gval1 << 16 | bval1 << 8);
             *(lined + jd) = pixel;                            /* pix 1 */
             *(linedp + jd) = pixel;                           /* pix 2 */
-            pixel = (((rval1 + rval2) << 23) & 0xff000000 |
-                     ((gval1 + gval2) << 15) & 0x00ff0000 |
-                     ((bval1 + bval2) << 7) & 0x0000ff00);
+            pixel = ((((rval1 + rval2) << 23) & 0xff000000) |
+                     (((gval1 + gval2) << 15) & 0x00ff0000) |
+                     (((bval1 + bval2) << 7) & 0x0000ff00));
             *(lined + jd + 1) = pixel;                        /* pix 3 */
             *(linedp + jd + 1) = pixel;                       /* pix 4 */
         }  
@@ -570,18 +568,71 @@ scaleGray2xLILineLow(l_uint32  *lined,
                      l_int32    wpls,
                      l_int32    lastlineflag)
 {
-l_int32    j, jd, wsm;
+l_int32    j, jd, wsm, w;
 l_int32    sval1, sval2, sval3, sval4;
 l_uint32  *linesp, *linedp;
+l_uint32   words, wordsp, wordd, worddp;
 
     wsm = ws - 1;
 
     if (lastlineflag == 0) {
         linesp = lines + wpls;
         linedp = lined + wpld;
-        sval2 = GET_DATA_BYTE(lines, 0);
-        sval4 = GET_DATA_BYTE(linesp, 0);
-        for (j = 0, jd = 0; j < wsm; j++, jd += 2) {
+
+            /* Unroll the loop 4x and work on full words */
+        words = lines[0];
+        wordsp = linesp[0];
+        sval2 = (words >> 24) & 0xff;
+        sval4 = (wordsp >> 24) & 0xff;
+        for (j = 0, jd = 0, w = 0; j + 3 < wsm; j += 4, jd += 8, w++) {
+                /* At the top of the loop,
+                 * words == lines[w], wordsp == linesp[w]
+                 * and the top bytes of those have been loaded into
+                 * sval2 and sval4. */
+            sval1 = sval2;
+            sval2 = (words >> 16) & 0xff;
+            sval3 = sval4;
+            sval4 = (wordsp >> 16) & 0xff;
+            wordd = (sval1 << 24) | (((sval1 + sval2) >> 1) << 16);
+            worddp = (((sval1 + sval3) >> 1) << 24) |
+                (((sval1 + sval2 + sval3 + sval4) >> 2) << 16);
+
+            sval1 = sval2;
+            sval2 = (words >> 8) & 0xff;
+            sval3 = sval4;
+            sval4 = (wordsp >> 8) & 0xff;
+            wordd |= (sval1 << 8) | ((sval1 + sval2) >> 1);
+            worddp |= (((sval1 + sval3) >> 1) << 8) |
+                ((sval1 + sval2 + sval3 + sval4) >> 2);
+            lined[w * 2] = wordd;
+            linedp[w * 2] = worddp;
+
+            sval1 = sval2;
+            sval2 = words & 0xff;
+            sval3 = sval4;
+            sval4 = wordsp & 0xff;
+            wordd = (sval1 << 24) |                              /* pix 1 */
+                (((sval1 + sval2) >> 1) << 16);                  /* pix 2 */
+            worddp = (((sval1 + sval3) >> 1) << 24) |            /* pix 3 */
+                (((sval1 + sval2 + sval3 + sval4) >> 2) << 16);  /* pix 4 */
+
+                /* Load the next word as we need its first byte */
+            words = lines[w + 1];
+            wordsp = linesp[w + 1];
+            sval1 = sval2;
+            sval2 = (words >> 24) & 0xff;
+            sval3 = sval4;
+            sval4 = (wordsp >> 24) & 0xff;
+            wordd |= (sval1 << 8) |                              /* pix 1 */
+                ((sval1 + sval2) >> 1);                          /* pix 2 */
+            worddp |= (((sval1 + sval3) >> 1) << 8) |            /* pix 3 */
+                ((sval1 + sval2 + sval3 + sval4) >> 2);          /* pix 4 */
+            lined[w * 2 + 1] = wordd;
+            linedp[w * 2 + 1] = worddp;
+        }
+
+            /* Finish up the last word */
+        for (; j < wsm; j++, jd += 2) {
             sval1 = sval2;
             sval3 = sval4;
             sval2 = GET_DATA_BYTE(lines, j + 1);
@@ -591,13 +642,40 @@ l_uint32  *linesp, *linedp;
             SET_DATA_BYTE(linedp, jd, (sval1 + sval3) / 2);      /* pix 3 */
             SET_DATA_BYTE(linedp, jd + 1,
                           (sval1 + sval2 + sval3 + sval4) / 4);  /* pix 4 */
-        }  
+        }
         sval1 = sval2;
         sval3 = sval4;
         SET_DATA_BYTE(lined, 2 * wsm, sval1);                     /* pix 1 */
         SET_DATA_BYTE(lined, 2 * wsm + 1, sval1);                 /* pix 2 */
         SET_DATA_BYTE(linedp, 2 * wsm, (sval1 + sval3) / 2);      /* pix 3 */
         SET_DATA_BYTE(linedp, 2 * wsm + 1, (sval1 + sval3) / 2);  /* pix 4 */
+
+#if DEBUG_UNROLLING
+#define CHECK_BYTE(a, b, c) if (GET_DATA_BYTE(a, b) != c) {\
+     fprintf(stderr, "Error: mismatch at %d, %d vs %d\n", \
+             j, GET_DATA_BYTE(a, b), c); }
+
+        sval2 = GET_DATA_BYTE(lines, 0);
+        sval4 = GET_DATA_BYTE(linesp, 0);
+        for (j = 0, jd = 0; j < wsm; j++, jd += 2) {
+            sval1 = sval2;
+            sval3 = sval4;
+            sval2 = GET_DATA_BYTE(lines, j + 1);
+            sval4 = GET_DATA_BYTE(linesp, j + 1);
+            CHECK_BYTE(lined, jd, sval1);                     /* pix 1 */
+            CHECK_BYTE(lined, jd + 1, (sval1 + sval2) / 2);   /* pix 2 */
+            CHECK_BYTE(linedp, jd, (sval1 + sval3) / 2);      /* pix 3 */
+            CHECK_BYTE(linedp, jd + 1,
+                          (sval1 + sval2 + sval3 + sval4) / 4);  /* pix 4 */
+        }
+        sval1 = sval2;
+        sval3 = sval4;
+        CHECK_BYTE(lined, 2 * wsm, sval1);                     /* pix 1 */
+        CHECK_BYTE(lined, 2 * wsm + 1, sval1);                 /* pix 2 */
+        CHECK_BYTE(linedp, 2 * wsm, (sval1 + sval3) / 2);      /* pix 3 */
+        CHECK_BYTE(linedp, 2 * wsm + 1, (sval1 + sval3) / 2);  /* pix 4 */
+#undef CHECK_BYTE
+#endif
     }
     else {   /* last row of src pixels: lastlineflag == 1 */
         linedp = lined + wpld;
@@ -996,8 +1074,6 @@ l_float32  wratio, hratio;
  *      (2) size is the full width of the lowpass smoothing filter.
  *          It is correlated with the reduction ratio, being the
  *          nearest integer such that size is approximately equal to hs / hd.
- *
- *  *** Warning: explicit assumption about RGB component ordering ***
  */
 l_int32
 scaleSmoothLow(l_uint32  *datad,
@@ -1071,22 +1147,23 @@ l_float32  wratio, hratio, norm;
                     ppixel = lines + m * wpls + xstart;
                     for (n = 0; n < size; n++) {
                         pixel = *(ppixel + n);
-                        rval += (pixel >> 24);
-                        gval += (pixel >> 16 & 0xff);
-                        bval += (pixel >> 8 & 0xff);
+                        rval += (pixel >> L_RED_SHIFT) & 0xff;
+                        gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+                        bval += (pixel >> L_BLUE_SHIFT) & 0xff;
                     }
                 }
                 rval = (l_int32)((l_float32)rval * norm);
                 gval = (l_int32)((l_float32)gval * norm);
                 bval = (l_int32)((l_float32)bval * norm);
-                *(lined + j) = rval << 24 | gval << 16 | bval << 8;
+                *(lined + j) = rval << L_RED_SHIFT |
+                               gval << L_GREEN_SHIFT |
+                               bval << L_BLUE_SHIFT;
             }
         }
     }
 
     FREE(srow);
     FREE(scol);
-
     return 0;
 }
 
@@ -1096,8 +1173,6 @@ l_float32  wratio, hratio, norm;
  *
  *  Note: This function is called with 32 bpp RGB src and 8 bpp,
  *        half-resolution dest.  The weights should add to 1.0.
- *
- *  *** Warning: explicit assumption about RGB component ordering ***
  */
 void
 scaleRGBToGray2Low(l_uint32  *datad,
@@ -1123,21 +1198,21 @@ l_uint32   pixel;
         for (j = 0; j < wd; j++) {
                 /* Sum each of the color components from 4 src pixels */
             pixel = *(lines + 2 * j);
-            rval = (pixel >> 24);
-            gval = (pixel >> 16 & 0xff);
-            bval = (pixel >> 8 & 0xff);
+            rval = (pixel >> L_RED_SHIFT) & 0xff;
+            gval = (pixel >> L_GREEN_SHIFT) & 0xff;
+            bval = (pixel >> L_BLUE_SHIFT) & 0xff;
             pixel = *(lines + 2 * j + 1);
-            rval += (pixel >> 24);
-            gval += (pixel >> 16 & 0xff);
-            bval += (pixel >> 8 & 0xff);
+            rval += (pixel >> L_RED_SHIFT) & 0xff;
+            gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+            bval += (pixel >> L_BLUE_SHIFT) & 0xff;
             pixel = *(lines + wpls + 2 * j);
-            rval += (pixel >> 24);
-            gval += (pixel >> 16 & 0xff);
-            bval += (pixel >> 8 & 0xff);
+            rval += (pixel >> L_RED_SHIFT) & 0xff;
+            gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+            bval += (pixel >> L_BLUE_SHIFT) & 0xff;
             pixel = *(lines + wpls + 2 * j + 1);
-            rval += (pixel >> 24);
-            gval += (pixel >> 16 & 0xff);
-            bval += (pixel >> 8 & 0xff);
+            rval += (pixel >> L_RED_SHIFT) & 0xff;
+            gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+            bval += (pixel >> L_BLUE_SHIFT) & 0xff;
                 /* Generate the dest byte as a weighted sum of the averages */
             val = (l_int32)(rwt * rval + gwt * gval + bwt * bval);
             SET_DATA_BYTE(lined, j, val);
@@ -1160,8 +1235,6 @@ l_uint32   pixel;
  *  and are weighted by the number of sub-pixels covered by
  *  the dest pixel.  This is about 2x slower than scaleSmoothLow(),
  *  but the results are significantly better on small text.
- *
- *  *** Warning: implicit assumption about RGB component ordering ***
  */
 void
 scaleColorAreaMapLow(l_uint32  *datad,
@@ -1186,7 +1259,7 @@ l_int32    v10r, v10g, v10b;  /* contrib from UR src pixel */
 l_int32    v11r, v11g, v11b;  /* contrib from LR src pixel */
 l_int32    vinr, ving, vinb;  /* contrib from all full interior src pixels */
 l_int32    vmidr, vmidg, vmidb;  /* contrib from side parts */
-l_int32    valr, valg, valb;
+l_int32    rval, gval, bval;
 l_uint32   pixel00, pixel10, pixel01, pixel11, pixel;
 l_uint32  *lines, *lined;
 l_float32  scx, scy;
@@ -1242,25 +1315,25 @@ l_float32  scx, scy;
             area10 = xlf * (16 - yuf);
             area01 = (16 - xuf) * ylf;
             area11 = xlf * ylf;
-            v00r = area00 * (pixel00 >> 24);
-            v00g = area00 * ((pixel00 >> 16) & 0xff);
-            v00b = area00 * ((pixel00 >> 8) & 0xff);
-            v10r = area10 * (pixel10 >> 24);
-            v10g = area10 * ((pixel10 >> 16) & 0xff);
-            v10b = area10 * ((pixel10 >> 8) & 0xff);
-            v01r = area01 * (pixel01 >> 24);
-            v01g = area01 * ((pixel01 >> 16) & 0xff);
-            v01b = area01 * ((pixel01 >> 8) & 0xff);
-            v11r = area11 * (pixel11 >> 24);
-            v11g = area11 * ((pixel11 >> 16) & 0xff);
-            v11b = area11 * ((pixel11 >> 8) & 0xff);
+            v00r = area00 * ((pixel00 >> L_RED_SHIFT) & 0xff);
+            v00g = area00 * ((pixel00 >> L_GREEN_SHIFT) & 0xff);
+            v00b = area00 * ((pixel00 >> L_BLUE_SHIFT) & 0xff);
+            v10r = area10 * ((pixel10 >> L_RED_SHIFT) & 0xff);
+            v10g = area10 * ((pixel10 >> L_GREEN_SHIFT) & 0xff);
+            v10b = area10 * ((pixel10 >> L_BLUE_SHIFT) & 0xff);
+            v01r = area01 * ((pixel01 >> L_RED_SHIFT) & 0xff);
+            v01g = area01 * ((pixel01 >> L_GREEN_SHIFT) & 0xff);
+            v01b = area01 * ((pixel01 >> L_BLUE_SHIFT) & 0xff);
+            v11r = area11 * ((pixel11 >> L_RED_SHIFT) & 0xff);
+            v11g = area11 * ((pixel11 >> L_GREEN_SHIFT) & 0xff);
+            v11b = area11 * ((pixel11 >> L_BLUE_SHIFT) & 0xff);
             vinr = ving = vinb = 0;
             for (k = 1; k < dely; k++) {  /* for full src pixels */
                 for (m = 1; m < delx; m++) {
                     pixel = *(lines + k * wpls + xup + m);
-                    vinr += 256 * (pixel >> 24);
-                    ving += 256 * ((pixel >> 16) & 0xff);
-                    vinb += 256 * ((pixel >> 8) & 0xff);
+                    vinr += 256 * ((pixel >> L_RED_SHIFT) & 0xff);
+                    ving += 256 * ((pixel >> L_GREEN_SHIFT) & 0xff);
+                    vinb += 256 * ((pixel >> L_BLUE_SHIFT) & 0xff);
                 }
             }
             vmidr = vmidg = vmidb = 0;
@@ -1270,39 +1343,39 @@ l_float32  scx, scy;
             areab = 16 * ylf;
             for (k = 1; k < dely; k++) {  /* for left side */
                 pixel = *(lines + k * wpls + xup);
-                vmidr += areal * (pixel >> 24);
-                vmidg += areal * ((pixel >> 16) & 0xff);
-                vmidb += areal * ((pixel >> 8) & 0xff);
+                vmidr += areal * ((pixel >> L_RED_SHIFT) & 0xff);
+                vmidg += areal * ((pixel >> L_GREEN_SHIFT) & 0xff);
+                vmidb += areal * ((pixel >> L_BLUE_SHIFT) & 0xff);
             }
             for (k = 1; k < dely; k++) {  /* for right side */
                 pixel = *(lines + k * wpls + xlp);
-                vmidr += arear * (pixel >> 24);
-                vmidg += arear * ((pixel >> 16) & 0xff);
-                vmidb += arear * ((pixel >> 8) & 0xff);
+                vmidr += arear * ((pixel >> L_RED_SHIFT) & 0xff);
+                vmidg += arear * ((pixel >> L_GREEN_SHIFT) & 0xff);
+                vmidb += arear * ((pixel >> L_BLUE_SHIFT) & 0xff);
             }
             for (m = 1; m < delx; m++) {  /* for top side */
                 pixel = *(lines + xup + m);
-                vmidr += areat * (pixel >> 24);
-                vmidg += areat * ((pixel >> 16) & 0xff);
-                vmidb += areat * ((pixel >> 8) & 0xff);
+                vmidr += areat * ((pixel >> L_RED_SHIFT) & 0xff);
+                vmidg += areat * ((pixel >> L_GREEN_SHIFT) & 0xff);
+                vmidb += areat * ((pixel >> L_BLUE_SHIFT) & 0xff);
             }
             for (m = 1; m < delx; m++) {  /* for bottom side */
                 pixel = *(lines + dely * wpls + xup + m);
-                vmidr += areab * (pixel >> 24);
-                vmidg += areab * ((pixel >> 16) & 0xff);
-                vmidb += areab * ((pixel >> 8) & 0xff);
+                vmidr += areab * ((pixel >> L_RED_SHIFT) & 0xff);
+                vmidg += areab * ((pixel >> L_GREEN_SHIFT) & 0xff);
+                vmidb += areab * ((pixel >> L_BLUE_SHIFT) & 0xff);
             }
 
                 /* Sum all the contributions */
-            valr = (v00r + v01r + v10r + v11r + vinr + vmidr + 128) / area;
-            valg = (v00g + v01g + v10g + v11g + ving + vmidg + 128) / area;
-            valb = (v00b + v01b + v10b + v11b + vinb + vmidb + 128) / area;
+            rval = (v00r + v01r + v10r + v11r + vinr + vmidr + 128) / area;
+            gval = (v00g + v01g + v10g + v11g + ving + vmidg + 128) / area;
+            bval = (v00b + v01b + v10b + v11b + vinb + vmidb + 128) / area;
 #if  DEBUG_OVERFLOW
-            if (valr > 255) fprintf(stderr, "valr ovfl: %d\n", valr);
-            if (valg > 255) fprintf(stderr, "valg ovfl: %d\n", valg);
-            if (valb > 255) fprintf(stderr, "valb ovfl: %d\n", valb);
+            if (rval > 255) fprintf(stderr, "rval ovfl: %d\n", rval);
+            if (rval > 255) fprintf(stderr, "gval ovfl: %d\n", gval);
+            if (rval > 255) fprintf(stderr, "bval ovfl: %d\n", bval);
 #endif  /* DEBUG_OVERFLOW */
-            *(lined + j) = (valr << 24 ) | (valg << 16) | (valb << 8);
+            composeRGBPixel(rval, gval, bval, lined + j);
         }
     }
 
@@ -1427,8 +1500,6 @@ l_float32  scx, scy;
  *
  *  Note: This function is called with either 8 bpp gray or 32 bpp RGB.
  *        The result is a 2x reduced dest.
- *
- *  *** Warning: explicit assumption about RGB component ordering ***
  */
 void
 scaleAreaMapLow2(l_uint32  *datad,
@@ -1465,21 +1536,21 @@ l_uint32   pixel;
             for (j = 0; j < wd; j++) {
                     /* Average each of the color components from 4 src pixels */
                 pixel = *(lines + 2 * j);
-                rval = (pixel >> 24);
-                gval = (pixel >> 16 & 0xff);
-                bval = (pixel >> 8 & 0xff);
+                rval = (pixel >> L_RED_SHIFT) & 0xff;
+                gval = (pixel >> L_GREEN_SHIFT) & 0xff;
+                bval = (pixel >> L_BLUE_SHIFT) & 0xff;
                 pixel = *(lines + 2 * j + 1);
-                rval += (pixel >> 24);
-                gval += (pixel >> 16 & 0xff);
-                bval += (pixel >> 8 & 0xff);
+                rval += (pixel >> L_RED_SHIFT) & 0xff;
+                gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+                bval += (pixel >> L_BLUE_SHIFT) & 0xff;
                 pixel = *(lines + wpls + 2 * j);
-                rval += (pixel >> 24);
-                gval += (pixel >> 16 & 0xff);
-                bval += (pixel >> 8 & 0xff);
+                rval += (pixel >> L_RED_SHIFT) & 0xff;
+                gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+                bval += (pixel >> L_BLUE_SHIFT) & 0xff;
                 pixel = *(lines + wpls + 2 * j + 1);
-                rval += (pixel >> 24);
-                gval += (pixel >> 16 & 0xff);
-                bval += (pixel >> 8 & 0xff);
+                rval += (pixel >> L_RED_SHIFT) & 0xff;
+                gval += (pixel >> L_GREEN_SHIFT) & 0xff;
+                bval += (pixel >> L_BLUE_SHIFT) & 0xff;
                 composeRGBPixel(rval >> 2, gval >> 2, bval >> 2, &pixel);
                 *(lined + j) = pixel;
             }
@@ -2352,4 +2423,3 @@ l_float32  ratio, w1, w2;
     FREE(scol);
     return 0;
 }
-
