@@ -23,6 +23,9 @@
  *      Distance function:
  *               void   distanceFunctionLow()
  *
+ *      Seed spread:
+ *               void   seedspreadLow()
+ *
  */
 
 #include <stdio.h>
@@ -253,7 +256,7 @@ l_uint32  *lines, *linem;
  *          surface, where the highest point is 200 and the low pixels
  *          around the rim are 30.  Beyond the rim, the mask goes up a bit.
  *          Suppose the seed, which is filled, consists of a single point
- *          of height 150 below the max of the mask, with the rest 0.
+ *          of height 150, located below the max of the mask, with the rest 0.
  *          Then in the raster scan, nothing happens until the high
  *          seed point is encountered, and then this value is propagated
  *          right and down, until it hits the side of the sombrero.
@@ -263,9 +266,11 @@ l_uint32  *lines, *linem;
  *          the seed image.  Then on the anti-raster scan, the seed
  *          fills flat inside the sombrero to the upper and left,
  *          and then out from the rim as before.  The final result
- *          has a seed that is flat outside the rim (and at the rim
- *          height), and inside it fills the sombrero but only
- *          up to 150.
+ *          has a seed that is flat outside the rim, and inside
+ *          it fills the sombrero but only up to 150.  If the rim
+ *          height varies, the filled seed outside the rim will be
+ *          at the highest point on the rim, which is a saddle point
+ *          on the rim.
  */
 void
 seedfillGrayLow(l_uint32  *datas,
@@ -400,7 +405,6 @@ l_uint32  *lines, *linem;
 }
 
 
-
 /*-----------------------------------------------------------------------*
  *                   Vincent's Distance Function method                  *
  *-----------------------------------------------------------------------*/
@@ -415,33 +419,17 @@ distanceFunctionLow(l_uint32  *datad,
                     l_int32    wpld,
                     l_int32    connectivity)
 {
-l_uint16   val1, val2, val3, val4, val5, val6, val7, val8, minval, val;
+l_int32    val1, val2, val3, val4, val5, val6, val7, val8, minval, val;
 l_int32    i, j, imax, jmax;
 l_uint32  *lined;
 
     PROCNAME("distanceFunctionLow");
 
-        /* further initialize the distance image, setting
-         * a single-pixel-width perimeter of 0 pixels */
+        /* One raster scan followed by one anti-raster scan.
+         * This does not re-set the 1-boundary of pixels that
+         * were initialized to either 0 or maxval. */
     imax = h - 1;
     jmax = w - 1;
-    for (j = 0; j < wpld; j++) {
-        *(datad + j) = 0;
-        *(datad + imax * wpld + j) = 0;
-    }
-    for (i = 1; i < imax; i++) {
-        lined = datad + i * wpld;
-        if (d == 8) {
-            SET_DATA_BYTE(lined, 0, 0);
-            SET_DATA_BYTE(lined, jmax, 0);
-        }
-        else {
-            SET_DATA_TWO_BYTES(lined, 0, 0);
-            SET_DATA_TWO_BYTES(lined, jmax, 0);
-        }
-    }
-
-        /* one raster scan followed by one anti-raster scan */
     switch (connectivity)
     {
     case 4:
@@ -456,7 +444,6 @@ l_uint32  *lined;
                         minval = L_MIN(val2, val4);
                         minval = L_MIN(minval, 254);
                         SET_DATA_BYTE(lined, j, minval + 1);
-    /*                    fprintf(stderr, "Val = %d\n", GET_DATA_BYTE(lined, j)); */
                     }
                 }
             }
@@ -469,9 +456,8 @@ l_uint32  *lined;
                         val7 = GET_DATA_BYTE(lined + wpld, j);
                         val5 = GET_DATA_BYTE(lined, j + 1);
                         minval = L_MIN(val5, val7);
-                        minval = L_MIN(minval, val);
-                        minval = L_MIN(minval, 254);
-                        SET_DATA_BYTE(lined, j, minval + 1);
+                        minval = L_MIN(minval + 1, val);
+                        SET_DATA_BYTE(lined, j, minval);
                     }
                 }
             }
@@ -485,6 +471,7 @@ l_uint32  *lined;
                         val2 = GET_DATA_TWO_BYTES(lined - wpld, j);
                         val4 = GET_DATA_TWO_BYTES(lined, j - 1);
                         minval = L_MIN(val2, val4);
+                        minval = L_MIN(minval, 0xfffe);
                         SET_DATA_TWO_BYTES(lined, j, minval + 1);
                     }
                 }
@@ -498,8 +485,8 @@ l_uint32  *lined;
                         val7 = GET_DATA_TWO_BYTES(lined + wpld, j);
                         val5 = GET_DATA_TWO_BYTES(lined, j + 1);
                         minval = L_MIN(val5, val7);
-                        minval = L_MIN(minval, val);
-                        SET_DATA_TWO_BYTES(lined, j, minval + 1);
+                        minval = L_MIN(minval + 1, val);
+                        SET_DATA_TWO_BYTES(lined, j, minval);
                     }
                 }
             }
@@ -538,9 +525,8 @@ l_uint32  *lined;
                         minval = L_MIN(val8, val7);
                         minval = L_MIN(minval, val6);
                         minval = L_MIN(minval, val5);
-                        minval = L_MIN(minval, val);
-                        minval = L_MIN(minval, 254);
-                        SET_DATA_BYTE(lined, j, minval + 1);
+                        minval = L_MIN(minval + 1, val);
+                        SET_DATA_BYTE(lined, j, minval);
                     }
                 }
             }
@@ -558,6 +544,7 @@ l_uint32  *lined;
                         minval = L_MIN(val1, val2);
                         minval = L_MIN(minval, val3);
                         minval = L_MIN(minval, val4);
+                        minval = L_MIN(minval, 0xfffe);
                         SET_DATA_TWO_BYTES(lined, j, minval + 1);
                     }
                 }
@@ -575,8 +562,8 @@ l_uint32  *lined;
                         minval = L_MIN(val8, val7);
                         minval = L_MIN(minval, val6);
                         minval = L_MIN(minval, val5);
-                        minval = L_MIN(minval, val);
-                        SET_DATA_TWO_BYTES(lined, j, minval + 1);
+                        minval = L_MIN(minval + 1, val);
+                        SET_DATA_TWO_BYTES(lined, j, minval);
                     }
                 }
             }
@@ -591,3 +578,149 @@ l_uint32  *lined;
     return;
 }
 
+
+/*-----------------------------------------------------------------------*
+ *                 Seed spread (based on distance function)              *
+ *-----------------------------------------------------------------------*/
+/*!
+ *  seedspreadLow()
+ *
+ *    See pixSeedspread() for a brief description of the algorithm here.
+ */
+void
+seedspreadLow(l_uint32  *datad,
+              l_int32    w,
+              l_int32    h,
+              l_int32    wpld,
+              l_uint32  *datat,
+              l_int32    wplt,
+              l_int32    connectivity)
+{
+l_int32    val1t, val2t, val3t, val4t, val5t, val6t, val7t, val8t;
+l_int32    val1d, val2d, val3d, val4d, val5d, val6d, val7d, val8d;
+l_int32    i, j, imax, jmax, minval, valt, vald;
+l_uint32  *linet, *lined;
+
+    PROCNAME("seedspreadLow");
+
+        /* One raster scan followed by one anti-raster scan.
+         * pixt is initialized to have 0 on pixels where the
+         * input is specified in pixd, and to have 1 on all
+         * other pixels.  We only change pixels in pixt and pixd
+         * that are non-zero in pixt. */
+    imax = h - 1;
+    jmax = w - 1;
+    switch (connectivity)
+    {
+    case 4:
+            /* UL --> LR scan */
+        for (i = 1; i < h; i++) {
+            linet = datat + i * wplt;
+            lined = datad + i * wpld;
+            for (j = 1; j < jmax; j++) {
+                if ((valt = GET_DATA_TWO_BYTES(linet, j)) > 0) {
+                    val2t = GET_DATA_TWO_BYTES(linet - wplt, j);
+                    val4t = GET_DATA_TWO_BYTES(linet, j - 1);
+                    minval = L_MIN(val2t, val4t);
+                    minval = L_MIN(minval, 0xfffe);
+                    SET_DATA_TWO_BYTES(linet, j, minval + 1);
+                    if (val2t < val4t)
+                        vald = GET_DATA_BYTE(lined - wpld, j);
+                    else
+                        vald = GET_DATA_BYTE(lined, j - 1);
+                    SET_DATA_BYTE(lined, j, vald);
+                }
+            }
+        }
+
+            /* LR --> UL scan */
+        for (i = imax - 1; i > 0; i--) {
+            linet = datat + i * wplt;
+            lined = datad + i * wpld;
+            for (j = jmax - 1; j > 0; j--) {
+                if ((valt = GET_DATA_TWO_BYTES(linet, j)) > 0) {
+                    val7t = GET_DATA_TWO_BYTES(linet + wplt, j);
+                    val5t = GET_DATA_TWO_BYTES(linet, j + 1);
+                    minval = L_MIN(val5t, val7t);
+                    minval = L_MIN(minval + 1, valt);
+		    if (valt > minval) {  /* replace */
+                        SET_DATA_TWO_BYTES(linet, j, minval);
+                        if (val5t < val7t)
+                            vald = GET_DATA_BYTE(lined, j + 1);
+                        else
+                            vald = GET_DATA_BYTE(lined + wplt, j);
+                        SET_DATA_BYTE(lined, j, vald);
+                    }
+                }
+            }
+        }
+        break;
+    case 8:
+            /* UL --> LR scan */
+        for (i = 1; i < h; i++) {
+            linet = datat + i * wplt;
+            lined = datad + i * wpld;
+            for (j = 1; j < jmax; j++) {
+                if ((valt = GET_DATA_TWO_BYTES(linet, j)) > 0) {
+                    val1t = GET_DATA_TWO_BYTES(linet - wplt, j - 1);
+                    val2t = GET_DATA_TWO_BYTES(linet - wplt, j);
+                    val3t = GET_DATA_TWO_BYTES(linet - wplt, j + 1);
+                    val4t = GET_DATA_TWO_BYTES(linet, j - 1);
+                    minval = L_MIN(val1t, val2t);
+                    minval = L_MIN(minval, val3t);
+                    minval = L_MIN(minval, val4t);
+                    minval = L_MIN(minval, 0xfffe);
+                    SET_DATA_TWO_BYTES(linet, j, minval + 1);
+                    if (minval == val1t)
+                        vald = GET_DATA_BYTE(lined - wpld, j - 1);
+                    else if (minval == val2t)
+                        vald = GET_DATA_BYTE(lined - wpld, j);
+                    else if (minval == val3t)
+                        vald = GET_DATA_BYTE(lined - wpld, j + 1);
+                    else  /* minval == val4t */
+                        vald = GET_DATA_BYTE(lined, j - 1);
+                    SET_DATA_BYTE(lined, j, vald);
+                }
+            }
+        }
+
+            /* LR --> UL scan */
+        for (i = imax - 1; i > 0; i--) {
+            linet = datat + i * wplt;
+            lined = datad + i * wpld;
+            for (j = jmax - 1; j > 0; j--) {
+                if ((valt = GET_DATA_TWO_BYTES(linet, j)) > 0) {
+                    val8t = GET_DATA_TWO_BYTES(linet + wplt, j + 1);
+                    val7t = GET_DATA_TWO_BYTES(linet + wplt, j);
+                    val6t = GET_DATA_TWO_BYTES(linet + wplt, j - 1);
+                    val5t = GET_DATA_TWO_BYTES(linet, j + 1);
+                    minval = L_MIN(val8t, val7t);
+                    minval = L_MIN(minval, val6t);
+                    minval = L_MIN(minval, val5t);
+                    minval = L_MIN(minval + 1, valt);
+                    if (valt > minval) {  /* replace */
+                        SET_DATA_TWO_BYTES(linet, j, minval);
+                        if (minval == val5t + 1)
+                            vald = GET_DATA_BYTE(lined, j + 1);
+			else if (minval == val6t + 1)
+                            vald = GET_DATA_BYTE(lined + wpld, j - 1);
+			else if (minval == val7t + 1)
+                            vald = GET_DATA_BYTE(lined + wpld, j);
+			else  /* minval == val8t + 1 */
+                            vald = GET_DATA_BYTE(lined + wpld, j + 1);
+                        SET_DATA_BYTE(lined, j, vald);
+                    }
+                }
+            }
+        }
+        break;
+    default:
+        ERROR_VOID("connectivity must be 4 or 8", procName);
+        break;
+    }
+
+    return;
+}
+
+
+
