@@ -150,6 +150,7 @@ SEL       *seld, *sel;
 
     PROCNAME("pixGenerateSelWithRuns");
 
+    if (ppixe) *ppixe = NULL;
     if (!pixs)
         return (SEL *)ERROR_PTR("pixs not defined", procName, NULL);
     if (pixGetDepth(pixs) != 1)
@@ -334,6 +335,7 @@ SEL       *seld, *sel;
 
     PROCNAME("pixGenerateSelRandom");
 
+    if (ppixe) *ppixe = NULL;
     if (!pixs)
         return (SEL *)ERROR_PTR("pixs not defined", procName, NULL);
     if (pixGetDepth(pixs) != 1)
@@ -485,6 +487,7 @@ PTA     *ptah, *ptam;
 
     PROCNAME("pixGenerateSelBoundary");
 
+    if (ppixe) *ppixe = NULL;
     if (!pixs)
         return (SEL *)ERROR_PTR("pixs not defined", procName, NULL);
     if (pixGetDepth(pixs) != 1)
@@ -714,7 +717,7 @@ PTA      *pta;
     if (y2 < 0 || y2 >= h)
         return (NUMA *)ERROR_PTR("y2 not valid", procName, NULL);
 
-    if ((pta = ptaGenerateLine(x1, y1, x2, y2)) == NULL)
+    if ((pta = generatePtaLine(x1, y1, x2, y2)) == NULL)
         return (NUMA *)ERROR_PTR("pta not made", procName, NULL);
     if ((npts = ptaGetCount(pta)) == 0)
         return (NUMA *)ERROR_PTR("pta has no pts", procName, NULL);
