@@ -18,13 +18,20 @@
  *  utils.c
  *
  *       error, warning and info procs; all invoked by macros
- *           l_int32    l_errorInt()
- *           l_float32  l_errorFloat()
- *           void      *l_errorPtr()
- *           void       l_errorVoid()
+ *           l_int32    returnErrorInt()
+ *           l_float32  returnErrorFloat()
+ *           void      *returnErrorPtr()
+ *           void       returnErrorVoid()
+ *           void       l_error()
+ *           void       l_errorString()
+ *           void       l_errorInt()
+ *           void       l_errorFloat()
  *           void       l_warning()
+ *           void       l_warningString()
  *           void       l_warningInt()
+ *           void       l_warningFloat()
  *           void       l_info()
+ *           void       l_infoString()
  *           void       l_infoInt()
  *           void       l_infoInt2()
  *           void       l_infoFloat()
@@ -34,7 +41,9 @@
  *           char      *stringNew()
  *           l_int32    stringReplace()
  *           char      *stringJoin()
+ *           char      *stringReverse()
  *           char      *strtokSafe()
+ *           l_int32    stringSplitOnToken()
  *
  *       find and replace procs
  *           char      *stringRemoveChars()
@@ -49,6 +58,7 @@
  *           l_uint8   *arrayRead()
  *           l_uint8   *arrayReadStream()
  *           l_int32    nbytesInFile()
+ *           l_int32    fnbytesInFile()
  *
  *       write memory to file
  *           l_int32    arrayWrite()
@@ -91,7 +101,7 @@
  *                                                                      *
  *----------------------------------------------------------------------*/
 /*!
- *  l_errorInt()
+ *  returnErrorInt()
  *
  *      Input:  msg (error message)
  *              procname
@@ -99,9 +109,9 @@
  *      Return: ival (typically 1)
  */
 l_int32
-l_errorInt(const char  *msg, 
-           const char  *procname, 
-           l_int32      ival)
+returnErrorInt(const char  *msg, 
+               const char  *procname, 
+               l_int32      ival)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return ival;
@@ -109,7 +119,7 @@ l_errorInt(const char  *msg,
 
 
 /*!
- *  l_errorFloat()
+ *  returnErrorFloat()
  *
  *      Input:  msg (error message)
  *              procname
@@ -117,9 +127,9 @@ l_errorInt(const char  *msg,
  *      Return: fval
  */
 l_float32
-l_errorFloat(const char  *msg, 
-             const char  *procname, 
-             l_float32    fval)
+returnErrorFloat(const char  *msg, 
+                 const char  *procname, 
+                 l_float32    fval)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return fval;
@@ -127,7 +137,7 @@ l_errorFloat(const char  *msg,
 
 
 /*!
- *  l_errorPtr()
+ *  returnErrorPtr()
  *
  *      Input:  msg (error message)
  *              procname
@@ -135,9 +145,9 @@ l_errorFloat(const char  *msg,
  *      Return: pval (typically null)
  */
 void *
-l_errorPtr(const char  *msg,
-           const char  *procname, 
-           void        *pval)
+returnErrorPtr(const char  *msg,
+               const char  *procname, 
+               void        *pval)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return pval;
@@ -145,20 +155,138 @@ l_errorPtr(const char  *msg,
 
 
 /*!
- *  l_errorVoid()
+ *  returnErrorVoid()
  *
  *      Input: msg (error message)
  *             procname
  */
 void
-l_errorVoid(const char  *msg, 
-            const char  *procname)
+returnErrorVoid(const char  *msg, 
+                const char  *procname)
 {
     fprintf(stderr, "Error in %s: %s\n", procname, msg);
     return;
 }
 
 
+/*!
+ *  l_error()
+ *
+ *      Input: msg (error message)
+ *             procname
+ */
+void
+l_error(const char  *msg,
+        const char  *procname)
+{
+    fprintf(stderr, "Error in %s: %s\n", procname, msg);
+    return;
+}
+
+
+/*!
+ *  l_errorString()
+ *
+ *      Input: msg (error message; must include '%s')
+ *             procname
+ *             str (embedded in error message via %s)
+ */
+void
+l_errorString(const char  *msg,
+              const char  *procname,
+              const char  *str)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname || !str) {
+        ERROR_VOID("msg, procname or str not defined in l_errorString()",
+                   procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_errorString()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Error in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, str);
+
+    FREE(charbuf);
+    return;
+}
+
+
+/*!
+ *  l_errorInt()
+ *
+ *      Input: msg (error message; must include '%d')
+ *             procname
+ *             ival (embedded in error message via %d)
+ */
+void
+l_errorInt(const char  *msg,
+           const char  *procname,
+           l_int32      ival)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname) {
+        ERROR_VOID("msg or procname not defined in l_errorInt()", procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_errorInt()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Error in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, ival);
+
+    FREE(charbuf);
+    return;
+}
+
+
+/*!
+ *  l_errorFloat()
+ *
+ *      Input: msg (error message; must include '%f')
+ *             procname
+ *             fval (embedded in error message via %f)
+ */
+void
+l_errorFloat(const char  *msg,
+             const char  *procname,
+             l_float32    fval)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname) {
+        ERROR_VOID("msg or procname not defined in l_errorFloat()", procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_errorFloat()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Error in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, fval);
+
+    FREE(charbuf);
+    return;
+}
+
+
 /*!
  *  l_warning()
  *
@@ -174,16 +302,51 @@ l_warning(const char  *msg,
 }
 
 
+/*!
+ *  l_warningString()
+ *
+ *      Input: msg (warning message; must include '%s')
+ *             procname
+ *             str (embedded in warning message via %s)
+ */
+void
+l_warningString(const char  *msg,
+                const char  *procname,
+                const char  *str)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname || !str) {
+        ERROR_VOID("msg, procname or str not defined in l_warningString()",
+                   procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_warningString()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Warning in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, str);
+
+    FREE(charbuf);
+    return;
+}
+
+
 /*!
  *  l_warningInt()
  *
- *      Input: msg (warning message)
+ *      Input: msg (warning message; must include '%d')
  *             procname
  *             ival (embedded in warning message via %d)
  */
 void
 l_warningInt(const char  *msg,
-             const char  *procname, 
+             const char  *procname,
              l_int32      ival)
 {
 l_int32  bufsize;
@@ -208,6 +371,40 @@ char    *charbuf;
 }
 
 
+/*!
+ *  l_warningFloat()
+ *
+ *      Input: msg (warning message; must include '%f')
+ *             procname
+ *             fval (embedded in warning message via %f)
+ */
+void
+l_warningFloat(const char  *msg,
+               const char  *procname,
+               l_float32    fval)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname) {
+        ERROR_VOID("msg or procname not defined in l_warningFloat()", procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_warningFloat()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Warning in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, fval);
+
+    FREE(charbuf);
+    return;
+}
+
+
 /*!
  *  l_info()
  *
@@ -223,10 +420,45 @@ l_info(const char  *msg,
 }
 
 
+/*!
+ *  l_infoString()
+ *
+ *      Input: msg (info message; must include '%s')
+ *             procname
+ *             str (embedded in warning message via %s)
+ */
+void
+l_infoString(const char  *msg,
+             const char  *procname,
+             const char  *str)
+{
+l_int32  bufsize;
+char    *charbuf;
+
+    if (!msg || !procname || !str) {
+        ERROR_VOID("msg, procname or str not defined in l_infoString()",
+                   procname);
+        return;
+    }
+
+    bufsize = strlen(msg) + strlen(procname) + 128;
+    if ((charbuf = (char *)CALLOC(bufsize, sizeof(char))) == NULL) {
+        ERROR_VOID("charbuf not made in l_infoString()", procname);
+        return;
+    }
+
+    sprintf(charbuf, "Info in %s: %s\n", procname, msg);
+    fprintf(stderr, charbuf, str);
+
+    FREE(charbuf);
+    return;
+}
+
+
 /*!
  *  l_infoInt()
  *
- *      Input: msg (info message)
+ *      Input: msg (info message; must include '%d')
  *             procname
  *             ival (embedded in info message via %d)
  */
@@ -260,7 +492,7 @@ char    *charbuf;
 /*!
  *  l_infoInt2()
  *
- *      Input: msg (info message)
+ *      Input: msg (info message; must include two '%d')
  *             procname
  *             ival1, ival2 (two args, embedded in info message via %d)
  */
@@ -295,7 +527,7 @@ char    *charbuf;
 /*!
  *  l_infoFloat()
  *
- *      Input: msg (info message)
+ *      Input: msg (info message; must include '%f')
  *             procname
  *             fval (embedded in info message via %f)
  */
@@ -318,7 +550,7 @@ char    *charbuf;
         return;
     }
 
-    sprintf(charbuf, "from %s: %s\n", procname, msg);
+    sprintf(charbuf, "Info in %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, fval);
 
     FREE(charbuf);
@@ -329,7 +561,7 @@ char    *charbuf;
 /*!
  *  l_infoFloat2()
  *
- *      Input: msg (info message)
+ *      Input: msg (info message; must include two '%f')
  *             procname
  *             fval1, fval2 (two args, embedded in info message via %f)
  */
@@ -353,7 +585,7 @@ char    *charbuf;
         return;
     }
 
-    sprintf(charbuf, "from %s: %s\n", procname, msg);
+    sprintf(charbuf, "Info in %s: %s\n", procname, msg);
     fprintf(stderr, charbuf, fval1, fval2);
 
     FREE(charbuf);
@@ -467,15 +699,42 @@ l_int32  srclen1, srclen2, destlen;
 }
 
 
+/*!
+ *  stringReverse()
+ *
+ *      Input:  src (string)
+ *      Return: dest (newly-allocated reversed string)
+ */
+char *
+stringReverse(const char  *src)
+{
+char    *dest;
+l_int32  i, len;
+
+    PROCNAME("stringReverse");
+
+    if (!src)
+        return (char *)ERROR_PTR("src not defined", procName, NULL);
+    len = strlen(src);
+    if ((dest = (char *)CALLOC(len + 1, sizeof(char))) == NULL)
+        return (char *)ERROR_PTR("calloc fail for dest", procName, NULL);
+    for (i = 0; i < len; i++)
+        dest[i] = src[len - 1 - i];
+
+    return dest;
+}
+
+
 /*!
  *  strtokSafe()
  *
- *      Input:  cstr (input string to be parsed;
+ *      Input:  cstr (input string to be sequentially parsed;
  *                    use NULL after the first call)
  *              seps (a string of character separators)
  *              &saveptr (<return> ptr to the next char after
  *                        the last encountered separator)
- *      Return: substr (a new string from the previous saveptr to the first
+ *      Return: substr (a new string that is copied from the previous
+ *                      saveptr up to but not including the next
  *                      separator character), or NULL if end of cstr.
  *
  *  Notes:
@@ -563,6 +822,56 @@ l_int32  istart, i, j, nchars;
 }
 
 
+/*!
+ *  stringSplitOnToken()
+ *
+ *      Input:  cstr (input string to be split; not altered)
+ *              seps (a string of character separators)
+ *              &head (<return> ptr to copy of the input string, up to
+ *                     the first separator token encountered)
+ *              &tail (<return> ptr to copy of the part of the input string
+ *                     starting with the first non-separator character
+ *                     that occurs after the first separator is found)
+ *      Return: 0 if OK, 1 on error
+ *
+ *  Notes:
+ *      (1) The input string is not altered; all split parts are new strings.
+ *      (2) The split occurs around the first consecutive sequence of
+ *          tokens encountered.
+ *      (3) The head goes from the beginning of the string up to
+ *          but not including the first token found.
+ *      (4) The tail contains the second part of the string, starting
+ *          with the first char in that part that is NOT a token.
+ *      (5) If no separator token is found, 'head' contains a copy
+ *          of the input string and 'tail' is null.
+ */
+l_int32
+stringSplitOnToken(char        *cstr,
+                   const char  *seps,
+                   char       **phead,
+                   char       **ptail)
+{
+char  *saveptr;
+
+    PROCNAME("stringSplitOnToken");
+
+    if (!phead)
+        return ERROR_INT("&head not defined", procName, 1);
+    if (!ptail)
+        return ERROR_INT("&tail not defined", procName, 1);
+    *phead = *ptail = NULL;
+    if (!cstr)
+        return ERROR_INT("cstr not defined", procName, 1);
+    if (!seps)
+        return ERROR_INT("seps not defined", procName, 1);
+
+    *phead = strtokSafe(cstr, seps, &saveptr);
+    if (saveptr)
+        *ptail = stringNew(saveptr);
+    return 0;
+}
+
+
 /*--------------------------------------------------------------------*
  *                       Find and replace procs                       *
  *--------------------------------------------------------------------*/
@@ -649,7 +958,7 @@ l_int32  nsrc, nsub1, nsub2, len, npre, loc;
         loc = *ploc;
     else
         loc = 0;
-    if ((ptr = strstr(src + loc, sub1)) == NULL) {
+    if ((ptr = (char *)strstr(src + loc, sub1)) == NULL) {
         return NULL;
     }
 
@@ -788,7 +1097,6 @@ l_int32  i, j, found, lastpos;
 }
 
 
-
 /*--------------------------------------------------------------------*
  *                             Safe realloc                           *
  *--------------------------------------------------------------------*/
@@ -920,7 +1228,7 @@ l_uint8  *data;
     if (!pnbytes)
         return (l_uint8 *)ERROR_PTR("ptr to nbytes not defined", procName, NULL);
 
-    *pnbytes = nbytesInFile(fp);
+    *pnbytes = fnbytesInFile(fp);
 
     if ((data = (l_uint8 *)CALLOC(1, *pnbytes + 1)) == NULL)
         return (l_uint8 *)ERROR_PTR("CALLOC fail for data", procName, NULL);
@@ -933,15 +1241,38 @@ l_uint8  *data;
 /*!
  *  nbytesInFile()
  *
+ *      Input:  filename
+ *      Return: nbytes in file; 0 on error
+ */
+l_int32
+nbytesInFile(const char  *filename)
+{
+l_int32  nbytes;
+FILE    *fp;
+
+    PROCNAME("nbytesInFile");
+
+    if (!filename)
+        return ERROR_INT("filename not defined", procName, 0);
+    fp = fopen(filename, "r");
+    nbytes = fnbytesInFile(fp);
+    fclose(fp);
+    return nbytes;
+}
+
+
+/*!
+ *  fnbytesInFile()
+ *
  *      Input:  file stream
  *      Return: nbytes in file; 0 on error
  */
 l_int32
-nbytesInFile(FILE  *fp)
+fnbytesInFile(FILE  *fp)
 {
 l_int32  nbytes, pos;
 
-    PROCNAME("nbytesInFile");
+    PROCNAME("fnbytesInFile");
 
     if (!fp)
         return ERROR_INT("stream not open", procName, 0);
@@ -950,7 +1281,6 @@ l_int32  nbytes, pos;
     fseek(fp, 0, SEEK_END);   /* EOF */
     nbytes = ftell(fp);
     fseek(fp, 0, pos);        /* back to initial position */
-
     return nbytes;
 }
 
@@ -1147,10 +1477,12 @@ char  *cpathname, *lastslash;
 
     PROCNAME("splitPathAtDirectory");
 
-    if (!pathname)
-        return ERROR_INT("pathname not defined", procName, 1);
     if (!pdir && !ptail)
         return ERROR_INT("null input for both strings", procName, 1);
+    if (pdir) *pdir = NULL;
+    if (ptail) *ptail = NULL;
+    if (!pathname)
+        return ERROR_INT("pathname not defined", procName, 1);
 
     cpathname = stringNew(pathname);
     if ((lastslash = strrchr(cpathname, '/'))) {
@@ -1208,10 +1540,12 @@ char   empty[4] = "";
 
     PROCNAME("splitPathExtension");
 
-    if (!pathname)
-        return ERROR_INT("pathname not defined", procName, 1);
     if (!pbasename && !pextension)
         return ERROR_INT("null input for both strings", procName, 1);
+    if (pbasename) *pbasename = NULL;
+    if (pextension) *pextension = NULL;
+    if (!pathname)
+        return ERROR_INT("pathname not defined", procName, 1);
 
         /* Split out the directory first */
     splitPathAtDirectory(pathname, &dir, &tail);
@@ -1351,4 +1685,3 @@ FILETIME  start, stop, kernel, user;
 }
 
 #endif
-
